/**
 * Title :	Property view
 * Description: Property editor
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package bsys.propertyview;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

/**
 * Color editor
 * 
 * @author Bernard Bou
 */
class ColorEditor extends AbstractCellEditor implements TableCellEditor
{
	private static final long serialVersionUID = 1L;

	/**
	 * Color being edited
	 */
	private Color theCurrentColor;

	/**
	 * Button (editor component)
	 */
	private final JButton theButton;

	/**
	 * Color chooser
	 */
	private final JColorChooser theColorChooser;

	/**
	 * Wrapping color dialog
	 */
	private final JDialog theColorDialog;

	/**
	 * Constructor
	 */
	public ColorEditor()
	{
		this.theButton = new JButton();
		this.theButton.setBorderPainted(false);
		this.theButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				ColorEditor.this.theColorChooser.setColor(ColorEditor.this.theCurrentColor);
				ColorEditor.this.theColorDialog.setVisible(true);

				// dialog returns control
				fireEditingStopped();
			}
		});

		// set up the dialog that the button brings up.
		this.theColorChooser = new JColorChooser();
		this.theColorDialog = JColorChooser.createDialog(this.theButton, Messages.getString("ColorEditor.0"), true, // modal //$NON-NLS-1$
				this.theColorChooser,
				// ok button handler
				new ActionListener()
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void actionPerformed(final ActionEvent e)
					{
						ColorEditor.this.theCurrentColor = ColorEditor.this.theColorChooser.getColor();
					}
				},
				// cancel button handler
				new ActionListener()
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void actionPerformed(final ActionEvent e)
					{
						ColorEditor.this.theCurrentColor = null;
					}
				});
	}

	// implement the one method defined by TableCellEditor.
	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
	public Component getTableCellEditorComponent(final JTable table, final Object thisValue, final boolean isSelected, final int row, final int column)
	{
		this.theCurrentColor = (Color) thisValue;
		this.theButton.setBackground(this.theCurrentColor);
		return this.theButton;
	}

	// implement the one CellEditor method that AbstractCellEditor doesn't.
	/*
	 * (non-Javadoc)
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	@Override
	public Object getCellEditorValue()
	{
		return this.theCurrentColor;
	}
}
