/**
 * Title :	Property view
 * Description: Property editor
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package bsys.propertyview;

import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableCellEditor;

/**
 * File editor
 * 
 * @author Bernard Bou
 */
class FileEditor extends AbstractCellEditor implements TableCellEditor
{
	private static final long serialVersionUID = 1L;

	/**
	 * File chooser
	 */
	private JFileChooser theFileChooser;

	/**
	 * Button component
	 */
	private JButton theButton;

	/**
	 * Current directory
	 */
	private File theCurrentDirectory;

	/**
	 * Constructor
	 */
	public FileEditor()
	{
		this.theButton = new JButton();
		this.theButton.setBorderPainted(false);
		this.theButton.setHorizontalAlignment(SwingConstants.LEFT);
		this.theButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				FileEditor.this.theFileChooser.setCurrentDirectory(FileEditor.this.theCurrentDirectory);
				FileEditor.this.theFileChooser.showOpenDialog(null);

				// dialog returns control
				fireEditingStopped();
			}
		});

		// file chooser
		final JButton thisDefaultFileButton = new JButton(Messages.getString("FileEditor.0")); //$NON-NLS-1$
		thisDefaultFileButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				FileEditor.this.theFileChooser.setSelectedFile(null);
				Container thisContainer = FileEditor.this.theFileChooser.getParent();
				while (thisContainer.getParent() != null)
				{
					thisContainer = thisContainer.getParent();
				}
				thisContainer.setVisible(false);
			}
		});
		this.theFileChooser = new JFileChooser();
		this.theFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		this.theFileChooser.setFileFilter(new FileFilter()
		{
			@Override
			public boolean accept(final File thisFile)
			{
				return thisFile.getName().toLowerCase().endsWith(".gif") || thisFile.getName().toLowerCase().endsWith(".jpg") || thisFile.getName().toLowerCase().endsWith(".png") || thisFile.isDirectory(); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}

			@Override
			public String getDescription()
			{
				return Messages.getString("FileEditor.1"); //$NON-NLS-1$
			}
		});
		this.theFileChooser.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				if (e.getActionCommand().equalsIgnoreCase(JFileChooser.APPROVE_SELECTION))
				{
					fireEditingStopped();
				}
				else
				{
					fireEditingCanceled();
				}
			}
		});
		this.theFileChooser.setAccessory(thisDefaultFileButton);
	}

	/**
	 * Set current directory
	 * 
	 * @param thisCurrentDirectory
	 *        current directory
	 */
	public void setDirectory(final File thisCurrentDirectory)
	{
		this.theCurrentDirectory = thisCurrentDirectory;
	}

	// I N T E R F A C E

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
	public Component getTableCellEditorComponent(final JTable thisTable, final Object thisValue, final boolean thisIsSelected, final int thisRow, final int thisColumn)
	{
		this.theButton.setText((String) thisValue);
		return this.theButton;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.CellEditor#getCellEditorValue()
	 */
	@Override
	public Object getCellEditorValue()
	{
		final File thisSelection = this.theFileChooser.getSelectedFile();
		return thisSelection == null ? null : thisSelection.getName();
	}
}
