/**
 * Title :	Property view
 * Description: Property editor
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package bsys.propertyview;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;

import bsys.propertyview.PropertyView.Attribute;
import bsys.propertyview.PropertyView.AttributeType;

class Renderer extends DefaultTableCellRenderer
{
	private static final long serialVersionUID = 1L;

	/**
	 * View
	 */
	private final PropertyView thePropertyView;

	/**
	 * Color panel
	 */
	private JPanel theColorPanel = new JPanel();

	/**
	 * Alternate row column
	 */
	Color theAltRowColor = new Color(0xf0f0f0);

	/**
	 * The bold font
	 */
	private final Font theBoldFont = new Font("Dialog", Font.BOLD, 12); //$NON-NLS-1$

	/**
	 * The italic font
	 */
	private final Font theItalicFont = new Font("Dialog", Font.ITALIC, 12); //$NON-NLS-1$

	// icons

	/**
	 * Mandatory attribute icon
	 */
	private final ImageIcon theMandatoryIcon = new ImageIcon(Renderer.class.getResource("images/mandatory.png")); //$NON-NLS-1$

	/**
	 * Optional attribute icon
	 */
	private final ImageIcon theOptionalIcon = new ImageIcon(Renderer.class.getResource("images/optional.png")); //$NON-NLS-1$

	/**
	 * Attribute icon
	 */
	private final ImageIcon theAttributeIcon = new ImageIcon(Renderer.class.getResource("images/attribute.png")); //$NON-NLS-1$

	/**
	 * Id icon
	 */
	private final ImageIcon theIdIcon = new ImageIcon(Renderer.class.getResource("images/id.png")); //$NON-NLS-1$

	/**
	 * Id reference icon
	 */
	private final ImageIcon theRefIdcon = new ImageIcon(Renderer.class.getResource("images/node.png")); //$NON-NLS-1$

	/**
	 * Text icon
	 */
	private final ImageIcon theTextIcon = new ImageIcon(Renderer.class.getResource("images/text.png")); //$NON-NLS-1$

	/**
	 * Long text icon
	 */
	private final ImageIcon theLongTextIcon = new ImageIcon(Renderer.class.getResource("images/longtext.png")); //$NON-NLS-1$

	/**
	 * Image icon
	 */
	private final ImageIcon theImageIcon = new ImageIcon(Renderer.class.getResource("images/image.png")); //$NON-NLS-1$

	/**
	 * Link icon
	 */
	private final ImageIcon theLinkIcon = new ImageIcon(Renderer.class.getResource("images/link.png")); //$NON-NLS-1$

	/**
	 * Color icon
	 */
	private final ImageIcon theColorIcon = new ImageIcon(Renderer.class.getResource("images/color.png")); //$NON-NLS-1$

	/**
	 * Stroke icon
	 */
	private final ImageIcon theStrokeIcon = new ImageIcon(Renderer.class.getResource("images/stroke.png")); //$NON-NLS-1$

	/**
	 * Terminator icon
	 */
	private final ImageIcon theTerminatorIcon = new ImageIcon(Renderer.class.getResource("images/terminator.png")); //$NON-NLS-1$

	/**
	 * Font icon
	 */
	private final ImageIcon theFontIcon = new ImageIcon(Renderer.class.getResource("images/font.png")); //$NON-NLS-1$

	/**
	 * Font size icon
	 */
	private final ImageIcon theFontSizeIcon = new ImageIcon(Renderer.class.getResource("images/fontsize.png")); //$NON-NLS-1$

	/**
	 * Boolean icon
	 */
	private final ImageIcon theBooleanIcon = new ImageIcon(Renderer.class.getResource("images/boolean.png")); //$NON-NLS-1$

	/**
	 * Constructor
	 * 
	 * @param thisPropertyView
	 *        property view
	 */
	public Renderer(final PropertyView thisPropertyView)
	{
		this.thePropertyView = thisPropertyView;
		this.theColorPanel = new JPanel();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	@Override
	public Component getTableCellRendererComponent(final JTable table, final Object thisValue, final boolean isSelected, final boolean hasFocus, final int thisRow, final int thisColumn)
	{
		setIcon(null);
		setFont(null);
		setForeground(Color.BLACK);
		setBackground(thisRow % 2 == 0 ? Color.WHITE : this.theAltRowColor);

		switch (thisColumn)
		{
		case PropertyView.VALUE:
		{
			final Attribute thisAttribute = (Attribute) thisValue;
			final Object thisAttributeValue = thisAttribute.theValue;
			if (thisAttributeValue == null)
			{
				setText(PropertyView.theDefaultString);
				return this;
			}

			final AttributeType thisType = thisAttribute.theDescriptor.theType;
			switch (thisType)
			{
			case COLOR:
				this.theColorPanel.setBackground((Color) thisAttributeValue);
				return this.theColorPanel;

			case LABEL:
				setFont(this.theBoldFont);
				setText(thisAttributeValue.toString());
				setBackground(Color.DARK_GRAY);
				setForeground(Color.WHITE);
				return this;

			case TEXT:
			case LONGTEXT:
			case BOOLEAN:
			case DOUBLE:
			case INTEGER:
				setText(thisAttributeValue.toString());
				return this;

			case ID:
				setText((String) thisAttributeValue);
				return this;

			case REFID:
				setText((String) thisAttributeValue);
				setIcon(this.theRefIdcon);
				return this;

			case LINK:
				setText(PropertyView.decode((String) thisAttributeValue));
				setForeground(Color.BLUE);
				setFont(this.theItalicFont);
				setIcon(this.theLinkIcon);
				return this;

			case FONTFACE:
				setText((String) thisAttributeValue);
				setFont(new Font((String) thisAttributeValue, Font.PLAIN, 18));
				return this;

			case IMAGE:
				final String thisImageFile = (String) thisAttributeValue;
				setIcon(makeIcon(thisImageFile));
				setText(thisImageFile);
				return this;

			case FONTSIZE:
				setIcon(this.theFontSizeIcon);
				setText(thisAttributeValue.toString());
				return this;

			case STROKE:
				setIcon(PropertyView.theStrokeIcons.get(thisAttributeValue));
				setText((String) thisAttributeValue);
				setForeground(Color.BLUE);
				return this;

			case TERMINATOR:
				setIcon(PropertyView.theTerminatorIcons.get(thisAttributeValue));
				setText((String) thisAttributeValue);
				setForeground(Color.BLUE);
				return this;

			default:
				System.err.println("Renderer doesn't handle this class: " + thisAttribute.theDescriptor.theName + " type:" + thisAttribute.theDescriptor.theType); //$NON-NLS-1$ //$NON-NLS-2$
				return super.getTableCellRendererComponent(table, thisValue, isSelected, hasFocus, thisRow, thisColumn);
			}
		}

		case PropertyView.STATUS:
		{
			final Boolean thisBooleanValue = (Boolean) thisValue;
			setIcon(thisBooleanValue.booleanValue() ? this.theMandatoryIcon : this.theOptionalIcon);
			setText(null);
			setHorizontalAlignment(SwingConstants.CENTER);
			return this;
		}

		case PropertyView.TYPE:
		{
			final AttributeType thisPropertyTypeValue = (AttributeType) thisValue;
			switch (thisPropertyTypeValue)
			{
			case TEXT:
				setIcon(this.theTextIcon);
				break;

			case LONGTEXT:
				setIcon(this.theLongTextIcon);
				break;

			case IMAGE:
				setIcon(this.theImageIcon);
				break;

			case LINK:
				setIcon(this.theLinkIcon);
				break;

			case COLOR:
				setIcon(this.theColorIcon);
				break;

			case BOOLEAN:
				setIcon(this.theBooleanIcon);
				break;

			case ID:
				setIcon(this.theIdIcon);
				break;

			case REFID:
				setIcon(this.theRefIdcon);
				break;

			case STROKE:
				setIcon(this.theStrokeIcon);
				break;

			case TERMINATOR:
				setIcon(this.theTerminatorIcon);
				break;

			case FONTFACE:
				setIcon(this.theFontIcon);
				break;

			case FONTSIZE:
				setIcon(this.theFontSizeIcon);
				break;

			default:
				setIcon(this.theAttributeIcon);
				break;
			}
			setText(null);
			setHorizontalAlignment(SwingConstants.CENTER);
			return this;
		}

		case PropertyView.NAME:
		{
			final String thisName = (String) thisValue;
			setText(thisName);
			return this;
		}
		}
		return super.getTableCellRendererComponent(table, thisValue, isSelected, hasFocus, thisRow, thisColumn);
	}

	/**
	 * Make icon
	 * 
	 * @param thisImageFile
	 *        image file
	 * @return icon
	 */
	private Icon makeIcon(final String thisImageFile)
	{
		try
		{
			final URL thisUrl = new URL(this.thePropertyView.getImageRepository(), thisImageFile);
			return new ImageIcon(thisUrl);
		}
		catch (final MalformedURLException e)
		{
			return null;
		}
	}
}