/**
 * Title :	Property view
 * Description: Property editor
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package bsys.propertyview;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import treebolic.commons.Utils;

/**
 * Text theDialog
 * 
 * @author Bernard Bou
 */
public class TextDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Ok result
	 */
	private boolean ok;

	// C O M P O N E N T S

	/**
	 * Action on ok
	 */
	private final ActionListener theOkAction;

	/**
	 * Action on cancel
	 */
	private final ActionListener theCancelAction;

	/**
	 * Text area
	 */
	private JTextArea theTextArea;

	/**
	 * Constructor
	 * 
	 * @param thisOkAction
	 *        ok action
	 * @param thisCancelAction
	 *        cancel action
	 */
	public TextDialog(final ActionListener thisOkAction, final ActionListener thisCancelAction)
	{
		super();
		this.theOkAction = thisOkAction;
		this.theCancelAction = thisCancelAction;

		initialize();
	}

	/**
	 * Initialize
	 * 
	 * @throws Exception
	 */
	protected void initialize()
	{
		setTitle(Messages.getString("TextDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// text
		this.theTextArea = new JTextArea();
		this.theTextArea.setLineWrap(true);
		this.theTextArea.setToolTipText(Messages.getString("TextDialog.1")); //$NON-NLS-1$
		final JScrollPane thisScrollPane = new JScrollPane(this.theTextArea);
		thisScrollPane.setPreferredSize(new Dimension(300, 100));

		// buttons
		final JButton thisOKButton = new JButton(Messages.getString("TextDialog.2")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("TextDialog.3")); //$NON-NLS-1$

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisOKButton);

		// event handling
		thisOKButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent thisEvent)
			{
				TextDialog.this.ok = true;
				TextDialog.this.theOkAction.actionPerformed(thisEvent);
				setVisible(false);
			}
		});
		thisCancelButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent thisEvent)
			{
				TextDialog.this.theCancelAction.actionPerformed(thisEvent);
				setVisible(false);
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BorderLayout());
		thisPanel.add(thisScrollPane, BorderLayout.CENTER);
		thisPanel.add(thisButtonPanel, BorderLayout.SOUTH);

		setContentPane(thisPanel);
	}

	/**
	 * Set text
	 * 
	 * @param thisText
	 *        text
	 */
	public void setText(final String thisText)
	{
		this.theTextArea.setText(thisText);
		this.theTextArea.setCaretPosition(0);
	}

	/**
	 * Get text
	 * 
	 * @return text
	 */
	public String getText()
	{
		if (this.ok)
			return this.theTextArea.getText();
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			this.ok = false;

			pack();
			Utils.center(this);
		}
		super.setVisible(thisFlag);
	}
}
