/**
 * Title : Treebolic application
 * Description: Treebolic application
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic;

import java.awt.Component;
import java.awt.Image;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JFrame;

/**
 * Application
 * 
 * @author Bernard Bou
 */
public class Application extends JFrame implements IContext
{
	private static final long serialVersionUID = 1L;

	/**
	 * Arguments
	 */
	private final String[] theArgs;

	/**
	 * Constructor
	 * 
	 * @param theseArgs
	 *        arguments
	 */
	public Application(final String[] theseArgs)
	{
		// System.out.println("CLASSPATH=<" + System.getProperty("java.class.path", ".") + ">");
		this.theArgs = theseArgs;
		setTitle("Treebolic");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		final IWidget thisWidget = new Widget(this);
		thisWidget.init();
		getContentPane().add((Component) thisWidget);

		pack();
		setVisible(true);
	}

	// C O N T E X T

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getBase()
	 */
	@Override
	public URL getBase()
	{
		// base parameter
		final String thisBase = getParameter("base");
		String thisURLString = thisBase != null ? thisBase : System.getProperty("user.dir");

		// tail
		if (!thisURLString.endsWith("/"))
		{
			thisURLString += "/";
		}

		// make
		try
		{
			return new URL(thisURLString);
		}
		catch (final MalformedURLException e)
		{
			// make from folder
			try
			{
				final File thisFolder = new File(thisURLString);
				return thisFolder.toURI().toURL();
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getHtmlDocumentBase()
	 */
	@Override
	public URL getHtmlDocumentBase()
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getImagesBase()
	 */
	@Override
	public URL getImagesBase()
	{
		URL thisBase = null;
		String thisImageBase = getParameter("images");
		if (thisImageBase != null && !thisImageBase.isEmpty())
		{
			// 1-use 'images' parameter as full-fledged url
			try
			{
				return new URL(thisImageBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			if (!thisImageBase.endsWith("/"))
			{
				thisImageBase += "/";
			}

			// 2-use 'images' parameter as relative to base
			thisBase = getBase();
			try
			{
				return new URL(thisBase, thisImageBase);
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}

		if (thisBase == null)
		{
			thisBase = getBase();
		}

		// 3-use set 'images/' path as relative to base
		try
		{
			return new URL(thisBase, "images/");
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getImage(java.net.URL)
	 */
	@Override
	public Image getImage(final URL thisURL)
	{
		return java.awt.Toolkit.getDefaultToolkit().getImage(thisURL);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getParameter(java.lang.String)
	 */
	@Override
	public String getParameter(final String thisName)
	{
		// param1=<val> param2=<"val with spaces"> ...
		if (this.theArgs == null)
			return null;
		for (final String thisParam : this.theArgs)
			if (thisParam.startsWith(thisName + "="))
			{
				String thisValue = thisParam.substring(thisName.length() + 1);
				if (thisValue.startsWith("\""))
				{
					thisValue = thisValue.substring(1);
					if (thisValue.endsWith("\""))
					{
						thisValue = thisValue.substring(0, thisValue.length() - 1);
					}
				}
				return thisValue;
			}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#linkTo(java.lang.String)
	 */
	@Override
	public boolean linkTo(final String thisLinkUrl, final String thisLinkTarget)
	{
		System.out.println("Link to: " + thisLinkUrl);
		return false;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#showStatus(java.lang.String)
	 */
	@Override
	public void showStatus(final String thisString)
	{
		System.out.println("Status: " + thisString);
	}

	/**
	 * Main
	 * 
	 * @param theseArgs
	 *        arguments
	 */
	public static void main(final String[] theseArgs)
	{
		new Application(theseArgs);
	}
}
