/**
 * Title : Treebolic applet
 * Description: Treebolic applet
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.applet;

import java.applet.AppletContext;
import java.awt.Component;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JApplet;
import javax.swing.UIManager;

import treebolic.IContext;
import treebolic.IWidget;
import treebolic.Widget;

/**
 * Applet
 * 
 * @author Bernard Bou
 */
public class Treebolic extends JApplet implements IContext
{
	private static final long serialVersionUID = 1L;

	/**
	 * Component widget
	 */
	private final IWidget theWidget;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public Treebolic()
	{
		UIManager.put("swing.boldMetal", false); //$NON-NLS-1$
		this.theWidget = new Widget(this);
		add((Component) this.theWidget);
	}

	/*
	 * (non-Javadoc)
	 * @see java.applet.Applet#init()
	 */
	@Override
	public void init()
	{
		this.theWidget.init();
	}

	// V E R S I O N (for Javascript)

	/**
	 * Get version (wrapper)
	 */
	public String getVersion()
	{
		return this.theWidget.getVersion();
	}

	// S E A R C H (for Javascript)

	/**
	 * Match node against string (wrapper)
	 * 
	 * @param thisTargetString
	 *        string to search for
	 * @param thisScopeString
	 *        scope ("label", "content", "link", "id")
	 * @param thisModeString
	 *        mode ("equals", "startswith", "includes")
	 * @return node id
	 */
	public String match(final String thisTargetString, final String thisScopeString, final String thisModeString)
	{
		final String thisResult = this.theWidget.match(thisTargetString, thisScopeString, thisModeString);
		return thisResult;
	}

	// F O C U S (for Javascript)

	/**
	 * Focus
	 * 
	 * @param thisNodeId
	 *        (wrapper) node id to get focus
	 */
	public void focus(final String thisNodeId)
	{
		this.theWidget.focus(thisNodeId);
	}

	// L I N K (for Javascript)

	/**
	 * Link to Url (wrapper)
	 * 
	 * @param thisUrlString
	 *        link url string
	 * @param thisUrlTarget
	 *        link target string
	 */
	public void link(final String thisUrlString, final String thisUrlTarget)
	{
		this.theWidget.link(thisUrlString, thisUrlTarget);
	}

	// P A R A M E T E R S

	/*
	 * (non-Javadoc)
	 * @see java.applet.Applet#getAppletInfo()
	 */
	@Override
	public String getAppletInfo()
	{
		return "Title: Treebolic\nAuthor: Bernard Bou";
	}

	/*
	 * (non-Javadoc)
	 * @see java.applet.Applet#getParameterInfo()
	 */
	@Override
	public String[][] getParameterInfo()
	{
		final String[][] theinfo = { { "doc", "string", "data source" }, { "xml", "string", "data source" }, { "base", "string", "data base" }, { "images", "string", "image base" }, { "relativeto", "string", "images base relative to (docbase|base)" } };
		return theinfo;
	}

	// C O N T E X T

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#linkTo(java.lang.String)
	 */
	@Override
	public boolean linkTo(final String thisHref, final String thisTarget)
	{
		System.out.println("Applet linking to " + thisHref + " - " + thisTarget);
		try
		{
			// make URL
			URL thisURL;
			try
			{
				thisURL = new URL(thisHref);
			}
			catch (final MalformedURLException e)
			{
				thisURL = new URL(getCodeBase(), thisHref);
			}

			// jump link
			System.out.println("URL is " + thisURL);
			final AppletContext thisAppletContext = getAppletContext();
			if (thisTarget == null)
			{
				thisAppletContext.showDocument(thisURL);
			}
			else
			{
				thisAppletContext.showDocument(thisURL, thisTarget);
			}
		}
		catch (final Exception thisException)
		{
			System.err.println("Can't link to " + thisHref + " - " + thisException);
			thisException.printStackTrace();
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getBase()
	 */
	@Override
	public URL getBase()
	{
		final URL thisDocumentBase = getDocumentBase();
		String thisBase = getParameter("base");
		if (thisBase != null && !thisBase.isEmpty())
		{
			// 1-use 'base' parameter as full-fledged url
			try
			{
				return new URL(thisBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			// 2-use 'base' parameter as relative to document base
			if (!thisBase.endsWith("/"))
			{
				thisBase += "/";
			}
			try
			{
				return new URL(thisDocumentBase, thisBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}
		}

		// 3-default to document base
		return thisDocumentBase;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getImagesBase()
	 */
	@Override
	public URL getImagesBase()
	{
		URL thisDocumentBase = null;
		URL thisBase = null;
		String thisImageBase = getParameter("images");
		final boolean relativeToBase = !"docbase".equals(getParameter("relativeto"));
		if (thisImageBase != null && !thisImageBase.isEmpty())
		{
			// 1-use 'images' parameter as full-fledged url
			try
			{
				return new URL(thisImageBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			if (!thisImageBase.endsWith("/"))
			{
				thisImageBase += "/";
			}

			if (relativeToBase)
			{
				// 2-use 'images' parameter as relative to base
				thisBase = getBase();
				try
				{
					return new URL(thisBase, thisImageBase);
				}
				catch (final MalformedURLException thisException)
				{
					// do nothing
				}
			}
			else
			{
				// 2-use 'images' parameter as relative to document base
				thisDocumentBase = getDocumentBase();
				try
				{
					return new URL(thisDocumentBase, thisImageBase);
				}
				catch (final MalformedURLException thisException)
				{
					// do nothing
				}
			}
		}

		if (relativeToBase)
		{
			// 3-use set 'images/' path as relative to base
			if (thisBase == null)
			{
				thisBase = getBase();
			}
			try
			{
				return new URL(thisBase, "images/");
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}
		else
		{
			// 3-use set 'images/' path as relative to document base
			if (thisDocumentBase == null)
			{
				thisDocumentBase = getDocumentBase();
			}
			try
			{
				return new URL(thisDocumentBase, "images/");
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getHtmlDocumentBase()
	 */
	@Override
	public URL getHtmlDocumentBase()
	{
		return getDocumentBase();
	}
}