/**
 * Title : Treebolic browser
 * Description: Treebolic browser
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.browser;

import java.awt.Image;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import treebolic.IContext;
import treebolic.IWidget;

/**
 * Context
 * 
 * @author Bernard Bou
 */
/**
 * @author bbou
 */
/**
 * @author bbou
 */
public class Context implements IContext
{
	/**
	 * Browser
	 */
	private final MainFrame theBrowser;

	/**
	 * Connected Widget
	 */
	private IWidget theWidget;

	/**
	 * Source
	 */
	private final String theSource;

	/**
	 * Source
	 */
	private final String theBase;

	/**
	 * Source
	 */
	private final String theImageBase;

	/**
	 * Constructor
	 */
	public Context(final MainFrame thisApplication, final String thisSource, final String thisBase, final String thisImageBase)
	{
		this.theBrowser = thisApplication;
		this.theWidget = null;
		this.theSource = thisSource;
		this.theBase = thisBase;
		this.theImageBase = thisImageBase;
	}

	/**
	 * Connected Widget
	 */
	public void connect(final IWidget thisWidget)
	{
		this.theWidget = thisWidget;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.IContext#getBase()
	 */
	@Override
	public URL getBase()
	{
		URL thisDefaultBase = null;
		if (this.theBase != null && !this.theBase.isEmpty())
		{
			// 1-use 'base' parameter as full-fledged
			try
			{
				return new URL(this.theBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			// 1b-use 'base' parameter as full-fledged directory
			try
			{
				final File thisFile = new File(this.theBase);
				if (thisFile.exists() && thisFile.isDirectory())
					return thisFile.toURI().toURL();
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			// 2-use 'base' parameter as path relative to default
			String thisBase = this.theBase;
			if (!thisBase.endsWith("/"))
			{
				thisBase += "/"; //$NON-NLS-1$
			}
			thisDefaultBase = getDefaultBase();
			try
			{
				return new URL(thisDefaultBase, thisBase);
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}
		}

		// 3-default
		if (thisDefaultBase == null)
		{
			thisDefaultBase = getDefaultBase();
		}
		return thisDefaultBase;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.IContext#getImagesBase()
	 */
	@Override
	public URL getImagesBase()
	{
		URL thisDocumentUrl = null;
		URL thisDefaultUrl = null;
		if (this.theImageBase != null && !this.theImageBase.isEmpty())
		{
			// 1-use 'images' parameter as full-fledged url
			try
			{
				return new URL(this.theImageBase);
			}
			catch (final Exception thisException)
			{
				// do nothing
			}

			// 1b-use 'base' parameter as full-fledged directory
			try
			{
				final File thisFile = new File(this.theImageBase);
				if (thisFile.exists() && thisFile.isDirectory())
					return thisFile.toURI().toURL();
			}
			catch (final MalformedURLException e)
			{
				// do nothing
			}

			String thisImageBase = this.theImageBase;
			if (!thisImageBase.endsWith("/"))
			{
				thisImageBase += "/"; //$NON-NLS-1$
			}

			// 2- use 'images' parameter as relative to document base
			try
			{
				thisDocumentUrl = makeURL(this.theSource);
				return new URL(thisDocumentUrl, thisImageBase);
			}
			catch (final Exception thisException)
			{
				// do nothing
			}

			// 3- use 'images' parameter as relative to default
			try
			{
				thisDefaultUrl = getDefaultImagesBase();
				return new URL(thisDefaultUrl, thisImageBase);
			}
			catch (final Exception thisException)
			{
				// do nothing
			}
		}

		// 4- document base + "images/"
		try
		{
			if (thisDocumentUrl == null)
			{
				thisDocumentUrl = makeURL(this.theSource);
			}
			return new URL(thisDocumentUrl, "images/"); //$NON-NLS-1$
		}
		catch (final Exception thisException)
		{
			// do nothing
		}

		// 5- default + "images/"
		try
		{
			if (thisDefaultUrl == null)
			{
				thisDefaultUrl = getDefaultImagesBase();
			}
			return new URL(thisDefaultUrl, "images/"); //$NON-NLS-1$
		}
		catch (final Exception thisException)
		{
			// do nothing
		}

		// default
		return thisDefaultUrl;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getHtmlDocumentBase()
	 */
	@Override
	public URL getHtmlDocumentBase()
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getImage(java.net.URL)
	 */
	@Override
	public Image getImage(final URL thisURL)
	{
		return java.awt.Toolkit.getDefaultToolkit().getImage(thisURL);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getParameter(java.lang.String)
	 */
	@Override
	public String getParameter(final String thisName)
	{
		return this.theBrowser.getParameter(thisName);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#linkTo(java.lang.String)
	 */
	@Override
	public boolean linkTo(final String thisLinkUrl, final String thisLinkTarget)
	{
		this.theBrowser.linkTo(thisLinkUrl, thisLinkTarget, this.theWidget, this);
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#showStatus(java.lang.String)
	 */
	@Override
	public void showStatus(final String thisString)
	{
		System.out.println(Messages.getString("Context.0") + thisString); //$NON-NLS-1$
	}

	// U R L F A C T O R Y

	/**
	 * Make url
	 * 
	 * @param thisSource
	 *        source
	 * @return url
	 * @throws MalformedURLException
	 */
	protected URL makeURLAlt(final String thisSource) throws MalformedURLException
	{
		// try to consider it well-formed full-fledged url
		try
		{
			return new URL(thisSource);
		}
		catch (final MalformedURLException e)
		{
			// do nothing
		}

		// try to consider it file
		final File thisFile = new File(thisSource);
		if (thisFile.exists() && thisFile.canRead())
		{
			try
			{
				return thisFile.toURI().toURL();
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}

		// default to source relative to a base
		return new URL(getDefaultBase(), thisSource);
	}

	/**
	 * Make url
	 * 
	 * @param thisSource
	 *        source
	 * @return url
	 */
	protected URL makeURL(final String thisSource)
	{
		if (thisSource == null)
			return null;

		// try to consider it well-formed full-fledged url
		try
		{
			return new URL(thisSource);
		}
		catch (final MalformedURLException e)
		{
			// do nothing
		}

		// default to source relative to a base
		try
		{
			return new URL(getBase(), thisSource);
		}
		catch (final MalformedURLException e)
		{
			// do nothing
		}
		return null;
	}

	/**
	 * Get base default
	 * 
	 * @return base default;
	 */
	protected URL getDefaultBase()
	{
		// base parameter
		final String thisBase = getParameter("base"); //$NON-NLS-1$
		String thisURLString = thisBase != null ? thisBase : System.getProperty("user.dir"); //$NON-NLS-1$

		// tail
		if (!thisURLString.endsWith("/"))
		{
			thisURLString += "/"; //$NON-NLS-1$
		}

		// make
		try
		{
			return new URL(thisURLString);
		}
		catch (final MalformedURLException e)
		{
			// make from folder
			try
			{
				final File thisFolder = new File(thisURLString);
				return thisFolder.toURI().toURL();
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}
		return null;
	}

	/**
	 * Get images base default
	 * 
	 * @return image base default;
	 */
	protected URL getDefaultImagesBase()
	{
		final URL thisBase = getDefaultBase();
		try
		{
			return new URL(thisBase, "images/"); //$NON-NLS-1$
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
		return null;
	}
}
