/**
 * Title : Treebolic browser
 * Description: Treebolic browser
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.browser;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import treebolic.IWidget;
import treebolic.Widget;
import treebolic.commons.ButtonTabComponent;
import treebolic.commons.FileDialogs;
import treebolic.commons.Help;
import treebolic.commons.OpenDialog;
import treebolic.commons.Persist;
import treebolic.commons.SettingsDialog;
import treebolic.commons.UrlDialog;

/**
 * Browser main frame
 * 
 * @author Bernard Bou
 */
public class MainFrame extends JFrame implements HyperlinkListener
{
	private static final long serialVersionUID = 1L;

	// D A T A

	/**
	 * Command line arguments
	 */
	private final String[] theArgs;

	/**
	 * Persist
	 */
	private Properties theSettings = new Properties();

	// C O M P O N E N T S

	/**
	 * Tabbed pane
	 */
	private JTabbedPane theTabbedPane;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 * 
	 * @param theseArgs
	 *        command line arguments
	 */
	public MainFrame(final String[] theseArgs)
	{
		// System.out.println("CLASSPATH=<" + System.getProperty("java.class.path", ".") + ">");

		// settings
		this.theArgs = theseArgs;
		this.theSettings = Persist.getSettings("treebolic-browser"); //$NON-NLS-1$
		setTitle(Messages.getString("MainFrame.0")); //$NON-NLS-1$

		// menu
		setJMenuBar(makeMenuBar());

		// components
		final Container thisContentPane = getContentPane();
		thisContentPane.setLayout(new BorderLayout());
		final JEditorPane thisHome = makeHome();
		thisHome.setPreferredSize(new Dimension(600, 620));
		thisContentPane.add(thisHome);

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		pack();
		setVisible(true);

		open();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Window#processWindowEvent(java.awt.event.WindowEvent)
	 */
	@Override
	protected void processWindowEvent(final WindowEvent thisEvent)
	{
		if (thisEvent.getID() == WindowEvent.WINDOW_CLOSING)
		{
			Persist.saveSettings("treebolic-browser", this.theSettings); //$NON-NLS-1$
		}
		super.processWindowEvent(thisEvent);
	}

	// M A K E . C O M P O N E N T S

	/**
	 * HTML home pane
	 */
	private JEditorPane makeHome()
	{
		return makeBrowserPane(MainFrame.class.getResource("doc/home.html")); //$NON-NLS-1$
	}

	/**
	 * HTML pane
	 */
	private JEditorPane makeBrowserPane(final URL thisUrl)
	{
		final JEditorPane thisPanel = new JEditorPane();
		thisPanel.setEditable(false);
		try
		{
			thisPanel.setPage(thisUrl);
		}
		catch (final IOException e)
		{
			thisPanel.setText(Messages.getString("MainFrame.101") + thisUrl); //$NON-NLS-1$
		}
		thisPanel.addHyperlinkListener(this);
		return thisPanel;
	}

	/**
	 * Make menu bar
	 * 
	 * @return menu bar
	 */
	private JMenuBar makeMenuBar()
	{
		final JMenuItem thisOpenMenuItem = makeItem(Messages.getString("MainFrame.11"), "open.png", Code.OPEN, 79); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisOpenHttpMenuItem = makeItem(Messages.getString("MainFrame.12"), "openurl.png", Code.OPENURL, 0); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisOpenParseMenuItem = makeItem(Messages.getString("MainFrame.13"), "openprovider.png", Code.OPENPROVIDER, 0); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenuItem thisSettingsMenuItem = makeItem(Messages.getString("MainFrame.21"), "settings.png", Code.SETTINGS, 0); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenuItem thisAboutMenuItem = makeItem(Messages.getString("MainFrame.31"), "about.png", Code.ABOUT, 0); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisHelpMenuItem = makeItem(Messages.getString("MainFrame.32"), "help.png", Code.HELP, 0); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenu thisFilesMenu = new JMenu();
		thisFilesMenu.setText(Messages.getString("MainFrame.10")); //$NON-NLS-1$
		thisFilesMenu.add(thisOpenMenuItem);
		thisFilesMenu.add(thisOpenHttpMenuItem);
		thisFilesMenu.add(thisOpenParseMenuItem);

		final JMenu thisHelpMenu = new JMenu();
		thisHelpMenu.setText(Messages.getString("MainFrame.30")); //$NON-NLS-1$
		thisHelpMenu.add(thisAboutMenuItem);
		thisHelpMenu.add(thisHelpMenuItem);

		final JMenu thisOptionsMenu = new JMenu();
		thisOptionsMenu.setText(Messages.getString("MainFrame.20")); //$NON-NLS-1$
		thisOptionsMenu.add(thisSettingsMenuItem);

		final JMenuBar thisMenuBar = new JMenuBar();
		thisMenuBar.add(thisFilesMenu);
		thisMenuBar.add(thisOptionsMenu);
		thisMenuBar.add(thisHelpMenu);
		return thisMenuBar;
	}

	/**
	 * Make menu item
	 * 
	 * @param thisText
	 *        text
	 * @param thisImage
	 *        image
	 * @param thisCommand
	 *        command code
	 * @param thisAcceleratorKey
	 *        accelerator key
	 * @return menu item
	 */
	private JMenuItem makeItem(final String thisText, final String thisImage, final Code thisCommand, final int thisAcceleratorKey)
	{
		final JMenuItem thisItem = new JMenuItem();
		thisItem.setText(thisText);
		if (thisImage != null)
		{
			thisItem.setIcon(new ImageIcon(MainFrame.class.getResource("images/" + thisImage))); //$NON-NLS-1$
		}
		if (thisAcceleratorKey != 0)
		{
			thisItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(thisAcceleratorKey, InputEvent.CTRL_MASK, false));
		}
		thisItem.addActionListener(new java.awt.event.ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				execute(thisCommand);
			}
		});
		return thisItem;
	}

	// C O M M A N D

	/**
	 * Command codes
	 */
	static private enum Code
	{
		OPEN, OPENURL, OPENPROVIDER, SETTINGS, ABOUT, HELP
	};

	/**
	 * Run command
	 * 
	 * @param thisCode
	 *        command code
	 */
	private void execute(final Code thisCode)
	{
		switch (thisCode)
		{
		case OPEN:
		{
			final String thisSource = FileDialogs.getXmlUrl(this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
			if (thisSource != null)
			{
				open(null, thisSource, null, null);
			}
			break;
		}
		case OPENURL:
		{
			final JDialog thisDialog = new UrlDialog(this.theSettings);
			thisDialog.setModal(true);
			thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			thisDialog.setVisible(true);
			final String thisSource = this.theSettings.getProperty("openurl"); //$NON-NLS-1$
			if (thisSource != null)
			{
				open(null, thisSource, null, null);
			}
			break;
		}
		case OPENPROVIDER:
		{
			final JDialog thisDialog = new OpenDialog(this.theSettings);
			thisDialog.setModal(true);
			thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			thisDialog.setVisible(true);
			final String thisSource = this.theSettings.getProperty("source"); //$NON-NLS-1$
			final String thisProvider = this.theSettings.getProperty("provider"); //$NON-NLS-1$
			if (thisSource != null && !thisSource.isEmpty() && thisProvider != null && !thisProvider.isEmpty())
			{
				open(thisProvider, thisSource, null, null);
			}
			break;
		}
		case ABOUT:
		{
			final JDialog thisDialog = new AboutDialog();
			thisDialog.setModal(true);
			thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			thisDialog.setVisible(true);
			break;
		}
		case SETTINGS:
		{
			final SettingsDialog thisDialog = new SettingsDialog(this.theSettings);
			thisDialog.setModal(true);
			thisDialog.setVisible(true);
			thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			if (thisDialog.ok)
			{
				Persist.saveSettings("treebolic-browser", this.theSettings); //$NON-NLS-1$
			}
			break;
		}
		case HELP:
		{
			help();
			break;
		}
		}
	}

	/**
	 * Open
	 */
	private void open()
	{
		final List<ContextData> theseContextData = getContextData();
		if (theseContextData != null)
		{
			for (final ContextData thisContextData : theseContextData)
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						open(thisContextData.theProvider, thisContextData.theSource, thisContextData.theBase, thisContextData.theImageBase);
					}
				});
			}
		}
	}

	/**
	 * Open source
	 * 
	 * @param thatProvider
	 *        provider
	 * @param thisSource
	 *        source
	 * @param thisBase
	 *        base
	 * @param thisImageBase
	 *        image base
	 */
	private void open(final String thatProvider, final String thisSource, final String thisBase, final String thisImageBase)
	{
		String thisProvider = thatProvider;

		// widget
		final Context thisContext = new Context(this, thisSource, thisBase, thisImageBase);
		final IWidget thisWidget = new Widget(thisContext);
		thisContext.connect(thisWidget);

		if (thisProvider != null && thisProvider.isEmpty())
		{
			thisProvider = null;
		}

		thisWidget.init(thisProvider, thisSource);

		// tabbed pane
		if (this.theTabbedPane == null)
		{
			this.theTabbedPane = new JTabbedPane();
			final Container thisContentPane = getContentPane();
			thisContentPane.removeAll();
			thisContentPane.add(this.theTabbedPane);
			validate();
		}

		// tab
		final String thisToolTip = "<html><body><strong>" + thisSource + "</strong><br>" + thisProvider + "</body></html>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		this.theTabbedPane.addTab(mangle(thisSource), null, (Component) thisWidget, thisToolTip);
		final int thisIndex = this.theTabbedPane.indexOfComponent((Component) thisWidget);
		this.theTabbedPane.setTabComponentAt(thisIndex, new ButtonTabComponent(this.theTabbedPane));
		this.theTabbedPane.setSelectedIndex(thisIndex);
	}

	/**
	 * Mangle string
	 * 
	 * @param thisString
	 *        string to mangle
	 * @return mangled string
	 */
	private String mangle(final String thatString)
	{
		String thisString = thatString;

		final int thisIndex = Math.max(thisString.lastIndexOf('/'), thisString.lastIndexOf('\\'));
		if (thisIndex != -1)
		{
			thisString = thisString.substring(thisIndex + 1);
		}
		return thisString;
	}

	/**
	 * Help
	 */
	private void help()
	{
		Help.help(this.theSettings.getProperty("browser"), this.theSettings.getProperty("help")); //$NON-NLS-1$ //$NON-NLS-2$
	}

	// A R G U M E N T S

	/**
	 * Get parameter
	 * 
	 * @param thisName
	 *        parameter
	 * @return parameter value
	 */
	public String getParameter(final String thisName)
	{
		// param1=<val> param2=<"val with spaces"> ...
		if (this.theArgs == null)
			return null;

		for (final String thisArg : this.theArgs)
			if (thisArg.startsWith(thisName + "=")) //$NON-NLS-1$
			{
				String thisValue = thisArg.substring(thisName.length() + 1);
				if (thisValue.startsWith("\"")) //$NON-NLS-1$
				{
					thisValue = thisValue.substring(1);
					if (thisValue.endsWith("\""))
					{
						thisValue = thisValue.substring(0, thisValue.length() - 1);
					}
				}
				return thisValue;
			}
		return null;
	}

	/**
	 * Context data
	 */
	private class ContextData
	{
		/**
		 * Provider
		 */
		public String theProvider;

		/**
		 * Source
		 */
		public String theSource;

		/**
		 * Document base
		 */
		public String theBase;

		/**
		 * Image base
		 */
		public String theImageBase;

		/**
		 * Constructor
		 * 
		 * @param thisProvider
		 *        provider class
		 * @param thisSource
		 *        document source
		 * @param thisBase
		 *        document base
		 * @param thisImageBase
		 *        image base
		 */
		public ContextData(final String thisProvider, final String thisSource, final String thisBase, final String thisImageBase)
		{
			this.theProvider = thisProvider;
			this.theSource = thisSource;
			this.theBase = thisBase;
			this.theImageBase = thisImageBase;
		}

		/*
		 * (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString()
		{
			return "p=" + this.theProvider + " s=" + this.theSource + " b=" + this.theBase + " i=" + this.theImageBase; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		}
	}

	/**
	 * Parse context data from command line
	 * 
	 * @return context data from command line
	 */
	private List<ContextData> getContextData()
	{
		if (this.theArgs == null)
			return null;
		boolean debug = false;
		String thisDefaultProvider = null;
		String thisDefaultBase = null;
		String thisDefaultImageBase = null;
		String thisProvider = null;
		String thisBase = null;
		String thisImageBase = null;
		List<ContextData> theseSources = null;
		for (final String thisArg : this.theArgs)
		{
			if (thisArg.startsWith("base=")) //$NON-NLS-1$
			{
				thisDefaultBase = thisArg.substring(5);
				continue;
			}
			if (thisArg.startsWith("images=")) //$NON-NLS-1$
			{
				thisDefaultImageBase = thisArg.substring(7);
				continue;
			}
			if (thisArg.startsWith("provider=")) //$NON-NLS-1$
			{
				thisDefaultProvider = thisArg.substring(9);
				continue;
			}
			if (thisArg.startsWith("debug=")) //$NON-NLS-1$
			{
				debug = true;
				continue;
			}
			if (thisArg.startsWith("provider1=")) //$NON-NLS-1$
			{
				thisProvider = thisArg.substring(10);
				continue;
			}
			if (thisArg.startsWith("base1=")) //$NON-NLS-1$
			{
				thisBase = thisArg.substring(6);
				continue;
			}
			if (thisArg.startsWith("images1=")) //$NON-NLS-1$
			{
				thisImageBase = thisArg.substring(8);
				continue;
			}

			final String[] theseFields = thisArg.split("="); //$NON-NLS-1$
			String thisSource = theseFields[0];
			if (theseFields.length > 1)
			{
				thisSource = theseFields[1];
			}

			if (theseSources == null)
			{
				theseSources = new ArrayList<ContextData>();
			}
			final ContextData thisContext = new ContextData(thisProvider != null ? thisProvider : thisDefaultProvider, thisSource, thisBase != null ? thisBase : thisDefaultBase, thisImageBase != null ? thisImageBase : thisDefaultImageBase);
			theseSources.add(thisContext);
			if (debug)
			{
				System.out.println("CONTEXT=" + thisContext);
			}

			// clear
			thisProvider = null;
			thisBase = null;
			thisImageBase = null;
		}
		return theseSources;
	}

	// E V E N T

	/**
	 * Link to
	 * 
	 * @param thisLinkUrl
	 *        link url
	 * @param thisLinkTarget
	 *        thisLinkTarget link target
	 * @param thisWidget
	 *        widget
	 * @param thisContext
	 *        widget's context
	 */
	public void linkTo(final String thisLinkUrl, final String thisLinkTarget, final IWidget thisWidget, final Context thisContext)
	{
		final int thisIndex = this.theTabbedPane.indexOfComponent((Component) thisWidget);
		final URL thisUrl = thisContext.makeURL(thisLinkUrl);
		if (thisUrl == null)
			return;

		final JComponent thisComponent = makeBrowserPane(thisUrl);

		this.theTabbedPane.setComponentAt(thisIndex, thisComponent);
		this.theTabbedPane.setTitleAt(thisIndex, mangle(thisLinkUrl));
		this.theTabbedPane.setToolTipTextAt(thisIndex, thisLinkUrl);
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.event.HyperlinkListener#hyperlinkUpdate(javax.swing.event.HyperlinkEvent)
	 */
	@Override
	public void hyperlinkUpdate(final HyperlinkEvent thisEvent)
	{
		// follow link
		final JEditorPane thisEditorPane = (JEditorPane) thisEvent.getSource();
		try
		{
			thisEditorPane.setPage(thisEvent.getURL());
		}
		catch (final IOException e)
		{
			thisEditorPane.setText(Messages.getString("MainFrame.103") + thisEvent.getURL()); //$NON-NLS-1$
		}
	}
}
