/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.commons;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Properties;
import java.util.Set;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

/**
 * Import dialog
 * 
 * @author Bernard Bou
 */
public class OpenDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	// V A L U E S

	/**
	 * Properties (input/output)
	 */
	protected Properties theProperties;

	/**
	 * Ok result
	 */
	public boolean ok;

	// C O M P O N E N T S

	/**
	 * Source text
	 */
	private JTextField theSourceTextField;

	/**
	 * Provider text
	 */
	private JComboBox theProviderComboBox;

	/**
	 * Data panel
	 */
	protected JPanel theDataPanel;

	/**
	 * Constructor
	 * 
	 * @param theseProperties
	 *        settings
	 */
	public OpenDialog(final Properties theseProperties)
	{
		super();
		this.theProperties = theseProperties;
		if (this.theProperties == null)
		{
			this.theProperties = new Properties();
		}

		final Set<String> thesePresetProviders = Searcher.findClasses(".*\\.Provider"); //$NON-NLS-1$
		initialize(thesePresetProviders);
	}

	/**
	 * Initialize
	 */
	@SuppressWarnings("serial")
	protected void initialize(final Collection<String> theseProviders)
	{
		setTitle(Messages.getString("OpenDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// images
		final Icon thisIcon = new ImageIcon(OpenDialog.class.getResource("images/open.png")); //$NON-NLS-1$
		final JLabel thisHeaderLabel = new JLabel();
		thisHeaderLabel.setIcon(thisIcon);
		thisHeaderLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisHeaderLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisHeaderLabel.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
		thisHeaderLabel.setText(Messages.getString("OpenDialog.1")); //$NON-NLS-1$
		thisHeaderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		// labels
		final JLabel thisSourceLabel = new JLabel(Messages.getString("OpenDialog.5")); //$NON-NLS-1$
		final JLabel thisProviderLabel = new JLabel(Messages.getString("OpenDialog.12")); //$NON-NLS-1$

		// text
		final ListCellRenderer thisRenderer = new DefaultListCellRenderer()
		{
			/*
			 * (non-Javadoc)
			 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
			 */
			@Override
			public Component getListCellRendererComponent(final JList thisList, final Object thisValue, final int thisIndex, final boolean thisIsSelected, final boolean thisCellHasFocus)
			{
				String thisString = (String) thisValue;
				if (thisString != null)
				{
					final int thisPosition = thisString.lastIndexOf('/');
					if (thisPosition != -1)
					{
						thisString = thisString.substring(thisPosition + 1);
					}
				}
				return super.getListCellRendererComponent(thisList, thisString, thisIndex, thisIsSelected, thisCellHasFocus);
			}
		};
		this.theSourceTextField = new JTextField(32);
		this.theProviderComboBox = new JComboBox();
		this.theProviderComboBox.setEditable(true);
		this.theProviderComboBox.setRenderer(thisRenderer);
		this.theProviderComboBox.setPreferredSize(new Dimension(300, 20));
		if (theseProviders != null)
		{
			for (final String thisItem : theseProviders)
			{
				this.theProviderComboBox.addItem(thisItem);
			}
		}

		// tooltips
		this.theSourceTextField.setToolTipText(Messages.getString("OpenDialog.6")); //$NON-NLS-1$
		this.theProviderComboBox.setToolTipText(Messages.getString("OpenDialog.15")); //$NON-NLS-1$

		// buttons
		final JButton thisSourceBrowseButton = new JButton(Messages.getString("OpenDialog.9")); //$NON-NLS-1$
		final JButton thisProviderAddButton = new JButton(Messages.getString("OpenDialog.10")); //$NON-NLS-1$
		final JButton thisOKButton = new JButton(Messages.getString("OpenDialog.2")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("OpenDialog.3")); //$NON-NLS-1$

		// panels
		this.theDataPanel = new JPanel();
		this.theDataPanel.setLayout(new GridBagLayout());
		this.theDataPanel.add(thisSourceLabel, new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
		this.theDataPanel.add(thisProviderLabel, new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
		this.theDataPanel.add(this.theSourceTextField, new GridBagConstraints(0, 1, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(this.theProviderComboBox, new GridBagConstraints(0, 3, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(thisSourceBrowseButton, new GridBagConstraints(1, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		this.theDataPanel.add(thisProviderAddButton, new GridBagConstraints(1, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisOKButton);

		// action
		thisSourceBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getXmlUrl(OpenDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					OpenDialog.this.theSourceTextField.setText(thisFolder);
				}
			}
		});
		thisProviderAddButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisProvider = ask(Messages.getString("OpenDialog.15")); //$NON-NLS-1$
				if (thisProvider != null && !thisProvider.isEmpty())
				{
					OpenDialog.this.theProviderComboBox.addItem(thisProvider);
					OpenDialog.this.theProviderComboBox.getEditor().setItem(thisProvider);
				}
			}
		});
		thisOKButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				OpenDialog.this.ok = true;
				setVisible(false);
			}
		});
		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				setVisible(false);
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisHeaderLabel);
		thisPanel.add(this.theDataPanel);
		thisPanel.add(thisButtonPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			this.ok = false;

			// read properties into components
			this.theSourceTextField.setText(this.theProperties.getProperty("source")); //$NON-NLS-1$
			final String thisValue = this.theProperties.getProperty("provider"); //$NON-NLS-1$
			this.theProviderComboBox.addItem(thisValue);
			this.theProviderComboBox.getEditor().setItem(thisValue);

			pack();
			Utils.center(this);
		}
		else
		{
			if (this.ok)
			{
				// update properties from components
				this.theProperties.setProperty("source", this.theSourceTextField.getText()); //$NON-NLS-1$
				this.theProperties.setProperty("provider", (String) this.theProviderComboBox.getEditor().getItem()); //$NON-NLS-1$
			}
		}
		super.setVisible(thisFlag);
	}

	/**
	 * Ask
	 * 
	 * @param thisMessage
	 *        message
	 * @return input
	 */
	protected String ask(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n"); //$NON-NLS-1$
		return JOptionPane.showInputDialog(null, theseLines);
	}

	/**
	 * Inform dialog
	 * 
	 * @param thisMessage
	 *        message
	 */
	protected void inform(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n"); //$NON-NLS-1$
		JOptionPane.showMessageDialog(null, theseLines, Messages.getString("OpenDialog.22"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
	}
}
