/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.commons;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

/**
 * Settings dialog
 * 
 * @author Bernard Bou
 */
public class SettingsDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	// V A L U E S

	/**
	 * Properties (input/output)
	 */
	protected Properties theSettings;

	/**
	 * Ok result
	 */
	public boolean ok;

	// C O M P O N E N T S

	/**
	 * Base text
	 */
	private JTextField theBaseTextField;

	/**
	 * Repository text
	 */
	private JTextField theImageRepositoryTextField;

	/**
	 * Help text
	 */
	private JTextField theHelpTextField;

	/**
	 * Browser text
	 */
	private JTextField theBrowserTextField;

	/**
	 * Data panel
	 */
	protected JPanel theDataPanel;

	/**
	 * Constructor
	 * 
	 * @param theseSettings
	 *        settings
	 */
	public SettingsDialog(final Properties theseSettings)
	{
		super();
		this.theSettings = theseSettings;
		initialize();
	}

	/**
	 * Initialize
	 * 
	 * @throws Exception
	 */
	protected void initialize()
	{
		setTitle(Messages.getString("SettingsDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// images
		final Icon thisIcon = new ImageIcon(SettingsDialog.class.getResource("images/settings.png")); //$NON-NLS-1$
		final JLabel thisHeaderLabel = new JLabel();
		thisHeaderLabel.setIcon(thisIcon);
		thisHeaderLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisHeaderLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisHeaderLabel.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
		thisHeaderLabel.setText(Messages.getString("SettingsDialog.1")); //$NON-NLS-1$
		thisHeaderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		// labels
		final JLabel thisWorkFolderLabel = new JLabel(Messages.getString("SettingsDialog.4")); //$NON-NLS-1$
		final JLabel thisImageRepositoryFolderLabel = new JLabel(Messages.getString("SettingsDialog.5")); //$NON-NLS-1$
		final JLabel thisHelpFolderLabel = new JLabel(Messages.getString("SettingsDialog.6")); //$NON-NLS-1$
		final JLabel thisBrowserLabel = new JLabel(Messages.getString("SettingsDialog.7")); //$NON-NLS-1$

		// text
		this.theBaseTextField = new JTextField(32);
		this.theImageRepositoryTextField = new JTextField(32);
		this.theHelpTextField = new JTextField(32);
		this.theBrowserTextField = new JTextField(32);

		// tooltips
		this.theBaseTextField.setToolTipText(Messages.getString("SettingsDialog.8")); //$NON-NLS-1$
		this.theImageRepositoryTextField.setToolTipText(Messages.getString("SettingsDialog.9")); //$NON-NLS-1$
		this.theHelpTextField.setToolTipText(Messages.getString("SettingsDialog.10")); //$NON-NLS-1$
		this.theBrowserTextField.setToolTipText(Messages.getString("SettingsDialog.11")); //$NON-NLS-1$

		// buttons
		final JButton thisBaseBrowseButton = new JButton(Messages.getString("SettingsDialog.12")); //$NON-NLS-1$
		final JButton thisImageRepositoryBrowseButton = new JButton(Messages.getString("SettingsDialog.12")); //$NON-NLS-1$
		final JButton thisHelpBrowseButton = new JButton(Messages.getString("SettingsDialog.12")); //$NON-NLS-1$
		final JButton thisBrowserBrowseButton = new JButton(Messages.getString("SettingsDialog.12")); //$NON-NLS-1$
		final JButton thisOKButton = new JButton(Messages.getString("SettingsDialog.20")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("SettingsDialog.21")); //$NON-NLS-1$

		// panels
		this.theDataPanel = new JPanel();
		this.theDataPanel.setLayout(new GridBagLayout());
		this.theDataPanel.add(thisWorkFolderLabel, new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(thisImageRepositoryFolderLabel, new GridBagConstraints(0, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0));
		// space
		this.theDataPanel.add(thisHelpFolderLabel, new GridBagConstraints(0, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(thisBrowserLabel, new GridBagConstraints(0, 4, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0));

		this.theDataPanel.add(this.theBaseTextField, new GridBagConstraints(1, 0, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		this.theDataPanel.add(this.theImageRepositoryTextField, new GridBagConstraints(1, 1, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		// space
		this.theDataPanel.add(this.theHelpTextField, new GridBagConstraints(1, 3, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		this.theDataPanel.add(this.theBrowserTextField, new GridBagConstraints(1, 4, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));

		this.theDataPanel.add(thisBaseBrowseButton, new GridBagConstraints(2, 0, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		this.theDataPanel.add(thisImageRepositoryBrowseButton, new GridBagConstraints(2, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		// space
		this.theDataPanel.add(thisHelpBrowseButton, new GridBagConstraints(2, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		this.theDataPanel.add(thisBrowserBrowseButton, new GridBagConstraints(2, 4, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisOKButton);

		// action
		thisBaseBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getFolder(SettingsDialog.this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					SettingsDialog.this.theBaseTextField.setText(thisFolder);
				}
			}
		});
		thisImageRepositoryBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getFolder(SettingsDialog.this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					SettingsDialog.this.theImageRepositoryTextField.setText(thisFolder);
				}
			}
		});
		thisHelpBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getFolder(SettingsDialog.this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					SettingsDialog.this.theHelpTextField.setText(thisFolder);
				}
			}
		});
		thisBrowserBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getExec(SettingsDialog.this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					SettingsDialog.this.theBrowserTextField.setText(thisFolder);
				}
			}
		});
		thisOKButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				SettingsDialog.this.ok = true;
				setVisible(false);
			}
		});
		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				setVisible(false);
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisHeaderLabel);
		thisPanel.add(this.theDataPanel);
		thisPanel.add(thisButtonPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			this.ok = false;

			// read properties into components
			this.theBaseTextField.setText(this.theSettings.getProperty("base")); //$NON-NLS-1$
			this.theImageRepositoryTextField.setText(this.theSettings.getProperty("images")); //$NON-NLS-1$
			this.theHelpTextField.setText(this.theSettings.getProperty(Messages.getString("SettingsDialog.28"))); //$NON-NLS-1$
			this.theBrowserTextField.setText(this.theSettings.getProperty(Messages.getString("SettingsDialog.29"))); //$NON-NLS-1$

			pack();
			Utils.center(this);
		}
		else
		{
			if (this.ok)
			{
				// update properties from components
				this.theSettings.setProperty(Messages.getString("SettingsDialog.30"), this.theBaseTextField.getText()); //$NON-NLS-1$
				this.theSettings.setProperty(Messages.getString("SettingsDialog.31"), this.theImageRepositoryTextField.getText()); //$NON-NLS-1$
				this.theSettings.setProperty(Messages.getString("SettingsDialog.32"), this.theHelpTextField.getText()); //$NON-NLS-1$
				this.theSettings.setProperty(Messages.getString("SettingsDialog.33"), this.theBrowserTextField.getText()); //$NON-NLS-1$
			}
		}
		super.setVisible(thisFlag);
	}
}
