/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.commons;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * Site dialog
 * 
 * @author Bernard Bou
 */
public class SiteDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Mode types
	 */
	static private enum Mode
	{
		FILE, NET
	}

	// V A L U E S

	/**
	 * Properties (input/output)
	 */
	private final Properties theProperties;

	/**
	 * Result
	 */
	public boolean ok;

	// C O M P O N E N T S

	/**
	 * Combo box
	 */
	private JComboBox theComboBox;

	/**
	 * Server field
	 */
	private JTextField theServerTextField;

	/**
	 * Directory field
	 */
	private JTextField theDirectoryTextField;

	/**
	 * Login field
	 */
	private JTextField theLoginTextField;

	/**
	 * Password field
	 */
	private JPasswordField thePasswordTextField;

	/**
	 * Path field
	 */
	private JTextField thePathTextField;

	// C O N S T R U C T OR

	/**
	 * Constructor
	 */
	public SiteDialog(final Properties theseProperties)
	{
		this.theProperties = theseProperties;
		initialize();
	}

	/**
	 * Initialize dialog
	 */
	private void initialize()
	{
		setTitle(Messages.getString("SiteDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// images
		final Icon thisIcon = new ImageIcon(SiteDialog.class.getResource("images/sitemake.png")); //$NON-NLS-1$
		final JLabel thisHeaderLabel = new JLabel();
		thisHeaderLabel.setIcon(thisIcon);
		thisHeaderLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisHeaderLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisHeaderLabel.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
		thisHeaderLabel.setText(Messages.getString("SiteDialog.1")); //$NON-NLS-1$
		thisHeaderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		final Icon thisFileIcon = new ImageIcon(SiteDialog.class.getResource("images/local.png"));
		final Icon thisHttpIcon = new ImageIcon(SiteDialog.class.getResource("images/net.png"));

		// mode combo
		final Object[] theseOptions = { Mode.FILE, Mode.NET };
		this.theComboBox = new JComboBox(theseOptions);
		this.theComboBox.setRenderer(new ListCellRenderer()
		{
			private final JLabel theLabel = new JLabel();

			@Override
			public Component getListCellRendererComponent(final JList thisList, final Object thisValue, final int thisIndex, final boolean thisIsSelected, final boolean thisCellHasFocus)
			{
				this.theLabel.setText(thisValue.toString());
				this.theLabel.setIcon(thisValue.equals(Mode.FILE) ? thisFileIcon : thisHttpIcon);
				return this.theLabel;
			}
		});

		// text fields
		this.theServerTextField = new JTextField(16);
		this.theDirectoryTextField = new JTextField(16);
		this.theLoginTextField = new JTextField(16);
		this.thePasswordTextField = new JPasswordField(16);
		this.thePathTextField = new JTextField(16);

		// buttons
		final JButton thisOKButton = new JButton(Messages.getString("SiteDialog.2")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("SiteDialog.3")); //$NON-NLS-1$
		final JButton thisBrowsePathButton = new JButton(Messages.getString("SiteDialog.4")); //$NON-NLS-1$

		// panels
		final JPanel thisSelectionPanel = new JPanel();
		thisSelectionPanel.add(new JLabel(Messages.getString("SiteDialog.5"))); //$NON-NLS-1$
		thisSelectionPanel.setLayout(new FlowLayout());
		thisSelectionPanel.add(this.theComboBox);

		final JPanel thePathPanel = new JPanel();
		thePathPanel.setLayout(new FlowLayout());
		thePathPanel.add(new JLabel(Messages.getString("SiteDialog.30"))); //$NON-NLS-1$
		thePathPanel.add(this.thePathTextField);

		final JPanel theFilePanel = new JPanel();
		theFilePanel.setLayout(new FlowLayout());
		theFilePanel.add(thePathPanel);
		theFilePanel.add(thisBrowsePathButton);

		final JPanel theFTPPanel = new JPanel();
		theFTPPanel.setLayout(new GridBagLayout());
		theFTPPanel.add(new JLabel(Messages.getString("SiteDialog.31")), new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		theFTPPanel.add(new JLabel(Messages.getString("SiteDialog.32")), new GridBagConstraints(0, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		theFTPPanel.add(new JLabel(Messages.getString("SiteDialog.33")), new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		theFTPPanel.add(new JLabel(Messages.getString("SiteDialog.34")), new GridBagConstraints(0, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		theFTPPanel.add(this.theServerTextField, new GridBagConstraints(1, 0, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		theFTPPanel.add(this.theDirectoryTextField, new GridBagConstraints(1, 1, 1, 1, 1., 01., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		theFTPPanel.add(this.theLoginTextField, new GridBagConstraints(1, 2, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		theFTPPanel.add(this.thePasswordTextField, new GridBagConstraints(1, 3, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisOKButton);

		// tab init
		final JTabbedPane thisTabbedPanel = new JTabbedPane();
		thisTabbedPanel.addTab(Mode.FILE.toString(), thisFileIcon, theFilePanel, Messages.getString("SiteDialog.20")); //$NON-NLS-1$
		thisTabbedPanel.addTab(Mode.NET.toString(), thisHttpIcon, theFTPPanel, Messages.getString("SiteDialog.21")); //$NON-NLS-1$

		// connect selections
		thisTabbedPanel.addChangeListener(new ChangeListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void stateChanged(final ChangeEvent thisE)
			{
				final int thisIndex = thisTabbedPanel.getSelectedIndex();
				SiteDialog.this.theComboBox.setSelectedIndex(thisIndex);
			}
		});
		this.theComboBox.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent thisE)
			{
				final int thisIndex = SiteDialog.this.theComboBox.getSelectedIndex();
				thisTabbedPanel.setSelectedIndex(thisIndex);
			}
		});

		// assemble tabs
		final JPanel thisSettingsPanel = new JPanel();
		thisSettingsPanel.add(thisTabbedPanel);

		// events
		thisBrowsePathButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final String thisPath = FileDialogs.getFolder(SiteDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisPath != null)
				{
					SiteDialog.this.thePathTextField.setText(thisPath);
				}
			}
		});
		thisOKButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				SiteDialog.this.ok = true;
				setVisible(false);
			}
		});
		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				setVisible(false);
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisHeaderLabel);
		thisPanel.add(thisSelectionPanel);
		thisPanel.add(thisSettingsPanel);
		thisPanel.add(thisButtonPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			this.ok = false;

			// read properties into components
			this.theServerTextField.setText(this.theProperties.getProperty("server")); //$NON-NLS-1$
			this.theDirectoryTextField.setText(this.theProperties.getProperty("directory")); //$NON-NLS-1$
			this.theLoginTextField.setText(this.theProperties.getProperty("login")); //$NON-NLS-1$
			this.thePasswordTextField.setText(this.theProperties.getProperty("password")); //$NON-NLS-1$
			this.thePathTextField.setText(this.theProperties.getProperty("path")); //$NON-NLS-1$

			final Mode thisMode = this.theProperties.getProperty("mode") != null ? Mode.valueOf(this.theProperties.getProperty("mode")) : Mode.FILE; //$NON-NLS-1$ //$NON-NLS-2$
			this.theComboBox.setSelectedItem(thisMode);

			pack();
			Utils.center(this);
		}
		else
		{
			if (this.ok)
			{
				// update properties from components
				final Mode thisMode = (Mode) this.theComboBox.getSelectedItem();
				this.theProperties.setProperty("mode", thisMode.toString()); //$NON-NLS-1$
				switch (thisMode)
				{
				case NET:
					this.theProperties.setProperty("server", this.theServerTextField.getText()); //$NON-NLS-1$
					this.theProperties.setProperty("directory", this.theDirectoryTextField.getText()); //$NON-NLS-1$
					this.theProperties.setProperty("login", this.theLoginTextField.getText()); //$NON-NLS-1$
					this.theProperties.setProperty("password", new String(this.thePasswordTextField.getPassword())); //$NON-NLS-1$
					break;

				case FILE:
					this.theProperties.setProperty("path", this.thePathTextField.getText()); //$NON-NLS-1$
					break;
				}
			}
		}
		super.setVisible(thisFlag);
	}
}
