/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.commons;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;

/**
 * Export dialog
 * 
 * @author Bernard Bou
 */
public class XslExportDialog extends JDialog
{
	static private final long serialVersionUID = 1L;

	static private String[] theViewOutputs = { "view:xml", "view:html", "view:text" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	// V A L U E S

	/**
	 * Properties (input/output)
	 */
	protected Properties theProperties;

	/**
	 * Ok result
	 */
	public boolean ok;

	// C O M P O N E N T S

	/**
	 * Export text
	 */
	private JComboBox theExportComboBox;

	/**
	 * XSL text
	 */
	private JComboBox theXslComboBox;

	/**
	 * Data panel
	 */
	protected JPanel theDataPanel;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 * 
	 * @param theseProperties
	 *        settings
	 */
	public XslExportDialog(final Properties theseProperties)
	{
		super();
		this.theProperties = theseProperties;

		final List<String> thesePresetXsls = new ArrayList<String>();
		for (final String thisUrl : Searcher.findFileUrls(".*treebolic2.*\\.xsl$"))
		{
			thesePresetXsls.add(thisUrl);
		}

		final String theXSLsProperty = this.theProperties.getProperty("exportxsls"); //$NON-NLS-1$
		if (theXSLsProperty != null)
		{
			final String[] theseProviders = theXSLsProperty.split(":"); //$NON-NLS-1$
			for (final String thisProvider : theseProviders)
			{
				thesePresetXsls.add(thisProvider);
			}
		}

		initialize(thesePresetXsls, XslExportDialog.theViewOutputs);
	}

	/**
	 * Initialize
	 */
	@SuppressWarnings("serial")
	protected void initialize(final List<String> thesePresetXsls, final String[] theseViewOutputs)
	{
		setTitle(Messages.getString("XslExportDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// images
		final Icon thisIcon = new ImageIcon(XslExportDialog.class.getResource("images/xsl.png")); //$NON-NLS-1$
		final JLabel thisHeaderLabel = new JLabel();
		thisHeaderLabel.setIcon(thisIcon);
		thisHeaderLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisHeaderLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisHeaderLabel.setFont(new Font("Dialog", Font.BOLD, 16)); //$NON-NLS-1$
		thisHeaderLabel.setText(Messages.getString("XslExportDialog.1")); //$NON-NLS-1$
		thisHeaderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		// labels
		final JLabel thisXmlLabel = new JLabel(Messages.getString("XslExportDialog.10")); //$NON-NLS-1$
		final JLabel thisXslLabel = new JLabel(Messages.getString("XslExportDialog.11")); //$NON-NLS-1$

		// text
		final ListCellRenderer thisRenderer = new DefaultListCellRenderer()
		{
			/*
			 * (non-Javadoc)
			 * @see javax.swing.DefaultListCellRenderer#getListCellRendererComponent(javax.swing.JList, java.lang.Object, int, boolean, boolean)
			 */
			@Override
			public Component getListCellRendererComponent(final JList thisList, final Object thisValue, final int thisIndex, final boolean thisIsSelected, final boolean thisCellHasFocus)
			{
				String thisString = (String) thisValue;
				if (thisString != null)
				{
					final int thisPosition = thisString.lastIndexOf('/');
					if (thisPosition != -1)
					{
						thisString = thisString.substring(thisPosition + 1);
					}
				}
				return super.getListCellRendererComponent(thisList, thisString, thisIndex, thisIsSelected, thisCellHasFocus);
			}
		};
		this.theExportComboBox = new JComboBox(theseViewOutputs);
		this.theExportComboBox.setEditable(true);
		this.theExportComboBox.setRenderer(thisRenderer);
		this.theExportComboBox.setPreferredSize(new Dimension(300, 20));
		this.theXslComboBox = new JComboBox();
		this.theXslComboBox.setEditable(true);
		this.theXslComboBox.setRenderer(thisRenderer);
		this.theXslComboBox.setPreferredSize(new Dimension(300, 20));
		for (final String thisItem : thesePresetXsls)
		{
			this.theXslComboBox.addItem(thisItem);
		}

		// tooltips
		this.theExportComboBox.setToolTipText(Messages.getString("XslExportDialog.12")); //$NON-NLS-1$
		this.theXslComboBox.setToolTipText(Messages.getString("XslExportDialog.13")); //$NON-NLS-1$

		// buttons
		final JButton thisXmlBrowseButton = new JButton(Messages.getString("XslExportDialog.15")); //$NON-NLS-1$
		final JButton thisXslBrowseButton = new JButton(Messages.getString("XslExportDialog.15")); //$NON-NLS-1$
		final JButton thisOKButton = new JButton(Messages.getString("XslExportDialog.20")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("XslExportDialog.21")); //$NON-NLS-1$

		// panels
		this.theDataPanel = new JPanel();
		this.theDataPanel.setLayout(new GridBagLayout());
		this.theDataPanel.add(thisXslLabel, new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
		this.theDataPanel.add(thisXmlLabel, new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 20, 0, 0), 0, 0));
		this.theDataPanel.add(this.theXslComboBox, new GridBagConstraints(0, 1, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(this.theExportComboBox, new GridBagConstraints(0, 3, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(thisXslBrowseButton, new GridBagConstraints(1, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		this.theDataPanel.add(thisXmlBrowseButton, new GridBagConstraints(1, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisOKButton);

		// action
		thisXmlBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getXmlUrl(XslExportDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					XslExportDialog.this.theExportComboBox.getEditor().setItem(thisFolder);
				}
			}
		});
		thisXslBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getXmlUrl(XslExportDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					XslExportDialog.this.theXslComboBox.getEditor().setItem(thisFolder);
				}
			}
		});
		thisOKButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				XslExportDialog.this.ok = true;
				setVisible(false);
			}
		});
		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				setVisible(false);
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisHeaderLabel);
		thisPanel.add(this.theDataPanel);
		thisPanel.add(thisButtonPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			this.ok = false;

			// read properties into components
			final String thisExportValue = this.theProperties.getProperty("exporturl"); //$NON-NLS-1$
			this.theExportComboBox.addItem(thisExportValue);
			this.theExportComboBox.getEditor().setItem(thisExportValue);

			final String thisXslValue = this.theProperties.getProperty("exportxsl"); //$NON-NLS-1$
			this.theXslComboBox.addItem(thisXslValue);
			this.theXslComboBox.getEditor().setItem(thisXslValue);

			pack();
			Utils.center(this);
		}
		else
		{
			if (this.ok)
			{
				// update properties from components
				this.theProperties.setProperty("exporturl", (String) this.theExportComboBox.getEditor().getItem()); //$NON-NLS-1$
				this.theProperties.setProperty("exportxsl", (String) this.theXslComboBox.getEditor().getItem()); //$NON-NLS-1$
			}
		}
		super.setVisible(thisFlag);
	}
}
