/**
 * Title: Hyperbolic engine
 * Description: Treebolic Engine
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.component;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import treebolic.control.Controller;
import treebolic.model.INode;
import treebolic.model.MenuItem;
import treebolic.model.MountPoint;
import treebolic.model.Settings;

/**
 * Popup context menu
 * 
 * @author Bernard Bou
 */
public class PopupMenu extends JPopupMenu
{
	private static final long serialVersionUID = 1L;

	/**
	 * Action-Image map
	 */
	static Map<Controller.Action, ImageIcon> theIconMap = PopupMenu.makeIconMap();

	/**
	 * Make Action-Image map
	 * 
	 * @return Action-Image map
	 */
	static Map<Controller.Action, ImageIcon> makeIconMap()
	{
		final Map<Controller.Action, ImageIcon> thisMap = new HashMap<Controller.Action, ImageIcon>();
		thisMap.put(Controller.Action.SEARCH, new ImageIcon(PopupMenu.class.getResource("images/find.png")));
		thisMap.put(Controller.Action.GOTO, new ImageIcon(PopupMenu.class.getResource("images/goto.png")));
		thisMap.put(Controller.Action.FOCUS, new ImageIcon(PopupMenu.class.getResource("images/focus.png")));
		return thisMap;
	}

	/**
	 * Constructor
	 */
	protected PopupMenu()
	{
		// do nothing
	}

	/**
	 * Make popup menu
	 * 
	 * @param thisController
	 *        controller
	 * @param thisValue
	 *        input value
	 * @param thisNode
	 *        target node
	 * @param theseSettings
	 *        settings
	 * @return opoup menu
	 */
	static public JPopupMenu makePopup(final Controller thisController, final String thisValue, final INode thisNode, final Settings theseSettings)
	{
		final PopupMenu thisPopupMenu = new PopupMenu();

		// cancel
		final JMenuItem thisCancelMenuItem = new JMenuItem("Cancel");
		thisCancelMenuItem.setIcon(new ImageIcon(PopupMenu.class.getResource("images/cancel.png")));
		thisCancelMenuItem.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				// do nothing
			}
		});
		thisPopupMenu.add(thisCancelMenuItem);

		// mount
		final MountPoint thisMountPoint = thisNode.getMountPoint();
		if (thisMountPoint != null)
		{
			final boolean isMounted = thisMountPoint instanceof MountPoint.Mounted;
			final JMenuItem thisMountMenuItem = new JMenuItem(isMounted ? "UnMount" : "Mount");
			thisMountMenuItem.setIcon(new ImageIcon(PopupMenu.class.getResource("images/mount.png")));
			thisMountMenuItem.addActionListener(new ActionListener()
			{
				@Override
				public void actionPerformed(final ActionEvent e)
				{
					thisController.dispatch(Controller.Action.MOUNT, null, null, null, null, null, thisNode);
				}
			});
			thisPopupMenu.add(thisMountMenuItem);
		}

		// link
		if (thisNode.getLink() != null)
		{
			final JMenuItem thisLinkMenuItem = new JMenuItem("Link");
			thisLinkMenuItem.setIcon(new ImageIcon(PopupMenu.class.getResource("images/link.png")));
			thisLinkMenuItem.addActionListener(new ActionListener()
			{
				@Override
				public void actionPerformed(final ActionEvent e)
				{
					thisController.dispatch(Controller.Action.LINK, null, null, null, null, null, thisNode);
				}
			});
			thisPopupMenu.add(thisLinkMenuItem);
		}

		// custom
		if (theseSettings.theMenu != null)
		{
			for (final MenuItem thatMenuItem : theseSettings.theMenu)
			{
				try
				{
					final String thisMenuLabel = PopupMenu.expandPercent(thatMenuItem.theLabel, thisValue, thisNode);
					final Controller.Action thisAction = thatMenuItem.theAction;
					final String thisLink = thatMenuItem.theLink;
					final String thisTarget = thatMenuItem.theTarget;
					final String thisMatchTarget = thatMenuItem.theMatchTarget;
					final Controller.MatchScope thisMatchScope = thatMenuItem.theMatchScope;
					final Controller.MatchMode thisMatchMode = thatMenuItem.theMatchMode;

					final JMenuItem thisMenuItem = new JMenuItem(thisMenuLabel);
					thisMenuItem.setIcon(PopupMenu.theIconMap.get(thisAction));
					thisMenuItem.addActionListener(new ActionListener()
					{
						@Override
						public void actionPerformed(final ActionEvent e)
						{
							thisController.dispatch(thisAction, thisLink, thisTarget, thisMatchTarget, thisMatchScope, thisMatchMode, thisNode);
						}
					});
					thisPopupMenu.add(thisMenuItem);
				}
				catch (final ArrayIndexOutOfBoundsException e)
				{
					// do nothing
				}
			}
		}
		return thisPopupMenu;
	}

	// S T R I N G . E X P A N D

	/**
	 * Expand string
	 * 
	 * @param thisString
	 *        string to expand
	 * @param thisValue
	 *        interactively supplied value
	 * @param thisNode
	 *        node node
	 * @return expanded string
	 */
	static public String expandPercent(final String thisString, final String thisValue, final INode thisNode)
	{
		if (thisString == null)
			return "";

		final StringBuffer thisBuffer = new StringBuffer();
		final int n = thisString.length();
		for (int i = 0; i < n; i++)
		{
			final char c = thisString.charAt(i);
			if (c != '%')
			{
				thisBuffer.append(c);
			}
			else
			{
				if (i < n - 1)
				{
					// c is not last : peek at next
					final char c2 = thisString.charAt(++i);
					switch (c2)
					{

					// label
					case 'l':
						final String thisLabel = thisNode.getLabel();
						if (thisLabel != null)
						{
							thisBuffer.append(thisLabel.toCharArray());
						}
						break;

					// content
					case 'c':
						final String thisContent = thisNode.getContent();
						if (thisContent != null)
						{
							thisBuffer.append(thisContent.toCharArray());
						}
						break;

					// link url
					case 'u':
						final String thisLink = thisNode.getLink();
						if (thisLink != null)
						{
							thisBuffer.append(thisLink.toCharArray());
						}
						break;

					// id
					case 'i':
						final String thisId = thisNode.getId();
						if (thisId != null)
						{
							thisBuffer.append(thisId.toCharArray());
						}
						break;

					// parent
					case 'p':
						final INode thisParent = thisNode.getParent();
						if (thisParent != null)
						{
							final String thisParentId = thisParent.getId();
							if (thisParentId != null)
							{
								thisBuffer.append(thisParentId.toCharArray());
							}
						}
						break;

					// element value
					case 'e':
						if (thisValue != null)
						{
							thisBuffer.append(thisValue.toCharArray());
						}
						break;

					// escaped %
					case '%':
						thisBuffer.append(c2);
						break;

					// unrecognized
					default:
						thisBuffer.append(c);
						thisBuffer.append(c2);
					}
				}
				else
				{
					// is last : copy
					thisBuffer.append(c);
				}
			}
		}
		return thisBuffer.toString();
	}
}
