/**
 * Title: Hyperbolic engine
 * Description: Treebolic Engine
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.component;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

/**
 * Progress panel
 * 
 * @author Bernard Bou
 */
public class Progress extends JPanel
{
	private static final long serialVersionUID = 1L;

	/**
	 * Stop icon
	 */
	static private Icon theStopIcon = new ImageIcon(Progress.class.getResource("images/stop.png"));

	/**
	 * Working icon
	 */
	static private Icon theWorkingIcon = new ImageIcon(Progress.class.getResource("images/wait.gif"));

	/**
	 * Progress bar
	 */
	private final JProgressBar theProgress;

	/**
	 * Label
	 */
	private final JLabel theLabel;

	/**
	 * Label
	 */
	private final JLabel theText;

	/**
	 * Constructor
	 */
	public Progress()
	{
		super();

		setBackground(Color.WHITE);

		this.theProgress = new JProgressBar(0, 100);
		this.theProgress.setMaximumSize(new Dimension(200, 32));
		this.theProgress.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.theProgress.setStringPainted(true);
		this.theProgress.setVisible(false);

		this.theLabel = new JLabel();
		this.theLabel.setText("Treebolic");
		this.theLabel.setFont(new Font("Dialog", Font.BOLD, 20));
		this.theLabel.setPreferredSize(new Dimension(300, 80));
		this.theLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.theLabel.setHorizontalAlignment(SwingConstants.CENTER);
		this.theLabel.setVerticalTextPosition(SwingConstants.TOP);
		this.theLabel.setHorizontalTextPosition(SwingConstants.CENTER);

		this.theText = new JLabel();
		this.theText.setText("starting");
		this.theText.setFont(new Font("Dialog", Font.PLAIN, 10));
		this.theText.setPreferredSize(new Dimension(300, 32));
		this.theText.setAlignmentX(Component.CENTER_ALIGNMENT);
		this.theText.setBackground(Color.BLUE);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(Box.createVerticalGlue());
		add(this.theLabel);
		add(this.theText);
		add(this.theProgress);
		add(Box.createVerticalGlue());
	}

	/**
	 * Put message
	 * 
	 * @param thisMessage
	 *        message
	 */
	public void put(final String thisMessage, final boolean fail)
	{
		final Runnable thisRoutine = new Runnable()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void run()
			{
				Progress.this.theText.setText(thisMessage);
				Progress.this.theLabel.setIcon(fail ? Progress.theStopIcon : Progress.theWorkingIcon);
				Progress.this.theProgress.setValue(fail ? 0 : Progress.this.theProgress.getValue() + 10);
				Progress.this.theProgress.setVisible(Progress.this.theProgress.getValue() > 0);
			}
		};
		SwingUtilities.invokeLater(thisRoutine);
	}
}
