/**
 * Title: Hyperbolic engine
 * Description: Treebolic Engine
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.component;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import treebolic.model.Settings;

/**
 * Status bar
 * 
 * @author Bernard Bou
 */
public class Statusbar extends JToolBar
{
	private static final long serialVersionUID = 1L;

	/**
	 * Put type
	 */
	public static enum PutType
	{
		INFO, LINK
	}

	/**
	 * Content panel increment
	 */
	static private final int theContentIncrement = 20;

	/**
	 * Content panel increment
	 */
	static private final int theContentMinimum = 20;

	/**
	 * Content panel increment
	 */
	static private final int theContentMaximum = 200;

	// D A T A

	// component
	/**
	 * Label
	 */
	private JTextField theLabelTextField;

	/**
	 * Input
	 */
	private JTextField theInputTextField;

	/**
	 * Content
	 */
	private JTextPane theContentEditor;

	/**
	 * Content pane
	 */
	private JComponent theContentPane;

	/**
	 * Ouput icon label
	 */
	private JLabel theOutputIconLabel;

	/**
	 * Input icon label
	 */
	private JLabel theInputIconLabel;

	/**
	 * Toggle button
	 */
	private JToggleButton theToggleButton;

	/**
	 * Edit box
	 */
	private Box theEditBox;

	// images
	/**
	 * Images
	 */
	static private ImageIcon[] theImages = new ImageIcon[] { //
	new ImageIcon(Statusbar.class.getResource("images/info.png")), //
			new ImageIcon(Statusbar.class.getResource("images/linking.png")), //
	};

	// colors
	/**
	 * Label background color
	 */
	static private Color[] theBackColor = { Color.WHITE, new Color(0, 0, 128) };

	/**
	 * Label foreground color
	 */
	static private Color[] theForeColor = { Color.BLACK, Color.WHITE };

	// font
	/**
	 * Label font
	 */
	private Font theLabelFont;

	// style
	static private SimpleAttributeSet theContentStyle = new SimpleAttributeSet();
	static
	{
		StyleConstants.setFontFamily(Statusbar.theContentStyle, "Dialog");
		StyleConstants.setFontSize(Statusbar.theContentStyle, 10);
	}

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public Statusbar()
	{
		super(SwingConstants.HORIZONTAL);
		setLayout(new GridBagLayout());
		setFloatable(true);
		init(null);
	}

	/**
	 * Init
	 * 
	 * @param theseSettings
	 *        settings
	 */
	public void init(final Settings theseSettings)
	{
		// colors
		if (theseSettings != null)
		{
			if (theseSettings.theBackColor != null)
			{
				Statusbar.theBackColor[PutType.INFO.ordinal()] = theseSettings.theBackColor.brighter();
				Statusbar.theBackColor[PutType.LINK.ordinal()] = theseSettings.theBackColor.darker();
			}
			if (theseSettings.theForeColor != null)
			{
				Statusbar.theForeColor[PutType.INFO.ordinal()] = theseSettings.theForeColor.darker();
				Statusbar.theForeColor[PutType.LINK.ordinal()] = theseSettings.theForeColor.brighter();
			}
		}

		// font
		final String thisFontFace = theseSettings == null || theseSettings.theFontFace == null || theseSettings.theFontFace.isEmpty() ? "Dialog" : theseSettings.theFontFace;
		int thisFontSize = theseSettings == null ? 12 : theseSettings.theFontSize;
		if (thisFontSize > 18)
		{
			thisFontSize = 18;
		}
		if (thisFontSize < 14)
		{
			thisFontSize = 12;
		}
		this.theLabelFont = new Font(thisFontFace, Font.PLAIN, thisFontSize);

		// toggle button
		this.theToggleButton = new JToggleButton();
		this.theToggleButton.setIcon(new ImageIcon(Statusbar.class.getResource("images/editstart.png")));
		this.theToggleButton.setSelectedIcon(new ImageIcon(Statusbar.class.getResource("images/editstop.png")));
		this.theToggleButton.setOpaque(true);
		this.theToggleButton.setBorder(null);
		this.theToggleButton.setToolTipText("Locks status bar and switches to edit mode");
		this.theToggleButton.addItemListener(new ItemListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void itemStateChanged(final ItemEvent e)
			{
				final boolean isSelected = e.getStateChange() == ItemEvent.SELECTED;
				if (isSelected)
				{
					Statusbar.this.theEditBox.add(Statusbar.this.theInputIconLabel);
					Statusbar.this.theEditBox.add(Statusbar.this.theInputTextField);
				}
				else
				{
					Statusbar.this.theEditBox.remove(Statusbar.this.theInputIconLabel);
					Statusbar.this.theEditBox.remove(Statusbar.this.theInputTextField);
				}
				Statusbar.this.theEditBox.validate();
				Statusbar.this.theOutputIconLabel.setIcon(Statusbar.theImages[PutType.INFO.ordinal()]);
			}
		});

		// enlarge/shrink buttons
		final JButton thisMoreButton = new JButton();
		thisMoreButton.setIcon(new ImageIcon(Statusbar.class.getResource("images/plus.png")));
		thisMoreButton.setBorder(null);
		thisMoreButton.setContentAreaFilled(false);
		thisMoreButton.setFocusable(false);
		thisMoreButton.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		thisMoreButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final Dimension thisSize = Statusbar.this.theContentPane.getPreferredSize();
				final int h = thisSize.height + Statusbar.theContentIncrement;
				if (h <= Statusbar.theContentMaximum)
				{
					thisSize.height = h;
					Statusbar.this.theContentPane.setPreferredSize(thisSize);
					getParent().validate();
					getParent().repaint();
				}
			}
		});
		final JButton thisLessButton = new JButton();
		thisLessButton.setIcon(new ImageIcon(Statusbar.class.getResource("images/minus.png")));
		thisLessButton.setBorder(null);
		thisLessButton.setContentAreaFilled(false);
		thisLessButton.setFocusable(false);
		thisLessButton.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		thisLessButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final Dimension thisSize = Statusbar.this.theContentPane.getPreferredSize();
				final int h = thisSize.height - Statusbar.theContentIncrement;
				if (h >= Statusbar.theContentMinimum)
				{
					thisSize.height = h;
					Statusbar.this.theContentPane.setPreferredSize(thisSize);
					getParent().validate();
					getParent().repaint();
				}
			}
		});

		// button box
		final Box thisButtonBox = new Box(BoxLayout.Y_AXIS);
		thisButtonBox.add(thisMoreButton);
		thisButtonBox.add(thisLessButton);

		// output
		this.theOutputIconLabel = new JLabel();
		this.theOutputIconLabel.setIcon(Statusbar.theImages[PutType.INFO.ordinal()]);
		this.theLabelTextField = new JTextField();
		this.theLabelTextField.setEditable(false);
		this.theLabelTextField.setBackground(Color.WHITE);
		this.theLabelTextField.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		this.theLabelTextField.setFont(this.theLabelFont);
		this.theLabelTextField.setToolTipText("Displays node information");

		// input
		this.theInputIconLabel = new JLabel();
		this.theInputIconLabel.setIcon(new ImageIcon(Statusbar.class.getResource("images/edit.png")));
		this.theInputTextField = new JTextField();
		this.theInputTextField.setEditable(true);
		this.theInputTextField.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		this.theInputTextField.setToolTipText("Input information");

		// content
		this.theContentEditor = new JTextPane();
		this.theContentEditor.setContentType("text/html; charset=UTF-8");
		this.theContentEditor.setEditable(false);
		this.theContentEditor.setBorder(null);
		this.theContentEditor.setFont(new Font("Dialog", Font.PLAIN, 8));
		this.theContentEditor.setToolTipText("Displays node content");
		this.theContentEditor.setPreferredSize(new Dimension(400, 30));

		this.theContentPane = new JScrollPane(this.theContentEditor);
		this.theContentPane.setBorder(BorderFactory.createLineBorder(Color.GRAY));

		// edit box
		this.theEditBox = new Box(BoxLayout.X_AXIS);
		this.theEditBox.add(this.theLabelTextField);

		// assemble
		add(this.theOutputIconLabel, new GridBagConstraints(0, 0, 1, 2, 0.0, 1.0, GridBagConstraints.NORTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 10), 0, 0));
		add(this.theEditBox, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.NORTH, GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));
		add(this.theContentPane, new GridBagConstraints(1, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 5, 0), 0, 0));
		add(this.theToggleButton, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		add(thisButtonBox, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.SOUTH, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
	}

	/**
	 * Set colors
	 * 
	 * @param thisType
	 *        information type
	 */
	private void setColors(final PutType thisType)
	{
		final Color thisBackColor = Statusbar.theBackColor[thisType.ordinal()];
		setBackground(thisBackColor);

		this.theEditBox.setBackground(thisBackColor);
		this.theLabelTextField.setBackground(thisBackColor);
		this.theInputTextField.setBackground(thisBackColor);
		this.theContentEditor.setBackground(thisBackColor);

		final Color thisForeColor = Statusbar.theForeColor[thisType.ordinal()];
		setForeground(thisForeColor);

		this.theEditBox.setForeground(thisForeColor);
		this.theLabelTextField.setForeground(thisForeColor);
		this.theInputTextField.setForeground(thisForeColor);
		this.theContentEditor.setForeground(thisForeColor);

		StyleConstants.setForeground(Statusbar.theContentStyle, thisForeColor);
	}

	/**
	 * Put status
	 * 
	 * @param thisLabel
	 *        label
	 * @param thisContent
	 *        content
	 * @param thisType
	 *        status type as per below
	 */
	public void put(final String thisLabel, final String thisContent, final PutType thisType)
	{
		// icon and colors
		this.theOutputIconLabel.setIcon(Statusbar.theImages[thisType.ordinal()]);
		setColors(thisType);

		// label
		this.theLabelTextField.setText(thisLabel);

		// content
		this.theContentEditor.setText(thisContent);
		this.theContentEditor.setCaretPosition(0);

		// style
		final StyledDocument thisStyledDocument = this.theContentEditor.getStyledDocument();
		thisStyledDocument.setCharacterAttributes(0, thisStyledDocument.getLength(), Statusbar.theContentStyle, false);
	}

	/**
	 * Put status
	 * 
	 * @param thisMessage
	 *        content
	 */
	public void put(final String thisMessage)
	{
		this.theContentEditor.setText(thisMessage);
		this.theContentEditor.setCaretPosition(0);
	}

	/**
	 * Get input
	 * 
	 * @return input
	 */
	public String get()
	{
		return this.theInputTextField.getText();
	}
}
