/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.dialogs.main;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import treebolic.Messages;
import treebolic.commons.FileDialogs;
import treebolic.commons.Persist;
import treebolic.commons.SettingsDialog;

/**
 * Settings dialog
 * 
 * @author Bernard Bou
 */
public class XSettingsDialog extends SettingsDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Repository text
	 */
	private JTextField theRepositoryTextField;

	/**
	 * Constructor
	 * 
	 * @param theseProperties
	 */
	public XSettingsDialog(final Properties theseProperties)
	{
		super(theseProperties);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.commons.SettingsDialog#initialize()
	 */
	@Override
	protected void initialize()
	{
		super.initialize();
		this.theRepositoryTextField = new JTextField(32);
		this.theRepositoryTextField.setToolTipText(Messages.getString("XSettingsDialog.2")); //$NON-NLS-1$
		final JLabel theRepositoryFolderLabel = new JLabel(Messages.getString("XSettingsDialog.1")); //$NON-NLS-1$
		final JButton thisRepositoryBrowseButton = new JButton(Messages.getString("XSettingsDialog.3")); //$NON-NLS-1$
		thisRepositoryBrowseButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent event)
			{
				final String thisFolder = FileDialogs.getFolder(XSettingsDialog.this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisFolder != null && !thisFolder.isEmpty())
				{
					XSettingsDialog.this.theRepositoryTextField.setText(thisFolder);
				}
				else
				{
					XSettingsDialog.this.theRepositoryTextField.setText(Messages.getString("XSettingsDialog.10")); //$NON-NLS-1$
				}
			}
		});

		this.theDataPanel.add(theRepositoryFolderLabel, new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0));
		this.theDataPanel.add(this.theRepositoryTextField, new GridBagConstraints(1, 2, 1, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		this.theDataPanel.add(thisRepositoryBrowseButton, new GridBagConstraints(2, 2, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			// read properties into components
			this.theRepositoryTextField.setText(this.theSettings.getProperty(Messages.getString("XSettingsDialog.11"), Messages.getString("XSettingsDialog.10"))); //$NON-NLS-1$ //$NON-NLS-2$
		}
		else
		{
			if (this.ok)
			{
				// update properties from components
				final String thisRepository = this.theRepositoryTextField.getText();
				if (thisRepository == null || thisRepository.isEmpty() || thisRepository.equals(Messages.getString("XSettingsDialog.10")))
				{
					this.theSettings.remove(Messages.getString("XSettingsDialog.11")); //$NON-NLS-1$
				}
				else
				{
					this.theSettings.setProperty(Messages.getString("XSettingsDialog.11"), thisRepository); //$NON-NLS-1$
				}
			}
		}
		super.setVisible(thisFlag);
	}

	static public void main(final String[] theseArgs)
	{
		UIManager.put("swing.boldMetal", false); //$NON-NLS-1$
		final Properties theseSettings = Persist.getSettings("treebolic-generator"); //$NON-NLS-1$
		final XSettingsDialog thisDialog = new XSettingsDialog(theseSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		if (thisDialog.ok)
		{
			Persist.saveSettings("treebolic-generator", theseSettings); //$NON-NLS-1$
		}
		System.exit(0);
	}
}
