/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.dtd;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.URL;

/**
 * DTD
 * 
 * @author Bernard Bou
 */
public class Dtd
{
	/**
	 * DTD file name
	 */
	static final String theDTDFile = "Treebolic.dtd"; //$NON-NLS-1$

	/**
	 * Get DTD as string
	 * 
	 * @return DTD as string
	 */
	static public String getString()
	{
		final ByteArrayOutputStream thisByteArrayOutputStream = new ByteArrayOutputStream();
		final BufferedOutputStream thisOutputStream = new BufferedOutputStream(thisByteArrayOutputStream);
		if (Dtd.copyToStream(thisOutputStream))
			return thisByteArrayOutputStream.toString();
		return null;
	}

	/**
	 * Copy DTD to output stream
	 * 
	 * @param thisOutputStream
	 *        output stream
	 * @return true if successful
	 */
	static public boolean copyToStream(final OutputStream thisOutputStream)
	{
		final URL thisURL = Dtd.class.getResource(Dtd.theDTDFile);
		if (thisURL == null)
			return false;
		try
		{
			final InputStream thisInputStream = thisURL.openStream();
			return Dtd.copyStreams(thisInputStream, thisOutputStream);
		}
		catch (final IOException thisException)
		{
			System.err.println("Can't find " + thisURL + " " + thisException.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			return false;
		}
	}

	/**
	 * Copy DTD to UTF8 output stream
	 * 
	 * @param thisOutstream
	 *        output stream
	 * @return true if successful
	 */
	static public boolean copyToUTF8Stream(final OutputStream thisOutstream)
	{
		final String thisString = Dtd.getString();
		if (thisString == null)
			return false;
		try
		{
			final PrintWriter theWriter = new PrintWriter(new OutputStreamWriter(thisOutstream, "UTF8"), false); //$NON-NLS-1$
			theWriter.print(thisString);
			theWriter.close();
			return true;
		}
		catch (final Exception e)
		{
			// do nothing
		}
		return false;
	}

	/**
	 * Copy input stream to output stream
	 * 
	 * @param thisInstream
	 *        input stream
	 * @param thisOutstream
	 *        output stream
	 * @return true if successful
	 */
	static private boolean copyStreams(final InputStream thisInstream, final OutputStream thisOutstream)
	{
		try
		{
			final int thisBufferCount = 512;
			final BufferedInputStream theReader = new BufferedInputStream(thisInstream, thisBufferCount);
			final BufferedOutputStream theWriter = new BufferedOutputStream(thisOutstream);
			final byte thisBuffer[] = new byte[thisBufferCount];
			int thisCount;
			while ((thisCount = theReader.read(thisBuffer, 0, thisBufferCount)) != -1)
			{
				theWriter.write(thisBuffer, 0, thisCount);
			}
			theReader.close();
			theWriter.close();
			return true;
		}
		catch (final IOException unused)
		{
			// do nothing
		}
		return false;
	}
}
