/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator;

import java.awt.Component;
import java.awt.Image;
import java.awt.Insets;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.WindowConstants;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.JTextComponent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import treebolic.IContext;
import treebolic.IWidget;
import treebolic.Messages;
import treebolic.commons.ButtonTabComponent;
import treebolic.commons.FileDialogs;
import treebolic.commons.Help;
import treebolic.commons.OpenDialog;
import treebolic.commons.Persist;
import treebolic.commons.XslExportDialog;
import treebolic.commons.XslImportDialog;
import treebolic.dialogs.main.XSettingsDialog;
import treebolic.dialogs.main.XSiteDialog;
import treebolic.dialogs.main.XUrlDialog;
import treebolic.dtd.Dtd;
import treebolic.generator.domtree.DomTreeView;
import treebolic.generator.model.ModelUtils;
import treebolic.generator.tree.TreeAdapter;
import treebolic.generator.tree.TreeView;
import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.MutableEdge;
import treebolic.model.MutableNode;
import treebolic.model.Settings;
import treebolic.model.Tree;
import treebolic.model.TreeMutableEdge;
import treebolic.model.TreeMutableNode;
import treebolic.provider.xml.dom.DocumentAdapter;
import treebolic.provider.xml.dom.Parser;
import treebolic.site.MakeSiteDialog;
import treebolic.site.RunSiteDialog;
import bsys.propertyview.PickListener;
import bsys.xml.transformer.DomTransformer;

/**
 * Controller
 * 
 * @author Bernard Bou
 */
public class Controller implements IContext, IProviderContext, PickListener, ChangeListener, CellEditorListener, HyperlinkListener
{
	// T R E E B O L I C . W I D G E T

	/**
	 * Treebolic widget
	 */
	private IWidget theWidget;

	/**
	 * Treebolic model
	 */
	public Model theModel;

	/**
	 * Id to node map
	 */
	private Map<String, MutableNode> theIdToNodeMap;

	// D O C U M E N T

	/**
	 * Document
	 */
	private Document theDocument;

	/**
	 * Document url
	 */
	private URL theUrl;

	/**
	 * Document mode
	 */
	private Mode theMode;

	/**
	 * XML validate
	 */
	static public boolean validate = true;

	// T R E E

	/**
	 * Tree model
	 */
	public TreeModel theTreeModel;

	/**
	 * Tree view
	 */
	private TreeView theTreeView;

	// P R O P E R T Y . V I E W

	private PropertyView thePropertyView;

	// O T H E R . V I E W S

	/**
	 * Text view
	 */
	private TextView theTextView;

	/**
	 * Dom tree view
	 */
	private DomTreeView theDomTreeView;

	// F R A M E

	/**
	 * Frame
	 */
	private JFrame theFrame;

	// T A B B E D P A N E

	/**
	 * Tabbed pane
	 */
	private JTabbedPane theTabbedPane;

	/**
	 * Tabbed pane selection index
	 */
	private int theSelectedTabIndex;

	// C O M M A N D L I N E

	/**
	 * Command line arguments
	 */
	private final String[] theArgs;

	// C O M M A N D . C O D E S

	/**
	 * Command code
	 */
	static public enum Code
	{
		NEW, OPEN, OPENURL, IMPORTXSL, IMPORTPROVIDER, //
		EXPORT, SAVE, SAVEAS, NEWELEMENT, REMOVEELEMENT, //
		EXPORTSETTINGS, //
		NORMALIZEIDS, LISTIMAGES, LISTLINKS, LISTMOUNTS, LISTIDS, //
		MAKESITE, RUNSITE, //
		SETTINGS, SETTINGSBASE, SETTINGSURL, //
		UPDATE, //
		OPTIONTREEBOLICRENDERER, OPTIONVALIDATEXML, OPTIONFOCUSPARENT, //
		EXPANDTREE, COLLAPSETREE, //
		SELECTTOP, SELECTTREE, SELECTNODES, SELECTEDGES, //
		DTD, //
		ABOUT, HELP
	};

	// O P E N . M O D E S

	/**
	 * Command code
	 */
	static public enum Mode
	{
		NEW, OPEN, IMPORT
	};

	// U P D A T E . R O U T I N E S

	/**
	 * Map component to update action
	 */
	public Map<Component, Runnable> theUpdateMap;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 * 
	 * @param theseArgs
	 *        command line arguments
	 */
	public Controller(final String[] theseArgs)
	{
		this.theArgs = theseArgs;
		this.theDocument = null;
		this.theUrl = null;
		this.theMode = null;
		this.theSettings = Persist.getSettings("treebolic-generator"); //$NON-NLS-1$
		this.theUpdateMap = new HashMap<Component, Runnable>();
	}

	// C O N N E C T

	/**
	 * Connect to frame
	 * 
	 * @param thisFrame
	 *        frame
	 */
	public void connect(final JFrame thisFrame)
	{
		this.theFrame = thisFrame;
	}

	/**
	 * Connect to tree view
	 * 
	 * @param thisTreeView
	 *        treeview
	 */
	public void connect(final TreeView thisTreeView)
	{
		this.theTreeView = thisTreeView;
	}

	/**
	 * Connect to property view
	 * 
	 * @param thisPropertyView
	 *        property view
	 */
	public void connect(final PropertyView thisPropertyView)
	{
		this.thePropertyView = thisPropertyView;
	}

	/**
	 * Connect to text view
	 * 
	 * @param thisTextView
	 *        text view
	 */
	public void connect(final TextView thisTextView)
	{
		this.theTextView = thisTextView;
	}

	/**
	 * Connect to Dom tree view
	 * 
	 * @param thisDomTreeView
	 *        DOM tree view
	 */
	public void connect(final DomTreeView thisDomTreeView)
	{
		this.theDomTreeView = thisDomTreeView;
	}

	/**
	 * Connect to widget
	 * 
	 * @param thisWidget
	 *        widget
	 */
	public void connect(final IWidget thisWidget)
	{
		this.theWidget = thisWidget;
	}

	/**
	 * Connect to tabbed pane
	 * 
	 * @param thisTabbedPane
	 *        tabbed pane
	 */
	public void connect(final JTabbedPane thisTabbedPane)
	{
		this.theTabbedPane = thisTabbedPane;
		this.theSelectedTabIndex = this.theTabbedPane.getSelectedIndex();
	}

	// D O C U M E N T

	/**
	 * Make document
	 * 
	 * @return document
	 */
	private Document makeDocument(final Model thisModel)
	{
		return ModelToDocumentTransformer.transform(thisModel);
	}

	/**
	 * Set document
	 * 
	 * @param thisDocument
	 *        document
	 * @param thisURL
	 *        url
	 */
	private void setDocument(final Document thisDocument, final URL thisURL)
	{
		this.theDocument = thisDocument;
		this.theUrl = this.theDocument == null ? null : thisURL;
		this.theModel = null;
		this.theIdToNodeMap = null;

		if (this.theDocument == null)
			return;

		// document -> model
		final DocumentAdapter thisAdapter = new DocumentAdapter()
		{
			@Override
			protected MutableNode makeNode(final MutableNode thisParent, final String thisId)
			{
				return new TreeMutableNode(thisParent, thisId);
			}

			@Override
			protected MutableEdge makeEdge(final MutableNode thisFromNode, final MutableNode thisToNode)
			{
				return new TreeMutableEdge(thisFromNode, thisToNode);
			}
		};
		this.theModel = thisAdapter.makeModel(this.theDocument);
		this.theIdToNodeMap = thisAdapter.getIdToNodeMap();
	}

	// M O D E L

	/**
	 * Get model
	 * 
	 * @return model
	 */
	public Model getModel()
	{
		return this.theModel;
	}

	/**
	 * Set model
	 * 
	 * @param thisModel
	 *        model
	 * @param thisIdToNodeMap
	 *        id to node map
	 */
	private void setModel(final Model thisModel, final Map<String, MutableNode> thisIdToNodeMap)
	{
		this.theDocument = null;
		this.theUrl = null;
		this.theModel = thisModel;
		this.theIdToNodeMap = thisIdToNodeMap;
	}

	// S E T T I N G S

	/**
	 * Settings
	 */
	private final Properties theSettings;

	// L I S T E N E R S

	// from tree

	/*
	 * (non-Javadoc)
	 * @see bsys.propertyview.PickListener#picked(java.lang.Object)
	 */
	@Override
	public void picked(final Object thisObject)
	{
		// relay to property view
		this.thePropertyView.picked(thisObject);
		// thePropertyView.requestFocus();
	}

	// from tab

	/*
	 * (non-Javadoc)
	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
	 */
	@Override
	public void stateChanged(final ChangeEvent thisEvent)
	{
		// on leaving tab 0
		if (this.theSelectedTabIndex == 0)
		{
			this.theDocument = makeDocument(this.theModel);
		}

		this.theSelectedTabIndex = this.theTabbedPane.getSelectedIndex();
		updateView();
	}

	// from property view

	/*
	 * (non-Javadoc)
	 * @see javax.swing.event.CellEditorListener#editingStopped(javax.swing.event.ChangeEvent)
	 */
	@Override
	public void editingStopped(final ChangeEvent thisEvent)
	{
		this.theTreeView.repaint();
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.event.CellEditorListener#editingCanceled(javax.swing.event.ChangeEvent)
	 */
	@Override
	public void editingCanceled(final ChangeEvent thisEvent)
	{
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.event.HyperlinkListener#hyperlinkUpdate(javax.swing.event.HyperlinkEvent)
	 */
	@Override
	public void hyperlinkUpdate(final HyperlinkEvent thisEvent)
	{
		if (thisEvent.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
		{
			final JEditorPane thisView = (JEditorPane) thisEvent.getSource();
			final URL thisURL = thisEvent.getURL();
			try
			{
				thisView.setPage(thisURL);
			}
			catch (final IOException e)
			{
				thisView.setText(Messages.getString("Controller.220") + thisEvent.getURL()); //$NON-NLS-1$
			}
		}
	}

	// E X E C U T E

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.IController#execute(int, int)
	 */
	public void execute(final Code thisCode, final int thisValue)
	{
		switch (thisCode)
		{
		case NEW:
			newDocument();
			break;

		case OPEN:
			open();
			break;

		case OPENURL:
			openUrl();
			break;

		case IMPORTXSL:
			importXsl();
			break;

		case IMPORTPROVIDER:
			importProvider();
			break;

		case SAVE:
			save();
			break;

		case SAVEAS:
			saveAs();
			break;

		case EXPORT:
			exportDocument();
			break;

		case EXPORTSETTINGS:
			exportSettings();
			break;

		case NEWELEMENT:
			newElement();
			break;

		case REMOVEELEMENT:
			removeElement();
			break;

		case NORMALIZEIDS:
			normalizeIds();
			break;

		case LISTIMAGES:
			listImages();
			break;

		case LISTLINKS:
			listLinks();
			break;

		case LISTMOUNTS:
			listMounts();
			break;

		case LISTIDS:
			listIds();
			break;

		case UPDATE:
			updateWidget();
			break;

		case MAKESITE:
			makeSite();
			break;

		case RUNSITE:
			runSite();
			break;

		case SETTINGS:
			settings();
			break;

		case SETTINGSBASE:
			base();
			break;

		case SETTINGSURL:
			url();
			break;

		case OPTIONTREEBOLICRENDERER:
			optionTreebolicRenderer();
			break;

		case OPTIONVALIDATEXML:
			optionValidateXml();
			break;

		case OPTIONFOCUSPARENT:
			optionFocusParent();
			break;

		case EXPANDTREE:
			this.theTreeView.expand();
			break;

		case COLLAPSETREE:
			this.theTreeView.collapse();
			break;

		case SELECTTOP:
			select("TopWrapper"); //$NON-NLS-1$
			break;

		case SELECTTREE:
			select("TreeWrapper"); //$NON-NLS-1$
			break;

		case SELECTNODES:
			select("NodesWrapper"); //$NON-NLS-1$
			break;

		case SELECTEDGES:
			select("EdgesWrapper"); //$NON-NLS-1$
			break;

		case ABOUT:
			about();
			break;

		case DTD:
			dtd();
			break;

		case HELP:
			help();
			break;

		default:
			System.err.println("Unhandled action: " + thisCode); //$NON-NLS-1$
			break;
		}
	}

	// C O M M A N D S

	/**
	 * Exit hook
	 */
	public void exit()
	{
		checkSave();
		Persist.saveSettings("treebolic-generator", this.theSettings); //$NON-NLS-1$
	}

	/**
	 * New command
	 */
	private void newDocument()
	{
		checkSave();
		final Model thisModel = new Model(new Tree(TreeMutableNode.makeDefaultTree(), null), new Settings());
		setModel(thisModel, ModelUtils.makeIdToNodeMap(thisModel));
		update(Mode.NEW);
	}

	/**
	 * Open
	 */
	private void open()
	{
		checkSave();
		open(FileDialogs.getXmlUrl(this.theSettings.getProperty("base", "."))); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * Open Http
	 */
	private void openUrl()
	{
		checkSave();
		final XUrlDialog thisDialog = new XUrlDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		if (thisDialog.ok)
		{
			open(this.theSettings.getProperty("openurl")); //$NON-NLS-1$
		}
	}

	/**
	 * Open Url string
	 * 
	 * @param thisUrlString
	 *        Url string
	 */
	public void open(final String thisUrlString)
	{
		final URL thisUrl = makeURL(thisUrlString);
		if (thisUrl == null)
			return;
		open(thisUrl);
	}

	/**
	 * Open Url
	 * 
	 * @param thisUrl
	 *        Url
	 */
	private void open(final URL thisUrl)
	{
		setDocument(new Parser(Controller.validate).makeDocument(thisUrl, makeEntityResolver()), thisUrl);
		update(Mode.OPEN);
	}

	/**
	 * Make entity resolver (for validation)
	 * 
	 * @return treebolic entity resolver
	 */
	protected EntityResolver makeEntityResolver()
	{
		return new EntityResolver()
		{
			@Override
			public InputSource resolveEntity(final String thisPublicId, final String thisSystemId) throws SAXException, IOException
			{
				final URL thisUrl = Dtd.class.getResource("Treebolic.dtd"); //$NON-NLS-1$
				return new InputSource(thisUrl.openStream());
			}
		};
	}

	/**
	 * Get file from Url
	 * 
	 * @param thisUrl
	 *        Url
	 * @return file
	 */
	static private File getFile(final URL thisUrl)
	{
		try
		{
			return new File(thisUrl.toURI());
		}
		catch (final URISyntaxException thisException)
		{
			// do nothing
		}
		return null;
	}

	/**
	 * Get file from Url
	 * 
	 * @param thisUrlString
	 *        url string
	 * @return file
	 */
	static private File getFile(final String thisUrlString)
	{
		try
		{
			return Controller.getFile(new URL(thisUrlString));
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
		return null;
	}

	/**
	 * Save
	 */
	private void save()
	{
		if (this.theMode != Mode.OPEN || this.theUrl == null)
		{
			saveAs();
			return;
		}
		final File thisFile = Controller.getFile(this.theUrl);
		save(thisFile);
	}

	/**
	 * Save as
	 */
	private void saveAs()
	{
		final String thisFilePath = FileDialogs.getXml(this.theSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
		if (thisFilePath == null)
			return;
		final File thisFile = new File(thisFilePath);
		if (thisFile.exists() && !confirm(new String[] { thisFilePath, Messages.getString("Controller.40"), Messages.getString("Controller.41") })) //$NON-NLS-1$ //$NON-NLS-2$
			return;
		save(thisFile);
	}

	/**
	 * Save as file
	 * 
	 * @param thisFile
	 *        file
	 */
	private void save(final File thisFile)
	{
		if (thisFile != null)
		{
			try
			{
				this.theDocument = makeDocument(this.theModel);
				if (this.theDocument != null)
				{
					new DomTransformer(false, "Treebolic.dtd").documentToFile(this.theDocument, thisFile); //$NON-NLS-1$
					this.theTreeView.dirty = false;
					this.thePropertyView.dirty = false;
				}
			}
			catch (final TransformerConfigurationException thisException)
			{
				System.err.println(Messages.getString("Controller.30") + thisFile + " " + thisException.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			}
			catch (final TransformerException thisException)
			{
				System.err.println(Messages.getString("Controller.30") + thisFile + " " + thisException.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			}
			catch (final IOException thisException)
			{
				System.err.println(Messages.getString("Controller.30") + thisFile + " " + thisException.toString()); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}

	/**
	 * Check if save is needed
	 */
	private void checkSave()
	{
		if (this.theTreeView.dirty || this.thePropertyView.dirty)
			if (confirm(new String[] { this.theUrl == null ? Messages.getString("Controller.60") : this.theUrl.toString(), Messages.getString("Controller.31"), Messages.getString("Controller.32") })) //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			{
				if (this.theMode != Mode.OPEN || this.theUrl == null)
				{
					saveAs();
				}
				else
				{
					save();
				}
			}
	}

	/**
	 * Import document through Xsl
	 */
	private void importXsl()
	{
		checkSave();
		final XslImportDialog thisDialog = new XslImportDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		if (thisDialog.ok)
		{
			final String thisXml = this.theSettings.getProperty("importurl"); //$NON-NLS-1$
			final String thisXslt = this.theSettings.getProperty("importxsl"); //$NON-NLS-1$
			if (thisXml != null && thisXslt != null)
			{
				this.theUrl = null;
				try
				{
					final URL thisXmlUrl = new URL(thisXml);
					final URL thisXsltUrl = new URL(thisXslt);
					setDocument(new Parser().makeDocument(thisXmlUrl, thisXsltUrl, null), thisXmlUrl);
					update(Mode.IMPORT);
				}
				catch (final MalformedURLException thisException)
				{
					thisException.printStackTrace();
				}
			}
		}
	}

	/**
	 * Import document from provider
	 */
	private void importProvider()
	{
		checkSave();
		final OpenDialog thisDialog = new OpenDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thisDialog.setVisible(true);
		if (thisDialog.ok)
		{
			final String thisSource = this.theSettings.getProperty("source"); //$NON-NLS-1$
			final String thisProviderName = this.theSettings.getProperty("provider"); //$NON-NLS-1$
			if (thisProviderName != null && !thisProviderName.isEmpty() && thisSource != null && !thisSource.isEmpty())
			{
				// make provider
				final IProvider thisProvider = makeProvider(thisProviderName);
				if (thisProvider == null)
				{
					final String[] theseLines = { Messages.getString("Controller.4"), thisProviderName, Messages.getString("Controller.250") }; //$NON-NLS-1$ //$NON-NLS-2$
					JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.3"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
					return;
				}
				thisProvider.connect(this);

				// make model
				final Model thisModel = thisProvider.makeModel(thisSource);
				if (thisModel == null)
				{
					final String[] theseLines = { Messages.getString("Controller.2"), thisProviderName, Messages.getString("Controller.242"), thisSource }; //$NON-NLS-1$ //$NON-NLS-2$
					JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.3"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
					return;
				}

				// make model mutable
				final Pair<Model, Map<String, MutableNode>> thisResult = ModelUtils.toMutable(thisModel);
				if (thisResult.theFirst == null)
				{
					final String[] theseLines = { Messages.getString("Controller.2"), thisProviderName, Messages.getString("Controller.241"), thisSource }; //$NON-NLS-1$ //$NON-NLS-2$
					JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.3"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
					return;
				}

				setModel(thisResult.theFirst, thisResult.theSecond);
				update(Mode.IMPORT);
			}
		}
	}

	/**
	 * Export document
	 */
	private void exportDocument()
	{
		final XslExportDialog thisDialog = new XslExportDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		if (thisDialog.ok)
		{
			final String thisExportUrl = this.theSettings.getProperty("exporturl"); //$NON-NLS-1$
			final String thisXsltUrl = this.theSettings.getProperty("exportxsl"); //$NON-NLS-1$
			if (thisExportUrl.startsWith("view:"))
			{
				transformToView(thisExportUrl, thisXsltUrl);
			}
			else
			{
				transformToFile(thisExportUrl, thisXsltUrl, thisExportUrl);
			}
		}
	}

	/**
	 * Export settings
	 */
	private void exportSettings()
	{
		final Settings theseSettings = this.theModel.theSettings;
		ModelUtils.saveSettings(theseSettings, FileDialogs.getPropertyFile(this.theSettings.getProperty("base", "."))); //$NON-NLS-1$ //$NON-NLS-2$
	}

	// E D I T I N G

	/**
	 * New element
	 */
	private void newElement()
	{
		this.theTreeView.editAdd();
	}

	/**
	 * Remove element
	 */
	private void removeElement()
	{
		this.theTreeView.editRemove();
	}

	/**
	 * Normalize ids
	 */
	private void normalizeIds()
	{
		final String thisPrefix = ask(Messages.getString("Controller.10"), "id"); //$NON-NLS-1$ //$NON-NLS-2$
		if (thisPrefix == null)
			return;
		this.theIdToNodeMap = ModelUtils.normalizeIds(this.theModel, thisPrefix);
		this.theDocument = makeDocument(this.theModel);
		updateView();
		this.theTreeView.dirty = true;
		this.thePropertyView.dirty = true;
	}

	// L I S T S

	/**
	 * List images
	 */
	private void listImages()
	{
		if (this.theModel == null)
			return;
		final ImageListDialog thisDialog = new ImageListDialog(this);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	/**
	 * List links
	 */
	private void listLinks()
	{
		if (this.theModel == null)
			return;
		final LinkListDialog thisDialog = new LinkListDialog(this);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	/**
	 * List mounts
	 */
	private void listMounts()
	{
		if (this.theModel == null)
			return;
		final MountListDialog thisDialog = new MountListDialog(this);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	/**
	 * List mounts
	 */
	private void listIds()
	{
		if (this.theModel == null)
			return;
		final IdListDialog thisDialog = new IdListDialog(this);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	// D T D

	/**
	 * Get Dtd
	 */
	public void dtd()
	{
		final TextView thisView = makeTextView();
		final JComponent thisComponent = new JScrollPane(thisView);
		this.theTabbedPane.addTab("DTD", null, thisComponent, null); //$NON-NLS-1$
		final int thisIndex = this.theTabbedPane.indexOfComponent(thisComponent);
		this.theTabbedPane.setTabComponentAt(thisIndex, new ButtonTabComponent(this.theTabbedPane));
		this.theTabbedPane.setSelectedIndex(thisIndex);

		final String thisText = Dtd.getString();
		thisView.setText(thisText);
		thisView.setCaretPosition(0);
	}

	// B E H A V I O U R

	/**
	 * Toggle treebolic rendering in Dom tree view
	 */
	private void optionTreebolicRenderer()
	{
		final TreeCellRenderer thatRenderer = this.theDomTreeView.getCellRenderer();
		final boolean isSpecific = thatRenderer instanceof treebolic.generator.domtree.treebolic.Renderer;
		final TreeCellRenderer thisRenderer = isSpecific ? new treebolic.generator.domtree.Renderer() : new treebolic.generator.domtree.treebolic.Renderer();
		this.theDomTreeView.setCellRenderer(thisRenderer);
		this.theDomTreeView.repaint();
	}

	/**
	 * Toggle XML validation
	 */
	private void optionValidateXml()
	{
		Controller.validate = !Controller.validate;
	}

	/**
	 * Toggle focus parent behaviour in tree view
	 */
	private void optionFocusParent()
	{
		treebolic.generator.tree.Tree.focusParent = !treebolic.generator.tree.Tree.focusParent;
	}

	// H E L P E R S

	/**
	 * Transform to view
	 */
	public void transformToView(final String thisExportUrl, final String thisXsltUrlString)
	{
		if (this.theDocument != null)
		{
			// type of view
			final boolean isHtml = thisExportUrl.endsWith("html"); //$NON-NLS-1$
			final JTextComponent thisView = isHtml ? makeHtmlView(this) : makeTextView();
			final JComponent thisComponent = new JScrollPane(thisView);

			// tab
			this.theTabbedPane.addTab(Messages.getString("Controller.20"), null, thisComponent, null); //$NON-NLS-1$
			final int thisIndex = this.theTabbedPane.indexOfComponent(thisComponent);
			this.theTabbedPane.setTabComponentAt(thisIndex, new ButtonTabComponent(this.theTabbedPane));
			this.theTabbedPane.setToolTipTextAt(thisIndex, thisXsltUrlString);
			this.theTabbedPane.setSelectedIndex(thisIndex);

			try
			{
				final URL thisXsltUrl = new URL(thisXsltUrlString);
				final DomTransformer thisTransformer = new DomTransformer(isHtml, null);
				final String thisText = thisTransformer.documentToString(this.theDocument, thisXsltUrl);
				thisView.setText(thisText);
				thisView.setCaretPosition(0);
				return;
			}
			catch (final TransformerConfigurationException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			catch (final TransformerException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			catch (final IOException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			return;
		}
		final String[] theseLines = { Messages.getString("Controller.50") }; //$NON-NLS-1$
		JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
	}

	/**
	 * Transform to file
	 */
	public void transformToFile(final String thisExportUrl, final String thisXsltUrlString, final String thisOutputFileUrl)
	{
		if (this.theDocument != null)
		{
			// type of view
			final boolean isHtml = thisExportUrl.endsWith("html"); //$NON-NLS-1$
			try
			{
				final File thisOutputFile = Controller.getFile(thisOutputFileUrl);
				if (thisOutputFile.exists() && !confirm(new String[] { thisOutputFileUrl, Messages.getString("Controller.40"), Messages.getString("Controller.41") })) //$NON-NLS-1$ //$NON-NLS-2$
					return;
				final URL thisXsltUrl = new URL(thisXsltUrlString);
				final DomTransformer thisTransformer = new DomTransformer(isHtml, null);
				thisTransformer.documentToFile(this.theDocument, thisXsltUrl, thisOutputFile);
			}
			catch (final TransformerConfigurationException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			catch (final TransformerException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			catch (final IOException thisException)
			{
				final String[] theseLines = { thisException.toString(), thisException.getMessage() };
				JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
			}
			return;
		}
		final String[] theseLines = { Messages.getString("Controller.50") }; //$NON-NLS-1$
		JOptionPane.showMessageDialog(null, theseLines, Messages.getString("Controller.0"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
	}

	// D I A L O G S

	/**
	 * Make site
	 */
	private void makeSite()
	{
		this.theDocument = makeDocument(this.theModel);
		if (this.theDocument != null)
		{
			final MakeSiteDialog thisDialog = new MakeSiteDialog(this.theSettings);
			thisDialog.setDocument(this.theDocument);
			thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			thisDialog.setVisible(true);
		}
	}

	/**
	 * Run site
	 */
	private void runSite()
	{
		final RunSiteDialog thisDialog = new RunSiteDialog(this.theSettings);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thisDialog.setVisible(true);
	}

	/**
	 * Persist
	 */
	private void settings()
	{
		final XSettingsDialog thisDialog = new XSettingsDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		if (thisDialog.ok)
		{
			treebolic.commons.Persist.saveSettings("treebolic-generator", this.theSettings); //$NON-NLS-1$
			this.thePropertyView.setImageRepository(makeImageRepositoryURL());
			updateView();
		}
	}

	/**
	 * Get base
	 */
	private void base()
	{
		final XSiteDialog thisDialog = new XSiteDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	/**
	 * Get Url
	 */
	private void url()
	{
		final XUrlDialog thisDialog = new XUrlDialog(this.theSettings);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
	}

	/**
	 * Help
	 */
	private void help()
	{
		Help.help(this.theSettings.getProperty("browser"), this.theSettings.getProperty("help")); //$NON-NLS-1$ //$NON-NLS-2$
	}

	/**
	 * About
	 */
	private void about()
	{
		new AboutDialog().setVisible(true);
	}

	/**
	 * Confirm
	 * 
	 * @param thisMessage
	 *        message
	 * @return true if confirmed
	 */
	private boolean confirm(final String[] thisMessage)
	{
		final int thisValue = JOptionPane.showConfirmDialog(null, thisMessage, Messages.getString("Controller.1"), JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
		return thisValue == JOptionPane.YES_OPTION;
	}

	/**
	 * Ask
	 * 
	 * @param thisMessage
	 *        message
	 * @return input
	 */
	private String ask(final String thisMessage, final String thisInitial)
	{
		final String[] theseLines = thisMessage.split("\n"); //$NON-NLS-1$
		return JOptionPane.showInputDialog(null, theseLines, thisInitial);
	}

	// S E L E C T

	/**
	 * Select tree node
	 * 
	 * @param thisClassName
	 *        class name of tree node to select
	 */
	private void select(final String thisClassName)
	{
		final DefaultMutableTreeNode thisNode = this.theTreeView.search(thisClassName);
		if (thisNode != null)
		{
			this.theTreeView.select(thisNode);
		}
	}

	// U P D A T E

	/**
	 * Update
	 * 
	 * @param thisMode
	 *        mode
	 */
	public void update(final Mode thisMode)
	{
		this.theMode = thisMode;

		// update view
		updateView();

		// title
		String thisTooltip = ""; //$NON-NLS-1$
		String thisShortTitle = ""; //$NON-NLS-1$
		switch (this.theMode)
		{
		case OPEN:
			if (this.theUrl != null)
			{
				thisTooltip = this.theUrl.toString();
				thisShortTitle = this.theUrl.getFile();
				final int thisIndex = thisShortTitle.lastIndexOf('/');
				if (thisIndex != -1)
				{
					thisShortTitle = thisShortTitle.substring(thisIndex + 1);
				}
				thisShortTitle = " - " + thisShortTitle; //$NON-NLS-1$
			}
			break;
		case NEW:
			thisTooltip = Messages.getString("Controller.80"); //$NON-NLS-1$
			thisShortTitle = Messages.getString("Controller.82"); //$NON-NLS-1$
			break;
		case IMPORT:
			thisTooltip = Messages.getString("Controller.83"); //$NON-NLS-1$
			thisShortTitle = Messages.getString("Controller.84"); //$NON-NLS-1$
			break;
		}
		for (int i = 0; i < 4; i++)
		{
			this.theTabbedPane.setToolTipTextAt(i, thisTooltip);
		}
		this.theFrame.setTitle(Messages.getString("Controller.1") + thisShortTitle); //$NON-NLS-1$
	}

	/**
	 * Update current view
	 */
	public void updateView()
	{
		final Component thisComponent = this.theTabbedPane.getSelectedComponent();
		final Runnable thisAction = this.theUpdateMap.get(thisComponent);
		if (thisAction != null)
		{
			thisAction.run();
		}
	}

	/**
	 * Update tree view
	 */
	public void updateTree()
	{
		// model -> tree model
		this.theTreeModel = this.theModel == null ? null : new TreeAdapter(this.theModel);

		// tree view
		this.theTreeView.set(this.theTreeModel);
		this.theTreeView.repaint();

		// property view
		this.thePropertyView.setIdToNodeMap(this.theIdToNodeMap);

		// reset dirty flags
		this.theTreeView.dirty = false;
		this.thePropertyView.dirty = false;

		// unselect
		picked(null);
	}

	/**
	 * Update widget view
	 */
	public void updateWidget()
	{
		this.theWidget.init(this.theModel);
	}

	/**
	 * Update xml text view
	 */
	public void updateText()
	{
		if (this.theDocument != null)
		{
			try
			{
				final String thisText = new DomTransformer(false, "Treebolic.dtd").documentToString(this.theDocument); //$NON-NLS-1$
				this.theTextView.setText(thisText);
				this.theTextView.setCaretPosition(0);
			}
			catch (final TransformerConfigurationException thisException)
			{
				// do nothing
			}
			catch (final TransformerException thisException)
			{
				// do nothing
			}
			catch (final IOException thisException)
			{
				// do nothing
			}
		}
	}

	/**
	 * Update Dom view
	 */
	public void updateDom()
	{
		if (this.theDocument != null)
		{
			this.theDomTreeView.setDocument(this.theDocument);
		}
	}

	// V I E W . F A C T O R Y

	/**
	 * Make the Html view
	 * 
	 * @return Html view
	 */
	private HtmlView makeHtmlView(final Controller thisController)
	{
		final HtmlView thisHtmlView = new HtmlView();
		thisHtmlView.addHyperlinkListener(thisController);
		return thisHtmlView;
	}

	/**
	 * Make the text view
	 * 
	 * @return text view
	 */
	private TextView makeTextView()
	{
		final TextView thisTextView = new TextView();
		thisTextView.setMargin(new Insets(10, 20, 10, 10));
		return thisTextView;
	}

	// U R L . F A C T O R Y

	/**
	 * Make image repository Url
	 * 
	 * @return image repository Url
	 */
	public URL makeImageRepositoryURL()
	{
		final String thisImageRepositoryPath = this.theSettings.getProperty("images"); //$NON-NLS-1$
		if (thisImageRepositoryPath != null)
		{
			File thisFolder = null;
			if (thisImageRepositoryPath.startsWith(Messages.getString("Controller.7")))
			{
				try
				{
					thisFolder = new File(new URI(thisImageRepositoryPath));
				}
				catch (final URISyntaxException thisException)
				{
					// do nothing
				}
			}
			else
			{
				thisFolder = new File(thisImageRepositoryPath);
			}
			if (thisFolder.exists() && thisFolder.isDirectory())
			{
				try
				{
					return thisFolder.toURI().toURL();
				}
				catch (final MalformedURLException thisException)
				{
					// do nothing
				}
			}
		}

		URL thisUrl;
		try
		{
			thisUrl = new URL(getBase(), "images"); //$NON-NLS-1$
			return thisUrl;
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
		return null;
	}

	/**
	 * Make based Url
	 * 
	 * @param thisSubPath
	 *        if not null this this the extra path to add to the base
	 * @param thisFilename
	 *        filename
	 * @return url
	 */
	private URL makeBasedURL(final String thisSubPath, final String thisFilename)
	{
		if (thisFilename == null)
			return null;
		try
		{
			return new URL(getBase(), (thisSubPath == null ? "" : thisSubPath + "/") + thisFilename); //$NON-NLS-1$ //$NON-NLS-2$
		}
		catch (final MalformedURLException e)
		{
			System.err.println(Messages.getString("Controller.210") + thisFilename); //$NON-NLS-1$
			return null;
		}
	}

	/**
	 * Make Url
	 * 
	 * @param thisSource
	 * @return url
	 * @see treebolic.model.IProviderContext#getParameter(java.lang.String)
	 */
	@Override
	public URL makeURL(final String thisSource)
	{
		if (thisSource == null)
			return null;

		// try to consider it well-formed full-fledged url
		try
		{
			return new URL(thisSource);
		}
		catch (final MalformedURLException e)
		{
			// do nothing
		}

		// try source relative to a base
		final URL thisBaseUrl = makeBasedURL(null, thisSource);
		if (thisBaseUrl != null)
			return thisBaseUrl;

		// try to consider it file
		final File thisFile = new File(thisSource);
		if (thisFile.exists() && thisFile.canRead())
		{
			try
			{
				return thisFile.toURI().toURL();
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
		}

		// fail
		return null;
	}

	// P R O V I D E R . F A C T O R Y

	/**
	 * Make provider
	 * 
	 * @param thisProviderName
	 *        provider name
	 * @return provider
	 */
	private IProvider makeProvider(final String thisProviderName)
	{
		try
		{
			final Class<?> thisClass = Class.forName(thisProviderName);
			final Class<?>[] theseArgsClass = new Class[] {};
			final Object[] theseArgs = new Object[] {};

			final Constructor<?> thisConstructor = thisClass.getConstructor(theseArgsClass);
			final Object thisInstance = thisConstructor.newInstance(theseArgs);
			return (IProvider) thisInstance;
		}
		catch (final ClassNotFoundException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		catch (final NoSuchMethodException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		catch (final IllegalAccessException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		catch (final InstantiationException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		catch (final IllegalArgumentException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		catch (final InvocationTargetException e)
		{
			System.err.println("Provider factory: " + e.toString()); //$NON-NLS-1$
		}
		return null;
	}

	// C O N T E X T

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getBase()
	 */
	@Override
	public URL getBase()
	{
		// base parameter
		final String thisBaseSetting = this.theSettings.getProperty("base"); //$NON-NLS-1$
		final String thisBaseParameter = getParameter("base"); //$NON-NLS-1$
		String thisURLString = thisBaseSetting != null ? thisBaseSetting : thisBaseParameter != null ? thisBaseParameter : System.getProperty("user.dir"); //$NON-NLS-1$

		// tail
		if (!thisURLString.endsWith("/"))
		{
			thisURLString += "/"; //$NON-NLS-1$
		}

		// make
		try
		{
			return new URL(thisURLString);
		}
		catch (final MalformedURLException e)
		{
			// make from folder
			try
			{
				final File thisFolder = new File(thisURLString);
				return thisFolder.toURI().toURL();
			}
			catch (final MalformedURLException e2)
			{
				// do nothing
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getHtmlDocumentBase()
	 */
	@Override
	public URL getHtmlDocumentBase()
	{
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getImagesBase()
	 */
	@Override
	public URL getImagesBase()
	{
		return makeImageRepositoryURL();
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#getImage(java.net.URL)
	 */
	@Override
	public Image getImage(final URL thisURL)
	{
		return java.awt.Toolkit.getDefaultToolkit().getImage(thisURL);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.Context#getParameter(java.lang.String)
	 * @see treebolic.model.IProviderContext#getParameter(java.lang.String)
	 */
	@Override
	public String getParameter(final String thisName)
	{
		// param1=<val> param2=<"val with spaces"> ...
		if (this.theArgs == null)
			return null;

		for (final String thisArg : this.theArgs)
			if (thisArg.startsWith(thisName + "=")) //$NON-NLS-1$
			{
				String thisValue = thisArg.substring(thisName.length() + 1);
				if (thisValue.startsWith("\"")) //$NON-NLS-1$
				{
					thisValue = thisValue.substring(1);
					if (thisValue.endsWith("\""))
					{
						thisValue = thisValue.substring(0, thisValue.length() - 1);
					}
				}
				return thisValue;
			}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#linkTo(java.lang.String)
	 */
	@Override
	public boolean linkTo(final String thisLinkUrl, final String thisUrlTarget)
	{
		System.out.println(Messages.getString("Controller.101") + thisLinkUrl + " , " + thisUrlTarget); //$NON-NLS-1$ //$NON-NLS-2$
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.component.Context#showStatus(java.lang.String)
	 */
	@Override
	public void showStatus(final String thisString)
	{
		System.out.println(Messages.getString("Controller.100") + thisString); //$NON-NLS-1$
	}

	// C O N T E X T

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProviderContext#putMessage(java.lang.String)
	 */
	@Override
	public void putMessage(final String thisString)
	{
		System.out.println(thisString);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProviderContext#putProgress(java.lang.String, boolean)
	 */
	@Override
	public void putProgress(final String thisString, final boolean thisFail)
	{
		System.out.println(thisString);
	}
}
