/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import treebolic.Messages;
import treebolic.generator.model.ModelUtils;

/**
 * Image list dialog
 * 
 * @author Bernard Bou
 */
public class ImageListDialog extends ReferenceListDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Image repository
	 */
	private URL theImageRepository;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public ImageListDialog(final Controller thisController)
	{
		super(thisController);
		setTitle(Messages.getString("ImageListDialog.0")); //$NON-NLS-1$
		this.theLabel.setText(Messages.getString("ImageListDialog.1")); //$NON-NLS-1$
		this.theReferenceTable.setRowHeight(32);
		this.theScrollPane.setPreferredSize(new Dimension(300, 320));

		final JButton thisCheckMissingButton = new JButton(Messages.getString("ImageListDialog.2")); //$NON-NLS-1$
		thisCheckMissingButton.addActionListener(new java.awt.event.ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				checkMissing();
			}
		});
		final JButton thisCheckUnusedButton = new JButton(Messages.getString("ImageListDialog.3")); //$NON-NLS-1$
		thisCheckUnusedButton.addActionListener(new java.awt.event.ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				checkUnused();
			}
		});
		this.theButtonPanel.add(thisCheckMissingButton, null);
		this.theButtonPanel.add(thisCheckUnusedButton, null);
	}

	// O V E R R I D E S

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.ReferenceListDialog#update()
	 */
	@Override
	protected void update()
	{
		this.theLabel.setText(Messages.getString("ImageListDialog.1")); //$NON-NLS-1$
		this.theImageRepository = this.theController.makeImageRepositoryURL();
		final Map<String, SortedSet<String>> thisImageToLocationMap = ModelUtils.getImageMap(this.theController.getModel());
		setModel(thisImageToLocationMap);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.ReferenceListDialog#setModel(java.util.Map)
	 */
	@Override
	protected void setModel(final Map<String, SortedSet<String>> thisTargetToLocationMap)
	{
		super.setModel(thisTargetToLocationMap);

		// renderer
		final TableColumn thisImageColumn = this.theReferenceTable.getColumnModel().getColumn(0);
		thisImageColumn.setWidth(150);
		thisImageColumn.setCellRenderer(new DefaultTableCellRenderer()
		{
			private static final long serialVersionUID = 1L;

			/*
			 * (non-Javadoc)
			 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
			 */
			@SuppressWarnings("synthetic-access")
			@Override
			public Component getTableCellRendererComponent(final JTable thisTable, final Object thisValue, final boolean thisIsSelected, final boolean thisHasFocus, final int thisRow, final int thisColumn)
			{
				final ParameterModel.Entry thisEntry = (ParameterModel.Entry) thisValue;
				final String thisImageFile = thisEntry.theKey;
				setIcon(makeIcon(thisImageFile));
				setText(thisImageFile);
				return this;
			}
		});

		final TableColumn thisLocationColumn = this.theReferenceTable.getColumnModel().getColumn(1);
		thisLocationColumn.setCellRenderer(new DefaultTableCellRenderer()
		{
			private static final long serialVersionUID = 1L;

			/*
			 * (non-Javadoc)
			 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
			 */
			@Override
			public Component getTableCellRendererComponent(final JTable thisTable, final Object thisValue, final boolean thisIsSelected, final boolean thisHasFocus, final int thisRow, final int thisColumn)
			{
				final ParameterModel.Entry thisEntry = (ParameterModel.Entry) thisValue;
				final String thisLocation = thisEntry.theValue;
				if (thisLocation.startsWith("default") || thisLocation.equals("background")) //$NON-NLS-1$ //$NON-NLS-2$
				{
					setBackground(Color.LIGHT_GRAY);
					setForeground(Color.WHITE);
				}
				else if (thisLocation.startsWith("node")) //$NON-NLS-1$
				{
					setBackground(Color.WHITE);
					setForeground(Color.BLUE);
				}
				else if (thisLocation.startsWith("treeedge") || thisLocation.startsWith("edge")) //$NON-NLS-1$ //$NON-NLS-2$
				{
					setBackground(Color.WHITE);
					setForeground(Color.BLACK);
				}
				else if (thisLocation.startsWith("unused")) //$NON-NLS-1$
				{
					setBackground(Color.WHITE);
					setForeground(Color.BLACK);
				}
				else
				{
					setBackground(Color.RED);
					setForeground(Color.WHITE);
				}

				setText(thisLocation);
				return this;
			}
		});
	}

	// H E L P E R S

	/**
	 * Check missing files
	 */
	private void checkMissing()
	{
		this.theImageRepository = this.theController.makeImageRepositoryURL();
		final Map<String, SortedSet<String>> thisImageToLocationMap = ModelUtils.getImageMap(this.theController.getModel());
		final Set<String> theseImages = thisImageToLocationMap.keySet();

		if (this.theImageRepository.getProtocol().equals("file")) //$NON-NLS-1$
		{
			try
			{
				final URI thisUri = this.theImageRepository.toURI();
				final File thisFile = new File(thisUri);
				if (thisFile.isDirectory())
				{
					for (final File thisDirectoryEntry : thisFile.listFiles())
					{
						final String thisName = thisDirectoryEntry.getName();
						if (theseImages.contains(thisName))
						{
							thisImageToLocationMap.remove(thisName);
						}
					}
					setModel(thisImageToLocationMap);
					this.theLabel.setText(Messages.getString("ImageListDialog.4") + thisUri.toString()); //$NON-NLS-1$
				}
			}
			catch (final URISyntaxException thisException)
			{
				// do nothing
			}
		}
	}

	/**
	 * Check unused files
	 */
	private void checkUnused()
	{
		this.theImageRepository = this.theController.makeImageRepositoryURL();
		final Map<String, SortedSet<String>> thisUnusedToLocationMap = new TreeMap<String, SortedSet<String>>();
		final Map<String, SortedSet<String>> thisImageToLocationMap = ModelUtils.getImageMap(this.theController.getModel());
		final Set<String> theseImages = thisImageToLocationMap.keySet();

		if (this.theImageRepository.getProtocol().equals("file")) //$NON-NLS-1$
		{
			try
			{
				final URI thisUri = this.theImageRepository.toURI();
				final File thisFile = new File(thisUri);
				if (thisFile.isDirectory())
				{
					for (final File thisDirectoryEntry : thisFile.listFiles())
					{
						if (thisDirectoryEntry.isDirectory())
						{
							continue;
						}
						final String thisName = thisDirectoryEntry.getName();
						if (!theseImages.contains(thisName))
						{
							final SortedSet<String> thisValue = new TreeSet<String>();
							thisValue.add(Messages.getString("ImageListDialog.6")); //$NON-NLS-1$
							thisUnusedToLocationMap.put(thisName, thisValue);
						}
					}
					setModel(thisUnusedToLocationMap);
					this.theLabel.setText(Messages.getString("ImageListDialog.5") + thisUri.toString()); //$NON-NLS-1$
				}
			}
			catch (final URISyntaxException thisException)
			{
				// do nothing
			}
		}
	}

	/**
	 * Make icon
	 * 
	 * @param thisImageFile
	 *        image file
	 * @return icon
	 */
	private Icon makeIcon(final String thisImageFile)
	{
		try
		{
			final URL thisUrl = new URL(this.theImageRepository, thisImageFile);
			return new ImageIcon(thisUrl);
		}
		catch (final MalformedURLException e)
		{
			return null;
		}
	}
}
