/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.TreeCellRenderer;

import treebolic.IContext;
import treebolic.IWidget;
import treebolic.Messages;
import treebolic.Widget;
import treebolic.generator.domtree.DomTreeView;
import treebolic.generator.tree.TreeView;

/**
 * Main frame
 * 
 * @author Bernard Bou
 */
/**
 * @author bbou
 */
/**
 * @author bbou
 */
public class MainFrame extends JFrame
{
	private static final long serialVersionUID = 1L;

	/**
	 * Tree icon
	 */
	static Icon theTreeIcon = new ImageIcon(MainFrame.class.getResource("images/treetab.png")); //$NON-NLS-1$

	/**
	 * Tree icon
	 */
	static Icon theTreebolicIcon = new ImageIcon(MainFrame.class.getResource("images/treebolictab.png")); //$NON-NLS-1$

	/**
	 * Rendering mode
	 */
	static public boolean hasTreebolicRendering = true;

	/**
	 * Property view
	 */
	private PropertyView thePropertyView;

	/**
	 * Treebolic widget
	 */
	private IWidget theWidget;

	/**
	 * Controller
	 */
	private final Controller theController;

	/**
	 * Tree view
	 */
	private TreeView theTreeView;

	/**
	 * Dom tree view
	 */
	private DomTreeView theDomTreeView;

	/**
	 * Text view
	 */
	private TextView theTextView;

	/**
	 * Tabbed pane
	 */
	private JTabbedPane theTabbedPane;

	/**
	 * Toolbar panel
	 */
	private JPanel theToolbarPanel;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public MainFrame(final String[] theseArgs)
	{
		super();
		this.theController = new Controller(theseArgs);

		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		initialize();

		// pack frame with preferred size data
		pack();

		// show frame
		setVisible(true);

		// open
		final String thisDocumentPath = this.theController.getParameter("doc"); //$NON-NLS-1$
		if (thisDocumentPath != null && !thisDocumentPath.isEmpty())
		{
			this.theController.open(thisDocumentPath);
		}
	}

	/**
	 * Make mainframe
	 */
	private void initialize()
	{
		this.theWidget = makeWidget(this.theController);
		this.theTreeView = makeTreeView(this.theController);
		this.thePropertyView = makePropertyView(this.theController);
		this.theTextView = makeTextView();
		this.theDomTreeView = makeDomTreeView();
		this.theToolbarPanel = makeToolbarPanel(this.theController);

		final JPanel thisContentPane = new JPanel();
		thisContentPane.setLayout(new BorderLayout());
		thisContentPane.add(makeTabbedPane(this.theController), BorderLayout.CENTER);
		thisContentPane.add(this.theToolbarPanel, BorderLayout.EAST);

		setJMenuBar(makeMenubar(this.theController));
		setContentPane(thisContentPane);
		setTitle(Messages.getString("MainFrame.0")); //$NON-NLS-1$

		// connect controller
		this.theController.connect(this.theWidget);
		this.theController.connect(this.thePropertyView);
		this.theController.connect(this.theTreeView);
		this.theController.connect(this.theTextView);
		this.theController.connect(this.theDomTreeView);
		this.theController.connect(this.theTabbedPane);
		this.theController.connect(this);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Window#processWindowEvent(java.awt.event.WindowEvent)
	 */
	@Override
	protected void processWindowEvent(final WindowEvent thisEvent)
	{
		if (thisEvent.getID() == WindowEvent.WINDOW_CLOSING)
		{
			this.theController.exit();
		}
		super.processWindowEvent(thisEvent);
	}

	// C O M P O N E N T S

	/**
	 * Make the menu bar
	 * 
	 * @return javax.swing.JMenuBar
	 */
	private Menubar makeMenubar(final Controller thisController)
	{
		final Menubar thisMenubar = new Menubar();
		thisMenubar.setController(thisController);
		return thisMenubar;
	}

	/**
	 * Make the toolbar space
	 * 
	 * @return javax.swing.JToolBar
	 */
	private JPanel makeToolbarPanel(final Controller thisController)
	{
		final JPanel thisToolbarPanel = new JPanel();
		thisToolbarPanel.setLayout(new BoxLayout(thisToolbarPanel, BoxLayout.Y_AXIS));
		thisToolbarPanel.add(makeToolbar(thisController));
		thisToolbarPanel.add(makeTreeToolbar(thisController));
		return thisToolbarPanel;
	}

	/**
	 * Make the toolbar
	 * 
	 * @return javax.swing.JToolBar
	 */
	private Toolbar makeToolbar(final Controller thisController)
	{
		final Toolbar thisToolBar = new Toolbar();
		thisToolBar.setController(thisController);
		return thisToolBar;
	}

	/**
	 * Make the toolbar
	 * 
	 * @return javax.swing.JToolBar
	 */
	private TreeToolbar makeTreeToolbar(final Controller thisController)
	{
		final TreeToolbar thisToolBar = new TreeToolbar();
		thisToolBar.setController(thisController);
		return thisToolBar;
	}

	/**
	 * Make the tabbed pane
	 * 
	 * @return javax.swing.JTabbedPane
	 */
	private JTabbedPane makeTabbedPane(final Controller thisController)
	{
		this.theTabbedPane = new JTabbedPane();
		makeTab(this.theTabbedPane, Messages.getString("MainFrame.10"), MainFrame.theTreeIcon, makeEditor(thisController), new Runnable() //$NON-NLS-1$
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						MainFrame.this.theController.updateTree();
					}
				});
		makeTab(this.theTabbedPane, Messages.getString("MainFrame.20"), MainFrame.theTreebolicIcon, makeView(thisController), new Runnable() //$NON-NLS-1$
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						MainFrame.this.theController.updateWidget();
					}
				});
		makeTab(this.theTabbedPane, Messages.getString("MainFrame.30"), null, new JScrollPane(this.theDomTreeView), new Runnable() //$NON-NLS-1$
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						MainFrame.this.theController.updateDom();
					}
				});
		makeTab(this.theTabbedPane, Messages.getString("MainFrame.40"), null, new JScrollPane(this.theTextView), new Runnable() //$NON-NLS-1$
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						MainFrame.this.theController.updateText();
					}
				});
		this.theTabbedPane.addChangeListener(thisController);
		this.theTabbedPane.addChangeListener(new ChangeListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void stateChanged(final ChangeEvent thisEvent)
			{
				final int thisSelectedTabIndex = MainFrame.this.theTabbedPane.getSelectedIndex();
				MainFrame.this.theToolbarPanel.getComponent(1).setVisible(thisSelectedTabIndex == 0);
			}
		});
		return this.theTabbedPane;
	}

	/**
	 * Make tab
	 * 
	 * @param thisTabbedPane
	 *        tabbed pane
	 * @param thisTitle
	 *        title
	 * @param thisIcon
	 *        tab icon
	 * @param thisComponent
	 *        component
	 * @param thisAction
	 *        component update runnable
	 */
	private void makeTab(final JTabbedPane thisTabbedPane, final String thisTitle, final Icon thisIcon, final Component thisComponent, final Runnable thisAction)
	{
		thisTabbedPane.addTab(thisTitle, thisIcon, thisComponent, null);
		this.theController.theUpdateMap.put(thisComponent, thisAction);
	}

	/**
	 * Make the html view
	 * 
	 * @return javax.swing.JPanel
	 */
	private TreeView makeTreeView(final Controller thisController)
	{
		final TreeView thisTreeView = new TreeView();
		thisTreeView.connect(thisController);
		thisTreeView.setPreferredSize(new Dimension(400, 500));
		return thisTreeView;
	}

	/**
	 * Make the treebolic view
	 * 
	 * @return treebolic view
	 */
	private JPanel makeView(final Controller thisController)
	{
		final JPanel thisView = new JPanel();
		thisView.setLayout(new BorderLayout());
		thisView.add((Component) this.theWidget, BorderLayout.CENTER);
		return thisView;
	}

	/**
	 * Make the widget
	 * 
	 * @return widget
	 */
	private Widget makeWidget(final IContext thisContext)
	{
		final Widget thisWidget = new Widget(thisContext);
		return thisWidget;
	}

	/**
	 * Make the text view
	 * 
	 * @return text view
	 */
	private TextView makeTextView()
	{
		final TextView thisTextView = new TextView();
		return thisTextView;
	}

	/**
	 * Make the property view
	 * 
	 * @return property view
	 */
	private PropertyView makePropertyView(final Controller thisController)
	{
		final PropertyView thisPropertyView = new PropertyView();
		thisPropertyView.setImageRepository(thisController.makeImageRepositoryURL());
		thisPropertyView.setCellEditorListener(thisController);
		return thisPropertyView;
	}

	/**
	 * Make the Dom tree view
	 * 
	 * @return Dom tree view
	 */
	private DomTreeView makeDomTreeView()
	{
		final TreeCellRenderer thisRenderer = MainFrame.hasTreebolicRendering ? new treebolic.generator.domtree.treebolic.Renderer() : new treebolic.generator.domtree.Renderer();
		final DomTreeView thisDomTreeView = new DomTreeView(thisRenderer);
		return thisDomTreeView;
	}

	/**
	 * Make the editor
	 * 
	 * @return javax.swing.JComponent
	 */
	private JComponent makeEditor(final Controller thisController)
	{
		final JSplitPane thisPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		thisPane.setDividerSize(8);
		thisPane.setOneTouchExpandable(true);
		thisPane.add(this.theTreeView, JSplitPane.LEFT);
		thisPane.add(makeSubEditor(thisController), JSplitPane.RIGHT);
		return thisPane;
	}

	/**
	 * Make thesubeditor
	 * 
	 * @return javax.swing.JComponent
	 */
	private JComponent makeSubEditor(final Controller thisController)
	{
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BorderLayout());
		thisPanel.add(this.thePropertyView, BorderLayout.CENTER);
		thisPanel.add(makeDefaultsPanel(thisController), BorderLayout.SOUTH);
		return thisPanel;
	}

	/**
	 * Make the defaults panel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel makeDefaultsPanel(final Controller thisController)
	{
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new FlowLayout());
		thisPanel.add(makeGlobalsButton(thisController, Messages.getString("MainFrame.51"), "top.png", Controller.Code.SELECTTOP)); //$NON-NLS-1$ //$NON-NLS-2$
		thisPanel.add(makeGlobalsButton(thisController, Messages.getString("MainFrame.52"), "tree.png", Controller.Code.SELECTTREE)); //$NON-NLS-1$ //$NON-NLS-2$
		thisPanel.add(makeGlobalsButton(thisController, Messages.getString("MainFrame.53"), "nodes.png", Controller.Code.SELECTNODES)); //$NON-NLS-1$ //$NON-NLS-2$
		thisPanel.add(makeGlobalsButton(thisController, Messages.getString("MainFrame.54"), "edges.png", Controller.Code.SELECTEDGES)); //$NON-NLS-1$ //$NON-NLS-2$
		return thisPanel;
	}

	/**
	 * Make the globals theButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton makeGlobalsButton(final Controller thisController, final String thisLabel, final String thisImage, final Controller.Code thisCode)
	{
		final JButton thisButton = new JButton();
		thisButton.setIcon(new ImageIcon(getClass().getResource("images/" + thisImage))); //$NON-NLS-1$
		thisButton.setToolTipText(thisLabel);
		thisButton.setBorder(null);
		thisButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		thisButton.addActionListener(new java.awt.event.ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent e)
			{
				thisController.execute(thisCode, 0);
			}
		});
		return thisButton;
	}
}
