/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import treebolic.Messages;

/**
 * Menu bar
 * 
 * @author Bernard Bou
 */
public class Menubar extends JMenuBar
{
	private static final long serialVersionUID = 1L;

	// D A T A

	/**
	 * Controller (command sink)
	 */
	private Controller theController;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public Menubar()
	{
		initialize();
	}

	/**
	 * Initialize
	 */
	private void initialize()
	{
		final JMenuItem thisOpenMenuItem = makeItem(Messages.getString("Menubar.110"), "open.png", Controller.Code.OPEN, KeyStroke.getKeyStroke('O', InputEvent.CTRL_DOWN_MASK)); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisOpenUrlMenuItem = makeItem(Messages.getString("Menubar.120"), "openurl.png", Controller.Code.OPENURL, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisImportXslMenuItem = makeItem(Messages.getString("Menubar.130"), "import.png", Controller.Code.IMPORTXSL, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisImportProviderMenuItem = makeItem(Messages.getString("Menubar.131"), "importprovider.png", Controller.Code.IMPORTPROVIDER, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisNewMenuItem = makeItem(Messages.getString("Menubar.140"), "newdoc.png", Controller.Code.NEW, KeyStroke.getKeyStroke('N', InputEvent.CTRL_DOWN_MASK)); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisSaveMenuItem = makeItem(Messages.getString("Menubar.150"), "save.png", Controller.Code.SAVE, KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK)); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisSaveAsMenuItem = makeItem(Messages.getString("Menubar.151"), "saveas.png", Controller.Code.SAVEAS, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisExportMenuItem = makeItem(Messages.getString("Menubar.160"), "export.png", Controller.Code.EXPORT, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisExportSettingsMenuItem = makeItem(Messages.getString("Menubar.161"), null, Controller.Code.EXPORTSETTINGS, null); //$NON-NLS-1$ 
		final JMenuItem thisMakeSiteMenuItem = makeItem(Messages.getString("Menubar.170"), "sitemake.png", Controller.Code.MAKESITE, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisRunSiteMenuItem = makeItem(Messages.getString("Menubar.171"), "siterun.png", Controller.Code.RUNSITE, null); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenuItem thisNewElementMenuItem = makeItem(Messages.getString("Menubar.200"), "new.png", Controller.Code.NEWELEMENT, KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0)); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisRemoveElementMenuItem = makeItem(Messages.getString("Menubar.210"), "delete.png", Controller.Code.REMOVEELEMENT, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0)); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisNormalizeIdsMenuItem = makeItem(Messages.getString("Menubar.220"), null, Controller.Code.NORMALIZEIDS, null); //$NON-NLS-1$
		final JMenuItem thisListImagesMenuItem = makeItem(Messages.getString("Menubar.230"), null, Controller.Code.LISTIMAGES, null); //$NON-NLS-1$
		final JMenuItem thisListLinksMenuItem = makeItem(Messages.getString("Menubar.231"), null, Controller.Code.LISTLINKS, null); //$NON-NLS-1$
		final JMenuItem thisListMountsMenuItem = makeItem(Messages.getString("Menubar.232"), null, Controller.Code.LISTMOUNTS, null); //$NON-NLS-1$
		final JMenuItem thisListIdsMenuItem = makeItem(Messages.getString("Menubar.233"), null, Controller.Code.LISTIDS, null); //$NON-NLS-1$

		final JMenuItem thisSettingsMenuItem = makeItem(Messages.getString("Menubar.300"), "settings.png", Controller.Code.SETTINGS, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisBaseMenuItem = makeItem(Messages.getString("Menubar.310"), "site.png", Controller.Code.SETTINGSBASE, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisUrlMenuItem = makeItem(Messages.getString("Menubar.320"), "url.png", Controller.Code.SETTINGSURL, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JCheckBoxMenuItem thisTreebolicRendererMenuItem = makeCheckboxItem(Messages.getString("Menubar.330"), null, Controller.Code.OPTIONTREEBOLICRENDERER, null, MainFrame.hasTreebolicRendering); //$NON-NLS-1$
		final JCheckBoxMenuItem thisValidateMenuItem = makeCheckboxItem(Messages.getString("Menubar.331"), null, Controller.Code.OPTIONVALIDATEXML, null, Controller.validate); //$NON-NLS-1$
		final JCheckBoxMenuItem thisFocusParentMenuItem = makeCheckboxItem(Messages.getString("Menubar.333"), null, Controller.Code.OPTIONFOCUSPARENT, null, treebolic.generator.tree.Tree.focusParent); //$NON-NLS-1$

		final JMenuItem thisDtdMenuItem = makeItem(Messages.getString("Menubar.410"), "dtd.png", Controller.Code.DTD, null); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenuItem thisAboutMenuItem = makeItem(Messages.getString("Menubar.911"), "about.png", Controller.Code.ABOUT, null); //$NON-NLS-1$ //$NON-NLS-2$
		final JMenuItem thisHelpMenuItem = makeItem(Messages.getString("Menubar.910"), "help.png", Controller.Code.HELP, null); //$NON-NLS-1$ //$NON-NLS-2$

		final JMenu thisFilesMenu = new JMenu();
		thisFilesMenu.setText(Messages.getString("Menubar.1")); //$NON-NLS-1$
		thisFilesMenu.add(thisNewMenuItem);
		thisFilesMenu.add(thisOpenMenuItem);
		thisFilesMenu.add(thisOpenUrlMenuItem);
		thisFilesMenu.add(thisImportXslMenuItem);
		thisFilesMenu.add(thisImportProviderMenuItem);
		thisFilesMenu.addSeparator();
		thisFilesMenu.add(thisSaveMenuItem);
		thisFilesMenu.add(thisSaveAsMenuItem);
		thisFilesMenu.add(thisExportMenuItem);
		thisFilesMenu.add(thisExportSettingsMenuItem);
		thisFilesMenu.addSeparator();
		thisFilesMenu.add(thisMakeSiteMenuItem);
		thisFilesMenu.add(thisRunSiteMenuItem);

		final JMenu thisElementsMenu = new JMenu();
		thisElementsMenu.setText(Messages.getString("Menubar.2")); //$NON-NLS-1$
		thisElementsMenu.add(thisNewElementMenuItem);
		thisElementsMenu.add(thisRemoveElementMenuItem);
		thisElementsMenu.addSeparator();
		thisElementsMenu.add(thisNormalizeIdsMenuItem);
		thisElementsMenu.addSeparator();
		thisElementsMenu.add(thisListImagesMenuItem);
		thisElementsMenu.add(thisListLinksMenuItem);
		thisElementsMenu.add(thisListMountsMenuItem);
		thisElementsMenu.add(thisListIdsMenuItem);

		final JMenu thisOptionsMenu = new JMenu();
		thisOptionsMenu.setText(Messages.getString("Menubar.3")); //$NON-NLS-1$
		thisOptionsMenu.add(thisSettingsMenuItem);
		thisOptionsMenu.add(thisBaseMenuItem);
		thisOptionsMenu.add(thisUrlMenuItem);
		thisOptionsMenu.addSeparator();
		thisOptionsMenu.add(thisTreebolicRendererMenuItem);
		thisOptionsMenu.add(thisValidateMenuItem);
		thisOptionsMenu.add(thisFocusParentMenuItem);

		final JMenu thisHelpMenu = new JMenu();
		thisHelpMenu.setText(Messages.getString("Menubar.9")); //$NON-NLS-1$
		thisHelpMenu.add(thisAboutMenuItem);
		thisHelpMenu.add(thisHelpMenuItem);

		final JMenu thisToolsMenu = new JMenu();
		thisToolsMenu.setText(Messages.getString("Menubar.4")); //$NON-NLS-1$
		thisToolsMenu.add(thisDtdMenuItem);

		this.add(thisFilesMenu);
		this.add(thisElementsMenu);
		this.add(thisOptionsMenu);
		this.add(thisToolsMenu);
		this.add(thisHelpMenu);
	}

	/**
	 * Set controller
	 * 
	 * @param thisController
	 *        controller
	 */
	public void setController(final Controller thisController)
	{
		this.theController = thisController;
	}

	/**
	 * Make menu item
	 * 
	 * @param thisText
	 *        text
	 * @param thisImage
	 *        image
	 * @param thisCommand
	 *        command code
	 * @param thisAcceleratorKey
	 *        accelerator key
	 * @return menu item
	 */
	private JMenuItem makeItem(final String thisText, final String thisImage, final Controller.Code thisCommand, final KeyStroke thisAcceleratorKey)
	{
		final JMenuItem thisItem = new JMenuItem();
		thisItem.setText(thisText);
		if (thisImage != null)
		{
			thisItem.setIcon(new ImageIcon(Menubar.class.getResource("images/" + thisImage))); //$NON-NLS-1$
		}
		if (thisAcceleratorKey != null)
		{
			thisItem.setAccelerator(thisAcceleratorKey);
		}
		thisItem.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				Menubar.this.theController.execute(thisCommand, 0);
			}
		});
		return thisItem;
	}

	/**
	 * Make menu item
	 * 
	 * @param thisText
	 *        text
	 * @param thisImage
	 *        image
	 * @param thisCommand
	 *        command code
	 * @param thisAcceleratorKey
	 *        accelerator key
	 * @return menu item
	 */
	private JCheckBoxMenuItem makeCheckboxItem(final String thisText, final String thisImage, final Controller.Code thisCommand, final KeyStroke thisAcceleratorKey, final boolean thisState)
	{
		final JCheckBoxMenuItem thisItem = new JCheckBoxMenuItem();
		thisItem.setText(thisText);
		thisItem.setSelected(thisState);
		if (thisImage != null)
		{
			thisItem.setIcon(new ImageIcon(Menubar.class.getResource("images/" + thisImage))); //$NON-NLS-1$
		}
		if (thisAcceleratorKey != null)
		{
			thisItem.setAccelerator(thisAcceleratorKey);
		}
		thisItem.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				Menubar.this.theController.execute(thisCommand, 0);
			}
		});
		return thisItem;
	}
}