/**
 * Title : Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright:	(c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.util.Map;
import java.util.SortedSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import treebolic.Messages;
import treebolic.commons.Utils;

/**
 * Reference list dialog
 * 
 * @author Bernard Bou
 */
public class ReferenceListDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Controller
	 */
	protected final Controller theController;

	// C O M P O N E N T S

	/**
	 * Label
	 */
	protected JLabel theLabel;

	/**
	 * Reference table
	 */
	protected JTable theReferenceTable;

	/**
	 * Reference table scrollpane
	 */
	protected JScrollPane theScrollPane;

	/**
	 * Button panel
	 */
	protected JPanel theButtonPanel;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public ReferenceListDialog(final Controller thisController)
	{
		super();
		this.theController = thisController;
		initialize();
	}

	/**
	 * Initialize
	 */
	private void initialize()
	{
		setResizable(true);

		// label
		this.theLabel = new JLabel();

		// reference table
		this.theReferenceTable = new JTable();
		this.theReferenceTable.setToolTipText(Messages.getString("ReferenceListDialog.0")); //$NON-NLS-1$

		// update
		update();

		// scroll pane
		this.theScrollPane = new JScrollPane(this.theReferenceTable);

		// buttons
		final JButton thisUpdateButton = new JButton(Messages.getString("ReferenceListDialog.1")); //$NON-NLS-1$
		thisUpdateButton.addActionListener(new java.awt.event.ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				update();
			}
		});
		final JButton thisCancelButton = new JButton(Messages.getString("ReferenceListDialog.2")); //$NON-NLS-1$
		thisCancelButton.addActionListener(new java.awt.event.ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				setVisible(false);
			}
		});

		// button panel
		this.theButtonPanel = new JPanel();
		this.theButtonPanel.add(thisCancelButton, null);
		this.theButtonPanel.add(thisUpdateButton, null);

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new GridBagLayout());
		thisPanel.add(this.theLabel, new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(10, 10, 0, 10), 0, 0));
		thisPanel.add(this.theScrollPane, new GridBagConstraints(0, 1, 1, 1, 1., 1., GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 10, 5, 10), 0, 0));
		thisPanel.add(this.theButtonPanel, new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 10, 10, 10), 0, 0));

		setContentPane(thisPanel);
	}

	/**
	 * Update
	 */
	protected void update()
	{
		// renderer
		final TableColumn thisImageColumn = this.theReferenceTable.getColumnModel().getColumn(0);
		thisImageColumn.setWidth(150);
		thisImageColumn.setCellRenderer(new DefaultTableCellRenderer()
		{
			private static final long serialVersionUID = 1L;

			/*
			 * (non-Javadoc)
			 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
			 */
			@Override
			public Component getTableCellRendererComponent(final JTable thisTable, final Object thisValue, final boolean thisIsSelected, final boolean thisHasFocus, final int thisRow, final int thisColumn)
			{
				final ParameterModel.Entry thisEntry = (ParameterModel.Entry) thisValue;
				setText(thisEntry.theKey);
				setBackground(thisEntry.theValueCount > 1 ? Color.PINK : Color.WHITE);
				return this;
			}
		});

		final TableColumn thisLocationColumn = this.theReferenceTable.getColumnModel().getColumn(1);
		thisLocationColumn.setCellRenderer(new DefaultTableCellRenderer()
		{
			private static final long serialVersionUID = 1L;

			/*
			 * (non-Javadoc)
			 * @see javax.swing.table.DefaultTableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
			 */
			@Override
			public Component getTableCellRendererComponent(final JTable thisTable, final Object thisValue, final boolean thisIsSelected, final boolean thisHasFocus, final int thisRow, final int thisColumn)
			{
				final ParameterModel.Entry thisEntry = (ParameterModel.Entry) thisValue;
				final String thisLocation = thisEntry.theValue;
				setText(thisLocation);
				return this;
			}
		});
	}

	/**
	 * Set model
	 * 
	 * @param thisTargetToLocationMap
	 *        target to location map
	 */
	protected void setModel(final Map<String, SortedSet<String>> thisTargetToLocationMap)
	{
		this.theReferenceTable.setModel(new ParameterModel(thisTargetToLocationMap));

		// renderer
		final TableColumn thisTargetColumn = this.theReferenceTable.getColumnModel().getColumn(0);
		thisTargetColumn.setWidth(150);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			pack();
			Utils.center(this);
		}
		super.setVisible(thisFlag);
	}

	// M O D E L

	/**
	 * Parameter model
	 * 
	 * @author Bernard Bou
	 */
	protected class ParameterModel extends AbstractTableModel
	{
		/**
		 * Serial version uid
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Properties
		 */
		private final Vector<Entry> theEntries = new Vector<Entry>();

		class Entry
		{
			/**
			 * Key
			 */
			public String theKey;

			/**
			 * Value
			 */
			public String theValue;

			/**
			 * Number of values for key
			 */
			public int theValueCount;

			/**
			 * Constructor
			 * 
			 * @param thisKey
			 *        key
			 * @param thisValue
			 *        value
			 */
			public Entry(final String thisKey, final String thisValue, final int thisValueCount)
			{
				this.theKey = thisKey;
				this.theValue = thisValue;
				this.theValueCount = thisValueCount;
			}
		}

		/**
		 * Parameter model
		 * 
		 * @param thisTargetToLocationMap
		 *        target location map
		 */
		public ParameterModel(final Map<String, SortedSet<String>> thisTargetToLocationMap)
		{
			if (thisTargetToLocationMap == null)
				return;

			for (final String thisTarget : thisTargetToLocationMap.keySet())
			{
				final SortedSet<String> theseLocations = thisTargetToLocationMap.get(thisTarget);
				final int thisCount = theseLocations.size();
				for (final String thisLocation : theseLocations)
				{
					newRow(new ParameterModel.Entry(thisTarget, thisLocation, thisCount));
				}
			}
		}

		/**
		 * New row
		 * 
		 * @param thisEntry
		 *        entry
		 * @return row index
		 */
		public int newRow(final Entry thisEntry)
		{
			this.theEntries.add(thisEntry);
			final int thisRowIdx = this.theEntries.size() - 1;
			fireTableRowsInserted(thisRowIdx, thisRowIdx);
			return thisRowIdx;
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		@Override
		public int getRowCount()
		{
			return this.theEntries.size();
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		@Override
		public int getColumnCount()
		{
			return 2;
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#isCellEditable(int, int)
		 */
		@Override
		public boolean isCellEditable(final int y, final int x)
		{
			return false;
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		@Override
		public Object getValueAt(final int y, final int x)
		{
			return this.theEntries.get(y);
		}

		/*
		 * (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnName(int)
		 */
		@Override
		public String getColumnName(final int x)
		{
			switch (x)
			{
			case 0:
				return Messages.getString("ReferenceListDialog.10"); //$NON-NLS-1$
			case 1:
				return Messages.getString("ReferenceListDialog.11"); //$NON-NLS-1$
			default:
				return null;
			}
		}
	}
}
