/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator.domtree.treebolic;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import treebolic.generator.domtree.DefaultDecorator;

/**
 * Treebolic-specific renderer
 * 
 * @author Bernard Bou
 */
public class Renderer extends treebolic.generator.domtree.Renderer
{
	// icons

	/**
	 * Node icon
	 */
	static protected ImageIcon theNodeIcon;

	/**
	 * Edge icon
	 */
	static protected ImageIcon theEdgeIcon;

	/**
	 * Link icon
	 */
	static protected ImageIcon theLinkIcon;

	// styles

	/**
	 * Style for node name
	 */
	static private SimpleAttributeSet theNodeNameStyle = new SimpleAttributeSet();

	/**
	 * Style for node data
	 */
	static private SimpleAttributeSet theNodeValueStyle = new SimpleAttributeSet();

	/**
	 * Style for edge name
	 */
	static private SimpleAttributeSet theEdgeNameStyle = new SimpleAttributeSet();

	/**
	 * Style for link name
	 */
	static private SimpleAttributeSet theLinkNameStyle = new SimpleAttributeSet();

	/**
	 * Style for edge data
	 */
	static private SimpleAttributeSet theEdgeValueStyle = new SimpleAttributeSet();

	/**
	 * Style for link data
	 */
	static private SimpleAttributeSet theLinkValueStyle = new SimpleAttributeSet();

	/**
	 * Style for src data
	 */
	static private SimpleAttributeSet theSrcStyle = new SimpleAttributeSet();

	/**
	 * Edge ends style
	 */
	static private SimpleAttributeSet theEdgeEndsStyle = new SimpleAttributeSet();

	// patterns

	/**
	 * Src pattern
	 */
	private final Pattern theSrcPattern = Pattern.compile("src=\"(([^\"]*))\""); //$NON-NLS-1$

	/**
	 * From pattern
	 */
	private final Pattern theFromPattern = Pattern.compile("from=\"(([^\"]*))\""); //$NON-NLS-1$

	/**
	 * To pattern
	 */
	private final Pattern theToPattern = Pattern.compile("to=\"(([^\"]*))\""); //$NON-NLS-1$

	// maps

	/**
	 * Name styles for classes
	 */
	private final Map<String, SimpleAttributeSet> theTypeToNameStyleMap;

	/**
	 * Value styles for classes
	 */
	private final Map<String, SimpleAttributeSet> theTypeToValueStyleMap;

	/**
	 * Icons for classes
	 */
	private final Map<String, Icon> theTypeToIconMap;

	static
	{
		// name styles
		StyleConstants.setFontFamily(Renderer.theNodeNameStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theNodeNameStyle, 11);
		StyleConstants.setBold(Renderer.theNodeNameStyle, true);
		StyleConstants.setForeground(Renderer.theNodeNameStyle, Color.BLACK);
		StyleConstants.setBackground(Renderer.theNodeNameStyle, Color.ORANGE);

		StyleConstants.setFontFamily(Renderer.theEdgeNameStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theEdgeNameStyle, 11);
		StyleConstants.setBold(Renderer.theEdgeNameStyle, true);
		StyleConstants.setForeground(Renderer.theEdgeNameStyle, Color.BLACK);
		StyleConstants.setBackground(Renderer.theEdgeNameStyle, Color.ORANGE);

		StyleConstants.setFontFamily(Renderer.theLinkNameStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theLinkNameStyle, 11);
		StyleConstants.setBold(Renderer.theLinkNameStyle, true);
		StyleConstants.setForeground(Renderer.theLinkNameStyle, Color.WHITE);
		StyleConstants.setBackground(Renderer.theLinkNameStyle, Color.BLUE);

		// value styles
		StyleConstants.setFontFamily(Renderer.theNodeValueStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theNodeValueStyle, 10);
		StyleConstants.setForeground(Renderer.theNodeValueStyle, Color.BLACK);

		StyleConstants.setFontFamily(Renderer.theEdgeValueStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theEdgeValueStyle, 10);
		StyleConstants.setForeground(Renderer.theEdgeValueStyle, Color.BLACK);

		StyleConstants.setFontFamily(Renderer.theLinkValueStyle, "Dialog"); //$NON-NLS-1$
		StyleConstants.setFontSize(Renderer.theLinkValueStyle, 10);
		StyleConstants.setForeground(Renderer.theLinkValueStyle, Color.BLACK);

		// pattern styles
		StyleConstants.setForeground(Renderer.theSrcStyle, Color.MAGENTA);

		// images
		Renderer.theNodeIcon = new ImageIcon(Renderer.class.getResource("images/treenode.gif")); //$NON-NLS-1$
		Renderer.theEdgeIcon = new ImageIcon(Renderer.class.getResource("images/treeedge.gif")); //$NON-NLS-1$
		Renderer.theLinkIcon = new ImageIcon(Renderer.class.getResource("images/treelink.gif")); //$NON-NLS-1$
	}

	/**
	 * Constructor
	 */
	public Renderer()
	{
		// styles for class
		this.theTypeToNameStyleMap = new HashMap<String, SimpleAttributeSet>();
		this.theTypeToNameStyleMap.put("node", Renderer.theNodeNameStyle); //$NON-NLS-1$
		this.theTypeToNameStyleMap.put("edge", Renderer.theNodeNameStyle); //$NON-NLS-1$
		this.theTypeToNameStyleMap.put("a", Renderer.theLinkNameStyle); //$NON-NLS-1$
		this.theTypeToValueStyleMap = new HashMap<String, SimpleAttributeSet>();
		this.theTypeToValueStyleMap.put("node", Renderer.theEdgeValueStyle); //$NON-NLS-1$
		this.theTypeToValueStyleMap.put("edge", Renderer.theEdgeValueStyle); //$NON-NLS-1$
		this.theTypeToValueStyleMap.put("a", Renderer.theLinkValueStyle); //$NON-NLS-1$

		// styles for patterns
		this.thePatterns.add(this.theSrcPattern);
		this.thePatterns.add(this.theFromPattern);
		this.thePatterns.add(this.theToPattern);
		this.thePatternToStyleMap.put(this.theSrcPattern, Renderer.theSrcStyle);
		this.thePatternToStyleMap.put(this.theFromPattern, Renderer.theEdgeEndsStyle);
		this.thePatternToStyleMap.put(this.theToPattern, Renderer.theEdgeEndsStyle);

		// icons for class
		this.theTypeToIconMap = new HashMap<String, Icon>();
		this.theTypeToIconMap.put("node", Renderer.theNodeIcon); //$NON-NLS-1$
		this.theTypeToIconMap.put("edge", Renderer.theEdgeIcon); //$NON-NLS-1$
		this.theTypeToIconMap.put("a", Renderer.theLinkIcon); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.domtree.Renderer#getIconStyle(org.w3c.dom.Node)
	 */
	@Override
	protected Icon getIconStyle(final Node thisNode)
	{
		if (thisNode instanceof Element)
		{
			final Element thisElement = (Element) thisNode;
			final Icon thisIcon = this.theTypeToIconMap.get(thisElement.getNodeName());
			if (thisIcon != null)
				return thisIcon;
		}
		return super.getIconStyle(thisNode);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.domtree.Renderer#getNameStyle(org.w3c.dom.Node)
	 */
	@Override
	protected SimpleAttributeSet getNameStyle(final Node thisNode)
	{
		if (thisNode instanceof Element)
		{
			final Element thisElement = (Element) thisNode;
			final SimpleAttributeSet thisStyle = this.theTypeToNameStyleMap.get(thisElement.getNodeName());
			if (thisStyle != null)
				return thisStyle;
		}
		return super.getNameStyle(thisNode);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.domtree.Renderer#getValueStyle(org.w3c.dom.Node)
	 */
	@Override
	protected SimpleAttributeSet getValueStyle(final Node thisNode)
	{
		if (thisNode instanceof Element)
		{
			final Element thisElement = (Element) thisNode;
			final SimpleAttributeSet thisStyle = this.theTypeToValueStyleMap.get(thisElement.getNodeName());
			if (thisStyle != null)
				return thisStyle;
		}
		return super.getValueStyle(thisNode);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.generator.domtree.Renderer#makeDecorator(org.w3c.dom.Node)
	 */
	@Override
	protected DefaultDecorator makeDecorator(final Node thisNode)
	{
		if (thisNode.getNodeType() != Node.ELEMENT_NODE)
			return super.makeDecorator(thisNode);

		final Element thisElement = (Element) thisNode;
		final String thisTag = thisElement.getTagName();
		if (thisTag.equals("node")) //$NON-NLS-1$
			return new NodeDecorator(thisElement);
		else if (thisTag.equals("edge")) //$NON-NLS-1$
			return new EdgeDecorator(thisElement);
		else if (thisTag.equals("a")) //$NON-NLS-1$
			return new LinkDecorator(thisElement);
		return super.makeDecorator(thisNode);
	}
}