/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.generator.tree;

import java.util.List;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;

import treebolic.model.IEdge;
import treebolic.model.INode;
import treebolic.model.MenuItem;
import treebolic.model.Model;
import treebolic.model.Node;
import treebolic.model.Settings;

/**
 * Tree adapter (converts treebolic model to DefaultTreeModel)
 * 
 * @author Bernard Bou
 */
public class TreeAdapter extends DefaultTreeModel
{
	private static final long serialVersionUID = 1L;

	/**
	 * Constructor
	 * 
	 * @param thisModel
	 *        treebolic model
	 */
	public TreeAdapter(final Model thisModel)
	{
		super(TreeAdapter.makeTreeModel(thisModel));
	}

	/**
	 * Make tree model out of treebolic model
	 * 
	 * @param thisModel
	 *        treebolic model
	 * @return tree model
	 */
	public static javax.swing.tree.TreeNode makeTreeModel(final Model thisModel)
	{
		if (thisModel == null)
			return null;

		// root
		final Node thisNode = (Node) thisModel.theTree.getRoot();
		final MutableTreeNode thisRootTreeNode = TreeAdapter.makeNode(thisNode, null);

		// nodes
		final DefaultMutableTreeNode thisNodesTreeNode = new DefaultMutableTreeNode();
		thisNodesTreeNode.setUserObject(new NodesWrapper(thisModel.theSettings));
		thisNodesTreeNode.add(thisRootTreeNode);
		thisNodesTreeNode.add(thisRootTreeNode);

		// edges
		final List<IEdge> theseEdges = thisModel.theTree.getEdges();
		final MutableTreeNode thisEdgesTreeNode = TreeAdapter.makeEdges(theseEdges, thisModel, thisModel.theSettings);

		// tree
		final DefaultMutableTreeNode thisTreeTreeNode = new DefaultMutableTreeNode();
		thisTreeTreeNode.setUserObject(new TreeWrapper(thisModel.theSettings));
		thisTreeTreeNode.add(thisNodesTreeNode);
		thisTreeTreeNode.add(thisEdgesTreeNode);

		// tools
		final MutableTreeNode thisToolsTreeNode = TreeAdapter.makeTools(thisModel.theSettings.theMenu, thisModel.theSettings);

		// top
		final DefaultMutableTreeNode thisTreeNode = new DefaultMutableTreeNode();
		thisTreeNode.setUserObject(new TopWrapper(thisModel.theSettings));
		thisTreeNode.add(thisTreeTreeNode);
		thisTreeNode.add(thisToolsTreeNode);
		return thisTreeNode;
	}

	/**
	 * Make tree node for treebolic node
	 * 
	 * @param thisNode
	 *        treebolic node
	 * @param thisParentTreeNode
	 *        parent tree node
	 * @return tree node
	 */
	private static MutableTreeNode makeNode(final Node thisNode, final MutableTreeNode thisParentTreeNode)
	{
		final DefaultMutableTreeNode thisTreeNode = new DefaultMutableTreeNode();
		thisTreeNode.setUserObject(thisNode);

		// recurse
		for (final INode thisIChild : thisNode.getChildren())
		{
			final Node thisChild = (Node) thisIChild;
			final MutableTreeNode thisChildTreeNode = TreeAdapter.makeNode(thisChild, thisTreeNode);

			// attach child to parent
			thisTreeNode.add(thisChildTreeNode);
		}
		return thisTreeNode;
	}

	/**
	 * Make tree node for edges
	 * 
	 * @param thisEdgeList
	 *        edge list
	 * @param thisModel
	 *        model
	 * @param theseSettings
	 *        settings
	 * @return tree node
	 */
	private static MutableTreeNode makeEdges(final List<IEdge> thisEdgeList, final Model thisModel, final Settings theseSettings)
	{
		// container
		final DefaultMutableTreeNode thisEdgesTreeNode = new DefaultMutableTreeNode();
		thisEdgesTreeNode.setUserObject(new EdgesWrapper(thisEdgeList, thisModel, theseSettings));

		// iterate
		if (thisEdgeList != null)
		{
			for (final IEdge thisEdge : thisEdgeList)
			{
				final DefaultMutableTreeNode thisEdgeTreeNode = new DefaultMutableTreeNode();
				thisEdgeTreeNode.setUserObject(thisEdge);

				// attach to container
				thisEdgesTreeNode.add(thisEdgeTreeNode);
			}
		}
		return thisEdgesTreeNode;
	}

	/**
	 * Make tree node for menu
	 * 
	 * @param thisMenu
	 *        menu
	 * @param theseSettings
	 *        settings
	 * @return tree node
	 */
	private static MutableTreeNode makeTools(final List<MenuItem> thisMenu, final Settings theseSettings)
	{
		final DefaultMutableTreeNode thisToolsTreeNode = new DefaultMutableTreeNode();
		thisToolsTreeNode.setUserObject(new ToolsWrapper(theseSettings));

		final DefaultMutableTreeNode thisMenuTreeNode = new DefaultMutableTreeNode();
		thisMenuTreeNode.setUserObject(new MenuWrapper(theseSettings.theMenu, theseSettings));
		thisToolsTreeNode.add(thisMenuTreeNode);

		if (theseSettings.theMenu != null)
		{
			for (final MenuItem thisMenuItem : theseSettings.theMenu)
			{
				final DefaultMutableTreeNode thisMenuItemTreeNode = new DefaultMutableTreeNode();
				thisMenuItemTreeNode.setUserObject(new MenuItemWrapper(thisMenuItem, theseSettings));
				thisMenuTreeNode.add(thisMenuItemTreeNode);
			}
		}
		return thisToolsTreeNode;
	}
}
