/**
 * Title: Hyperbolic engine
 * Description: Treebolic Engine
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.model;

import java.awt.Color;
import java.awt.Image;

/**
 * Edge data
 * 
 * @author Bernard Bou
 */
class EdgeData
{
	/**
	 * Label
	 */
	protected String theLabel;

	/**
	 * Edge color
	 */
	protected Color theColor;

	/**
	 * Edge style
	 */
	protected Integer theStyle;

	/**
	 * Edge image filename
	 */
	protected String theImageFile;

	/**
	 * Constructor
	 */
	public EdgeData()
	{
		this.theLabel = null;
		this.theColor = null;
		this.theStyle = null;
		this.theImageFile = null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getLabel()
	 */
	public String getLabel()
	{
		return this.theLabel;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getColor()
	 */
	public Color getColor()
	{
		return this.theColor;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getStyle()
	 */
	public Integer getStyle()
	{
		return this.theStyle;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getImageFile()
	 */
	public String getImageFile()
	{
		return this.theImageFile;
	}
}

/**
 * Extended edge data (internal use)
 * 
 * @author Bernard Bou
 */
class XEdgeData extends EdgeData
{
	/**
	 * Edge image
	 */
	protected Image theImage;

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getImage()
	 */
	public Image getImage()
	{
		return this.theImage;
	}

	/**
	 * Set image
	 * 
	 * @param thisImage
	 *        image
	 */
	public void setImage(final Image thisImage)
	{
		this.theImage = thisImage;
	}
}

/**
 * Edge
 * 
 * @author Bernard Bou
 */
public class Edge extends XEdgeData implements IEdge
{
	/**
	 * From-node
	 */
	protected INode theFromNode;

	/**
	 * To-node
	 */
	protected INode theToNode;

	/**
	 * Construct edge
	 * 
	 * @param thisFromINode
	 *        from node
	 * @param thisToINode
	 *        to node
	 */
	public Edge(final INode thisFromINode, final INode thisToINode)
	{
		super();
		this.theFromNode = thisFromINode;
		this.theToNode = thisToINode;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getFrom()
	 */
	@Override
	public INode getFrom()
	{
		return this.theFromNode;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.kernel.IEdge#getTo()
	 */
	@Override
	public INode getTo()
	{
		return this.theToNode;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return (this.theFromNode == null ? "null" : this.theFromNode.getLabel()) + "->" + (this.theToNode == null ? "null" : this.theToNode.getLabel());
	}
}
