/**
 * Title :		Hyperbolic tree engine
 * Description: Treebolic Engine
 * Version:		1.10.x
 * Copyright:	(c) 2001-2005
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author:		Bernard Bou
 * Company:		bsys
 * Update:		19 juil. 2008
 */

package treebolic.model.graph;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import treebolic.model.IEdge;
import treebolic.model.INode;
import treebolic.model.MutableEdge;
import treebolic.model.Tree;
import treebolic.model.TreeMutableNode;

/**
 * Graph converter
 * 
 * @author Bernard Bou
 */
public class Converter<T extends TreeMutableNode>
{
	/**
	 * Convert graph to tree
	 * 
	 * @param thisGraph
	 * @return tree
	 */
	@SuppressWarnings({ "unchecked" })
	public Tree graphToTree(final treebolic.model.graph.Graph thisGraph)
	{
		final treebolic.model.graph.Tree thisSpanningTree = thisGraph.makeSpanningTree();

		// tree edges
		final Collection<GraphEdge> theseGraphEdges = thisSpanningTree.theGraph.getEdges();
		for (final GraphEdge thisGraphEdge : theseGraphEdges)
		{
			final T thisFromNode = (T) thisGraphEdge.getFrom();
			final T thisToNode = (T) thisGraphEdge.getTo();
			thisToNode.addToParent(thisFromNode);

			// transfer style
			final IEdge thisEdge = (IEdge) thisGraphEdge.getUserData();
			thisToNode.setEdgeColor(thisEdge.getColor());
			thisToNode.setEdgeStyle(thisEdge.getStyle());
			thisToNode.setEdgeLabel(thisEdge.getLabel());
			thisToNode.setEdgeImageFile(thisEdge.getImageFile());
		}

		// non-tree edges
		List<IEdge> theseEdges = null;
		for (final GraphEdge thisGraphEdge : thisGraph.getEdges())
		{
			if (theseGraphEdges.contains(thisGraphEdge))
			{
				continue;
			}
			final MutableEdge thisEdge = (MutableEdge) thisGraphEdge.getUserData();
			if (theseEdges == null)
			{
				theseEdges = new ArrayList<IEdge>();
			}
			theseEdges.add(thisEdge);
		}

		return new Tree((INode) thisSpanningTree.theRoot, theseEdges);
	}
}
