/**
 * Title: Treebolic DOT provider
 * Description: Treebolic DOT provider
 * Version: 2.x
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.dot;

import java.net.URL;

import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.Tree;

/**
 * Provider factory implementation for DOT provider
 * 
 * @author Bernard Bou
 */
public class Provider implements IProvider
{
	/**
	 * Provider context
	 */
	private IProviderContext theContext;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public Provider()
	{
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#set(treebolic.model.IProviderContext)
	 */
	@Override
	public void connect(final IProviderContext thisContext)
	{
		this.theContext = thisContext;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeModel(java.lang.String)
	 */
	@Override
	public Model makeModel(final String thisSource)
	{
		// get dot file
		String thisDotFile = thisSource;
		if (thisDotFile == null)
		{
			thisDotFile = this.theContext.getParameter("dot");
		}

		if (thisDotFile != null)
		{
			// URL
			final URL thisURL = this.theContext.makeURL(thisDotFile);
			if (thisURL != null)
			{
				// parse
				this.theContext.putProgress("Loading ..." + thisDotFile, false);
				final Model thisModel = DotParser.parseModel(thisURL);
				if (thisModel != null)
				{
					this.theContext.putProgress("Loaded " + thisDotFile, false);
					return thisModel;
				}
				else
				{
					this.theContext.putMessage("Cannot load Dot file <" + thisDotFile + ">");
				}
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisDotFile);
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeTree(java.lang.String, boolean)
	 */
	@Override
	public Tree makeTree(final String thisSource, final boolean checkRecursion)
	{
		// get dot file
		String thisDotFile = thisSource;
		if (thisDotFile == null)
		{
			thisDotFile = this.theContext.getParameter("dot");
		}

		if (thisDotFile != null)
		{
			// URL
			final URL thisURL = this.theContext.makeURL(thisDotFile);
			if (thisURL != null)
			{
				// parse
				this.theContext.putProgress("Loading ..." + thisDotFile, false);
				final Tree thisTree = DotParser.parseTree(thisURL);
				if (thisTree != null)
				{
					this.theContext.putProgress("Loaded " + thisDotFile, false);
					return thisTree;
				}
				else
				{
					this.theContext.putMessage("Cannot load Dot file <" + thisDotFile + ">");
				}
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisDotFile);
			}
		}
		return null;
	}
}
