/**
 * Title: Treebolic OWL provider
 * Description: Treebolic OWL provider
 * Version:	2.x
 * Copyright: (c) 2001-2008
 * Terms of use: see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.owl;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.Tree;
import treebolic.model.Utils;

/**
 * Provider for OWL
 * 
 * @author Bernard Bou
 */
public class Provider implements IProvider
{
	/**
	 * Provider context
	 */
	private IProviderContext theContext;

	/**
	 * Constructor
	 */
	public Provider()
	{
		// do nothing
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#set(treebolic.model.IProviderContext)
	 */
	@Override
	public void connect(final IProviderContext thisContext)
	{
		this.theContext = thisContext;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeModel(java.lang.String)
	 */
	@Override
	public Model makeModel(final String thisSource)
	{
		// get xml file
		String thisOwlFileUrl = thisSource;
		if (thisOwlFileUrl == null)
		{
			thisOwlFileUrl = this.theContext.getParameter("owl");
		}

		// parse XML file
		if (thisOwlFileUrl != null)
		{
			// settings properties
			final Properties theseProperties = getProperties();

			// parser
			final OwlParser thisParser = new OwlParser(theseProperties);

			// URL
			final URL thisUrl = this.theContext.makeURL(thisOwlFileUrl);
			if (thisUrl != null)
			{
				// parse
				this.theContext.putProgress("Loading ..." + thisOwlFileUrl, false);
				final Model thisModel = thisParser.parseModel(thisUrl.toString());
				if (thisModel != null)
				{
					this.theContext.putProgress("Loaded " + thisOwlFileUrl, false);
					return thisModel;
				}
				this.theContext.putMessage("Cannot load OWL file <" + thisOwlFileUrl + ">");
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisOwlFileUrl);
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeTree(java.lang.String, boolean)
	 */
	@Override
	public Tree makeTree(final String thisSource, final boolean checkRecursion)
	{
		// get owl file
		String thisOwlFileUrl = thisSource;
		if (thisOwlFileUrl == null)
		{
			thisOwlFileUrl = this.theContext.getParameter("owl");
		}

		// parse owl file
		if (thisOwlFileUrl != null)
		{
			// URL
			final URL thisUrl = this.theContext.makeURL(thisOwlFileUrl);
			if (thisUrl != null)
			{
				// settings properties
				final Properties theseProperties = getProperties();

				// parser
				final OwlParser thisParser = new OwlParser(theseProperties);

				// parse
				this.theContext.putProgress("Loading ..." + thisOwlFileUrl, false);
				final Tree thisTree = thisParser.parseTree(thisUrl.toString());
				if (thisTree != null)
				{
					this.theContext.putProgress("Loaded " + thisOwlFileUrl, false);
					return thisTree;
				}
				this.theContext.putMessage("Cannot load OWL file <" + thisOwlFileUrl + ">");
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisOwlFileUrl);
			}
		}
		return null;
	}

	/**
	 * Get properties in settings file
	 * 
	 * @return properties
	 */
	private Properties getProperties()
	{
		// settings properties
		Properties theseProperties = null;
		final String thisPropertyFileUrl = this.theContext.getParameter("settings");
		if (thisPropertyFileUrl != null && !thisPropertyFileUrl.isEmpty())
		{
			final URL thisUrl = this.theContext.makeURL(thisPropertyFileUrl);
			if (thisUrl != null)
			{
				this.theContext.putProgress("Loading ..." + thisPropertyFileUrl, false);
				try
				{
					theseProperties = Utils.load(thisUrl);
					this.theContext.putProgress("Loaded " + thisPropertyFileUrl, false);
				}
				catch (final IOException thisException)
				{
					this.theContext.putMessage("Cannot load Settings file <" + thisPropertyFileUrl + ">");
				}
			}
		}
		return theseProperties;
	}
}
