/**
 * Title: Treebolic SQL provider
 * Description: Treebolic SQL provider
 * Version: 2.x
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.sql;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.table.TableModel;

/**
 * SQL statement dialog
 * 
 * @author Bernard Bou
 */
public class SqlDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * Attributes
	 */
	private final String[][] theNameValues = new String[][] { //
	{ "nodes.id", null }, //
			{ "nodes.label", null }, //
			{ "nodes.content", null }, //
			{ "nodes.backcolor", null }, //
			{ "nodes.forecolor", null }, //
			{ "nodes.image", null }, //
			{ "nodes.link", null }, //
			{ "nodes.target", null }, //
			{ "nodes.weight", null }, //
			{ "nodes.mountpoint", null }, //
			{ "nodes.mountnow", null }, //

			{ "edges.istree", null }, //
			{ "edges.from", null }, //
			{ "edges.to", null }, //
			{ "edges.label", null }, //
			{ "edges.image", null }, //
			{ "edges.color", null }, //
			{ "edges.hidden", null }, //
			{ "edges.stroke", null }, //
			{ "edges.fromterminator", null }, //
			{ "edges.toterminator", null }, //

			{ "settings.backimage", null }, //
			{ "settings.backcolor", null }, //
			{ "settings.forecolor", null }, //
			{ "settings.fontface", null }, //
			{ "settings.fontsize", null }, //
			{ "settings.fontsizestep", null }, //
			{ "settings.orientation", null }, //
			{ "settings.expansion", null }, //
			{ "settings.sweep", null }, //
			{ "settings.preserveorientation", null }, //
			{ "settings.hyperbolic", null }, //
			{ "settings.hastoolbar", null }, //
			{ "settings.hasstatusbar", null }, //
			{ "settings.haspopupmenu", null }, //
			{ "settings.hastooltip", null }, //
			{ "settings.tooltipdisplayscontent", null }, //
			{ "settings.focusonhover", null }, //
			{ "settings.focus", null }, //
			{ "settings.xmoveto", null }, //
			{ "settings.ymoveto", null }, //
			{ "settings.xshift", null }, //
			{ "settings.yshift", null }, //
			{ "settings.nodebackcolor", null }, //
			{ "settings.nodeforecolor", null }, //
			{ "settings.nodeimage", null }, //
			{ "settings.treeedgecolor", null }, //
			{ "settings.treeedgehidden", null }, //
			{ "settings.treeedgestroke", null }, //
			{ "settings.treeedgefromterminator", null }, //
			{ "settings.treeedgetoterminator", null }, //
			{ "settings.treeedgeimage", null }, //
			{ "settings.edgecolor", null }, //
			{ "settings.edgehidden", null }, //
			{ "settings.edgestroke", null }, //
			{ "settings.edgefromterminator", null }, //
			{ "settings.edgetoterminator", null }, //
			{ "settings.edgeimage", null }, //

			{ "menu.action", null }, //
			{ "menu.label", null }, //
			{ "menu.target", null }, //
			{ "menu.scope", null }, //
			{ "menu.mode", null }, //
			{ "menu.link", null }, //
	};

	/**
	 * Ok button pressed
	 */
	public boolean ok;

	/**
	 * Property file
	 */
	private String thePropertyUrlString;

	/**
	 * URL combo box
	 */
	private final JTextField theURLField;

	/**
	 * Querier text area
	 */
	private final JTextField theUserTextField;

	/**
	 * Password field
	 */
	private final JPasswordField thePasswdField;

	/**
	 * Name table
	 */
	private final JTable theNameTable;

	/**
	 * Nodes SQL statement text area
	 */
	private final JTextArea theNodesSqlTextArea;

	/**
	 * Tree Edges SQL statement text area
	 */
	private final JTextArea theTreeEdgesSqlTextArea;

	/**
	 * Edges SQL statement text area
	 */
	private final JTextArea theEdgesSqlTextArea;

	/**
	 * Settings SQL statement text area
	 */
	private final JTextArea theSettingsSqlTextArea;

	/**
	 * Constructor
	 * 
	 * @param thisPropertyUrlString
	 *        property file string
	 */
	public SqlDialog(final String thisPropertyUrlString)
	{
		super((JFrame) null, true);
		this.ok = false;
		this.thePropertyUrlString = thisPropertyUrlString;
		this.theURLField = new JTextField();
		this.theUserTextField = new JTextField();
		this.thePasswdField = new JPasswordField();
		this.theNodesSqlTextArea = new JTextArea();
		this.theTreeEdgesSqlTextArea = new JTextArea();
		this.theEdgesSqlTextArea = new JTextArea();
		this.theSettingsSqlTextArea = new JTextArea();
		this.theNameTable = new JTable(this.theNameValues, new String[] { "name", "value" });

		initialize();
		load();
	}

	/**
	 * Initialize
	 * 
	 * @throws Exception
	 */
	private void initialize()
	{
		final JLabel thisTitleLabel = new JLabel("Treebolic SQL provider");
		thisTitleLabel.setFont(new Font("Dialog", Font.BOLD, 16));
		thisTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
		thisTitleLabel.setHorizontalTextPosition(SwingConstants.CENTER);

		final JLabel thisURLLabel = new JLabel("URL");
		final JLabel thisUserLabel = new JLabel("User");
		final JLabel thisPasswdLabel = new JLabel("Password");
		final JLabel thisSqlLabel = new JLabel("SQL");
		final JLabel thisMapLabel = new JLabel("Field names");

		this.theURLField.setEditable(true);
		this.theURLField.setToolTipText("URL for data source");

		this.theUserTextField.setToolTipText("user to query database as");
		this.theUserTextField.setText("user");

		this.thePasswdField.setToolTipText(null);
		this.thePasswdField.setText("password to query database");

		this.theNodesSqlTextArea.setEditable(true);
		this.theNodesSqlTextArea.setLineWrap(true);
		this.theNodesSqlTextArea.setWrapStyleWord(true);
		this.theNodesSqlTextArea.setRequestFocusEnabled(true);
		this.theNodesSqlTextArea.setToolTipText("SQL statement selecting nodes");
		this.theNodesSqlTextArea.setText(null);
		this.theNodesSqlTextArea.setRows(4);

		this.theTreeEdgesSqlTextArea.setEditable(true);
		this.theTreeEdgesSqlTextArea.setLineWrap(true);
		this.theTreeEdgesSqlTextArea.setWrapStyleWord(true);
		this.theTreeEdgesSqlTextArea.setRequestFocusEnabled(true);
		this.theTreeEdgesSqlTextArea.setToolTipText("SQL statement selecting tree edges");
		this.theTreeEdgesSqlTextArea.setText(null);
		this.theTreeEdgesSqlTextArea.setRows(4);

		this.theEdgesSqlTextArea.setEditable(true);
		this.theEdgesSqlTextArea.setLineWrap(true);
		this.theEdgesSqlTextArea.setWrapStyleWord(true);
		this.theEdgesSqlTextArea.setRequestFocusEnabled(true);
		this.theEdgesSqlTextArea.setToolTipText("SQL statement selecting edges");
		this.theEdgesSqlTextArea.setText(null);
		this.theEdgesSqlTextArea.setRows(4);

		this.theSettingsSqlTextArea.setEditable(true);
		this.theSettingsSqlTextArea.setLineWrap(true);
		this.theSettingsSqlTextArea.setWrapStyleWord(true);
		this.theSettingsSqlTextArea.setRequestFocusEnabled(true);
		this.theSettingsSqlTextArea.setToolTipText("SQL statement selecting settings");
		this.theSettingsSqlTextArea.setText(null);
		this.theSettingsSqlTextArea.setRows(4);

		final JScrollPane thisNodesSqlScrollPane = new JScrollPane(this.theNodesSqlTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		final JScrollPane thisTreeEdgesSqlScrollPane = new JScrollPane(this.theTreeEdgesSqlTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		final JScrollPane thisEdgesSqlScrollPane = new JScrollPane(this.theEdgesSqlTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		final JScrollPane thisSettingsSqlScrollPane = new JScrollPane(this.theSettingsSqlTextArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		thisNodesSqlScrollPane.setBorder(null);
		thisTreeEdgesSqlScrollPane.setBorder(null);
		thisEdgesSqlScrollPane.setBorder(null);
		thisSettingsSqlScrollPane.setBorder(null);

		final JTabbedPane thisTabbedPane = new JTabbedPane();
		thisTabbedPane.add("Nodes", thisNodesSqlScrollPane);
		thisTabbedPane.add("TreeEdges", thisTreeEdgesSqlScrollPane);
		thisTabbedPane.add("Edges", thisEdgesSqlScrollPane);
		thisTabbedPane.add("Settings", thisSettingsSqlScrollPane);

		this.theNameTable.setToolTipText("Database field representing node id");
		final JScrollPane thisMapsSqlScrollPane = new JScrollPane(this.theNameTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		thisMapsSqlScrollPane.setPreferredSize(new Dimension(500, 100));

		final JButton theOkButton = new JButton();
		theOkButton.setText("Ok");
		theOkButton.addActionListener(new ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(final ActionEvent e)
			{
				SqlDialog.this.ok = true;
				setVisible(false);
			}
		});

		final JButton theCancelButton = new JButton();
		theCancelButton.setText("Cancel");
		theCancelButton.addActionListener(new ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			public void actionPerformed(final ActionEvent e)
			{
				setVisible(false);
			}
		});

		final JButton theLoadButton = new JButton();
		theLoadButton.setText("Load");
		theLoadButton.addActionListener(new ActionListener()
		{
			/*
			 * (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@SuppressWarnings("synthetic-access")
			public void actionPerformed(final ActionEvent e)
			{
				final JFileChooser thisFileChooser = new JFileChooser();
				thisFileChooser.setFileFilter(new javax.swing.filechooser.FileFilter()
				{
					/*
					 * (non-Javadoc)
					 * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
					 */
					public boolean accept(final File thisFile)
					{
						return thisFile.getName().toLowerCase().endsWith(".properties") || thisFile.isDirectory();
					}

					/*
					 * (non-Javadoc)
					 * @see javax.swing.filechooser.FileFilter#getDescription()
					 */
					public String getDescription()
					{
						return "Property files";
					}
				});
				thisFileChooser.setDialogTitle("TreebolicSQl");
				thisFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				thisFileChooser.setCurrentDirectory(new File("."));
				if (thisFileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION)
				{
					try
					{
						SqlDialog.this.thePropertyUrlString = thisFileChooser.getSelectedFile().toURI().toURL().toString();
						load();
					}
					catch (final MalformedURLException thisException)
					{
						// do nothing
					}
				}
			}
		});
		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(theLoadButton);
		thisButtonPanel.add(theCancelButton);
		thisButtonPanel.add(theOkButton);

		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new GridBagLayout());
		thisPanel.add(thisTitleLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 20, 5), 0, 0));

		thisPanel.add(thisURLLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(thisUserLabel, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 1));
		thisPanel.add(thisPasswdLabel, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(thisSqlLabel, new GridBagConstraints(0, 5, 1, 5, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(thisMapLabel, new GridBagConstraints(0, 20, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

		thisPanel.add(this.theURLField, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(this.theUserTextField, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(this.thePasswdField, new GridBagConstraints(1, 4, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 200, 0));
		thisPanel.add(thisTabbedPane, new GridBagConstraints(1, 5, 1, 5, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		thisPanel.add(thisMapsSqlScrollPane, new GridBagConstraints(1, 20, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

		thisPanel.add(thisButtonPanel, new GridBagConstraints(0, 25, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));

		setContentPane(thisPanel);
	}

	/**
	 * Get properties
	 * 
	 * @return properties
	 */
	public Properties getProperties()
	{
		final Properties theseProperties = new Properties();
		theseProperties.put("url", this.theURLField.getText() == null ? "" : this.theURLField.getText());
		theseProperties.put("user", this.theUserTextField.getText());
		theseProperties.put("passwd", new String(this.thePasswdField.getPassword()));
		theseProperties.put("nodesSql", this.theNodesSqlTextArea.getText());
		theseProperties.put("treeedgesSql", this.theTreeEdgesSqlTextArea.getText());
		theseProperties.put("edgesSql", this.theEdgesSqlTextArea.getText());
		theseProperties.put("settingsSql", this.theSettingsSqlTextArea.getText());
		final TableModel thisModel = this.theNameTable.getModel();
		int thisRowIndex = 0;
		for (final String[] thisRow : this.theNameValues)
		{
			final String thisKey = thisRow[0];
			final String thisValue = (String) thisModel.getValueAt(thisRowIndex, 1);
			if (thisValue != null)
			{
				theseProperties.setProperty(thisKey, thisValue);
			}
			thisRowIndex++;
		}
		return theseProperties;
	}

	/**
	 * Set properties
	 * 
	 * @param theseProperties
	 *        properties
	 */
	public void setProperties(final Properties theseProperties)
	{
		this.theURLField.setText((String) theseProperties.get("url"));
		this.theUserTextField.setText((String) theseProperties.get("user"));
		this.thePasswdField.setText((String) theseProperties.get("passwd"));
		this.theNodesSqlTextArea.setText((String) theseProperties.get("nodesSql"));
		this.theTreeEdgesSqlTextArea.setText((String) theseProperties.get("treeEdgesSql"));
		this.theEdgesSqlTextArea.setText((String) theseProperties.get("edgesSql"));
		this.theSettingsSqlTextArea.setText((String) theseProperties.get("settingsSql"));
		final TableModel thisModel = this.theNameTable.getModel();
		int thisRowIndex = 0;
		for (final String[] thisRow : this.theNameValues)
		{
			final String thisKey = thisRow[0];
			final String thisValue = (String) theseProperties.getProperty(thisKey);
			thisModel.setValueAt(thisValue, thisRowIndex, 1);
			thisRowIndex++;
		}
	}

	/**
	 * Load properties
	 */
	public void load()
	{
		if (this.thePropertyUrlString == null)
			return;

		try
		{
			final URL thisUrl = new URL(this.thePropertyUrlString);
			final Properties theseProperties = SqlProperties.load(thisUrl);
			if (theseProperties != null)
			{
				setProperties(theseProperties);
			}
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
	}

	/**
	 * Save properties
	 */
	public void save()
	{
		if (this.thePropertyUrlString == null)
			return;
		try
		{
			final File thisFile = new File(new URL(this.thePropertyUrlString).toURI());
			final Properties theseProperties = getProperties();
			SqlProperties.save(theseProperties, thisFile.getAbsolutePath());
		}
		catch (final MalformedURLException thisException)
		{
			// do nothing
		}
		catch (final URISyntaxException thisException)
		{
			// do nothing
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			pack();
			SqlDialog.center(this);
		}
		super.setVisible(thisFlag);
	}

	/**
	 * Center on screen
	 * 
	 * @param thisComponent
	 *        component to center
	 */
	static public void center(final Component thisComponent)
	{
		final Dimension thisScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
		final Dimension thisComponentSize = thisComponent.getSize();
		if (thisComponentSize.height > thisScreenSize.height)
		{
			thisComponentSize.height = thisScreenSize.height;
		}
		if (thisComponentSize.width > thisScreenSize.width)
		{
			thisComponentSize.width = thisScreenSize.width;
		}
		thisComponent.setLocation((thisScreenSize.width - thisComponentSize.width) / 2, (thisScreenSize.height - thisComponentSize.height) / 2);
	}

	/**
	 * Main
	 * 
	 * @param theseArgs
	 *        arguments
	 */
	static public void main(final String[] theseArgs)
	{
		UIManager.put("swing.boldMetal", false);
		final SqlDialog thisDialog = new SqlDialog(theseArgs.length > 0 ? theseArgs[0] : null);
		thisDialog.setModal(true);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thisDialog.setVisible(true);
		if (thisDialog.ok)
		{
			thisDialog.save();
		}
		System.exit(0);
	}
}
