/**
 * Title: Treebolic WordNet provider
 * Description: Treebolic WordNet Provider
 * Version: 2.x
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.wn;

import java.awt.Color;
import java.awt.Image;
import javax.swing.ImageIcon;

import edu.princeton.wordnet.WordNet;

/**
 * Node for WordNet link
 * 
 * @author Bernard Bou
 */
public class LinkNode extends Node
{
	// S T A T I C . D A T A

	/**
	 * Images
	 */
	private static final Image theImages[] = new Image[WordNet.MAXLINKTYPE + 1];

	static
	{
		theImages[WordNet.ANTPTR] = new ImageIcon(LinkNode.class.getResource("images/antonym.gif")).getImage();
		theImages[WordNet.HYPERPTR] = new ImageIcon(LinkNode.class.getResource("images/hypernym.gif")).getImage();
		theImages[WordNet.HYPOPTR] = new ImageIcon(LinkNode.class.getResource("images/hyponym.gif")).getImage();
		theImages[WordNet.ENTAILPTR] = new ImageIcon(LinkNode.class.getResource("images/entail.gif")).getImage();
		theImages[WordNet.CAUSETO] = new ImageIcon(LinkNode.class.getResource("images/cause.gif")).getImage();
		theImages[WordNet.MERONYM] = new ImageIcon(LinkNode.class.getResource("images/meronym.gif")).getImage();
		theImages[WordNet.HOLONYM] = new ImageIcon(LinkNode.class.getResource("images/holonym.gif")).getImage();
		theImages[WordNet.SIMPTR] = new ImageIcon(LinkNode.class.getResource("images/similar.gif")).getImage();
		theImages[WordNet.SEEALSOPTR] = new ImageIcon(LinkNode.class.getResource("images/also.gif")).getImage();
		theImages[WordNet.DERIVATION] = new ImageIcon(LinkNode.class.getResource("images/derivation.gif")).getImage();
		theImages[WordNet.CLASSIFICATION] = new ImageIcon(LinkNode.class.getResource("images/domain.gif")).getImage();
		theImages[WordNet.CLASS] = new ImageIcon(LinkNode.class.getResource("images/domainterm.gif")).getImage();

		theImages[WordNet.ISMEMBERPTR] = new ImageIcon(LinkNode.class.getResource("images/member.holonym.gif")).getImage();
		theImages[WordNet.ISSTUFFPTR] = new ImageIcon(LinkNode.class.getResource("images/substance.holonym.gif")).getImage();
		theImages[WordNet.ISPARTPTR] = new ImageIcon(LinkNode.class.getResource("images/part.holonym.gif")).getImage();

		theImages[WordNet.HASMEMBERPTR] = new ImageIcon(LinkNode.class.getResource("images/member.meronym.gif")).getImage();
		theImages[WordNet.HASSTUFFPTR] = new ImageIcon(LinkNode.class.getResource("images/substance.meronym.gif")).getImage();
		theImages[WordNet.HASPARTPTR] = new ImageIcon(LinkNode.class.getResource("images/part.meronym.gif")).getImage();

		theImages[WordNet.ATTRIBUTE] = new ImageIcon(LinkNode.class.getResource("images/attribute.gif")).getImage();
		theImages[WordNet.VERBGROUP] = new ImageIcon(LinkNode.class.getResource("images/verb.group.gif")).getImage();
		theImages[WordNet.PERTPTR] = new ImageIcon(LinkNode.class.getResource("images/pertainym.gif")).getImage();
		theImages[WordNet.PPLPTR] = new ImageIcon(LinkNode.class.getResource("images/participle.gif")).getImage();

		theImages[WordNet.CLASSIF_CATEGORY] = new ImageIcon(LinkNode.class.getResource("images/domain.category.gif")).getImage();
		theImages[WordNet.CLASSIF_USAGE] = new ImageIcon(LinkNode.class.getResource("images/domain.usage.gif")).getImage();
		theImages[WordNet.CLASSIF_REGIONAL] = new ImageIcon(LinkNode.class.getResource("images/domain.region.gif")).getImage();
		theImages[WordNet.CLASS_CATEGORY] = new ImageIcon(LinkNode.class.getResource("images/domain.member.category.gif")).getImage();
		theImages[WordNet.CLASS_USAGE] = new ImageIcon(LinkNode.class.getResource("images/domain.member.usage.gif")).getImage();
		theImages[WordNet.CLASS_REGIONAL] = new ImageIcon(LinkNode.class.getResource("images/domain.member.region.gif")).getImage();

		theImages[WordNet.INSTANCE] = new ImageIcon(LinkNode.class.getResource("images/instance.hypernym.gif")).getImage();
		theImages[WordNet.INSTANCES] = new ImageIcon(LinkNode.class.getResource("images/instance.hyponym.gif")).getImage();
	}

	// D A T A

	/**
	 * Link index
	 */
	private int theLinkIdx;

	/**
	 * Background color
	 */
	private static final Color theLinkBackColor = Color.BLUE;

	/**
	 * Foreground color
	 */
	private static final Color theLinkForeColor = Color.WHITE;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 * 
	 * @param thisParentNode
	 *        parent node
	 * @param thisLinkIdx
	 *        link index
	 */
	public LinkNode(Node thisParentNode, int thisLinkIdx)
	{
		super(thisParentNode, WordNet.theSearchTypeString[thisLinkIdx], theImages[thisLinkIdx + 1], theLinkBackColor, theLinkForeColor);
		theLinkIdx = thisLinkIdx;
		if (getImage() != null)
			setLabel(null);
		setEdgeLabel(WordNet.theSearchTypeString[theLinkIdx]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see treebolic.node.INode#getContent()
	 */
	public String getContent()
	{
		return WordNet.theSearchTypeString[theLinkIdx];
	}
}
