/**
 * Title: Treebolic WordNet provider
 * Description: Treebolic WordNet Provider
 * Version: 2.x
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.wn;

import java.awt.Color;
import java.awt.Image;
import javax.swing.ImageIcon;

import treebolic.model.IEdge;

import edu.princeton.wordnet.Synset;

/**
 * Item node
 * 
 * @author Bernard Bou
 */
public class WordNode extends Node
{
	/**
	 * Synset
	 */
	private Synset theSynset;

	/**
	 * Link
	 */
	private String theLink;

	/**
	 * Image
	 */
	private static final Image theImage = new ImageIcon(WordNode.class.getResource("images/item.gif")).getImage();

	/**
	 * Item foreground color
	 */
	private static final Color theWordForeColor = Color.BLACK;

	/**
	 * Item background colors
	 */
	private static final Color[] theWordBackColors = new Color[5];

	static
	{
		theWordBackColors[0] = new Color(0xffff80);
		theWordBackColors[1] = new Color(0xffff90);
		theWordBackColors[2] = new Color(0xffffa0);
		theWordBackColors[3] = new Color(0xffffb0);
		theWordBackColors[4] = new Color(0xffffc0);
	}

	/**
	 * Constructor
	 * 
	 * @param thisParentNode
	 *        parent node
	 * @param thisWord
	 *        target word
	 * @param thisSynset
	 *        synset
	 * @param thisLevel
	 *        recursion level
	 */
	public WordNode(Node thisParentNode, String thisWord, Synset thisSynset, int thisLevel)
	{
		super(thisParentNode, thisWord, theImage, theWordBackColors[thisLevel % theWordBackColors.length], theWordForeColor);
		theSynset = thisSynset;
		theLink = "wordnet:" + thisWord;
		theEdgeColor = Color.BLUE;
		theEdgeStyle = IEdge.DOT | IEdge.TOTRIANGLE;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see treebolic.node.INode#getContent()
	 */
	public String getContent()
	{
		return theSynset != null ? theSynset.defn : theLabel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see treebolic.node.INode#getLink()
	 */
	public String getLink()
	{
		return theLink;
	}
}
