/**
 * Title: Treebolic provider
 * Description: Treebolic Provider
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.provider.xml.dom;

import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;

import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.Tree;

public class Provider implements IProvider
{
	// D A T A

	/**
	 * Context used to query for Url
	 */
	protected IProviderContext theContext;

	/**
	 * Url
	 */
	protected URL theUrl;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public Provider()
	{
		this.theUrl = null;
	}

	// M A K E

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#set(treebolic.model.IProviderContext)
	 */
	@Override
	public void connect(final IProviderContext thisContext)
	{
		this.theContext = thisContext;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeTree(java.lang.String, boolean)
	 */
	@Override
	public Tree makeTree(final String thisSource, final boolean checkRecursion)
	{
		final URL thisUrl = makeUrl(thisSource);
		if (thisUrl == null)
			return null;

		// direct recursion prevention
		if (checkRecursion && thisUrl.equals(this.theUrl))
		{
			this.theContext.putMessage("Recursion: " + thisUrl.toString());
			return null;
		}

		this.theUrl = thisUrl;
		this.theContext.putProgress("Loading ..." + thisUrl.toString(), false);
		final Tree thisTree = makeTree(thisUrl);
		this.theContext.putProgress("Loaded ..." + thisUrl.toString(), false);
		return thisTree;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeModel(java.lang.String)
	 */
	@Override
	public Model makeModel(final String thisSource)
	{
		final URL thisUrl = makeUrl(thisSource);
		if (thisUrl == null)
			return null;

		this.theUrl = thisUrl;
		this.theContext.putProgress("Loading ..." + thisUrl.toString(), false);
		final Model thisModel = makeModel(thisUrl);
		this.theContext.putProgress("Loaded ..." + thisUrl.toString(), false);
		return thisModel;
	}

	/**
	 * Make Url
	 * 
	 * @param thatSource
	 *        source
	 * @return URL
	 */
	private URL makeUrl(final String thatSource)
	{
		String thisSource = thatSource;

		// try as is
		try
		{
			return new URL(thisSource);
		}
		catch (final MalformedURLException e)
		{
			// do nothing
		}

		while (true)
		{
			// give the context a chance
			final URL thisUrl = this.theContext.makeURL(thisSource);
			if (thisUrl != null)
				return thisUrl;

			// ask
			thisSource = ask("Source: " + thisSource + "\nEnter data URL:\n");
			if (thisSource == null || thisSource.isEmpty())
			{
				// give up
				inform("Source is null.\nAborting.");
				return null;
			}
		}
	}

	// P A R S E

	/**
	 * Make model from url
	 * 
	 * @param thisUrl
	 *        url
	 * @return model
	 */
	protected Model makeModel(final URL thisUrl)
	{
		final Document thisDocument = makeDocument(thisUrl);
		if (thisDocument == null)
			return null;
		return new DocumentAdapter(this).makeModel(thisDocument);
	}

	/**
	 * Make tree from url
	 * 
	 * @param thisUrl
	 *        url
	 * @return tree
	 */
	protected Tree makeTree(final URL thisUrl)
	{
		final Document thisDocument = makeDocument(thisUrl);
		if (thisDocument == null)
			return null;
		return new DocumentAdapter(this).makeTree(thisDocument);
	}

	/**
	 * Make DOM document from its Url
	 * 
	 * @param thisUrl
	 *        document url
	 * @return DOM document
	 */
	protected Document makeDocument(final URL thisUrl)
	{
		return new Parser().makeDocument(thisUrl, null);
	}

	/**
	 * Ask
	 * 
	 * @param thisMessage
	 *        message
	 * @return input
	 */
	protected String ask(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n");
		return JOptionPane.showInputDialog(null, theseLines);
	}

	/**
	 * Inform dialog
	 * 
	 * @param thisMessage
	 *        message
	 */
	protected void inform(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n");
		JOptionPane.showMessageDialog(null, theseLines, "Treebolic XML provider", JOptionPane.WARNING_MESSAGE);
	}
}
