/**
 * Title: Treebolic provider
 * Description: Treebolic Provider
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */

package treebolic.provider.xml.fast;

import java.net.URL;

import javax.swing.JOptionPane;

import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.Tree;

/**
 * Provider factory implementation
 * 
 * @author Bernard Bou
 */
public class Provider implements IProvider
{
	// D A T A

	/**
	 * Context used to query for Url
	 */
	private IProviderContext theContext;

	/**
	 * Url
	 */
	protected URL theUrl;

	// CO N S T R U C T O R

	/**
	 * Constructor
	 */
	public Provider()
	{
		this.theUrl = null;
	}

	// M A K E

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#set(treebolic.model.IProviderContext)
	 */
	@Override
	public void connect(final IProviderContext thisContext)
	{
		this.theContext = thisContext;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeModel(java.lang.String)
	 */
	@Override
	public Model makeModel(final String thisSource)
	{
		final URL thisUrl = makeUrl(thisSource);
		if (thisUrl == null)
			return null;

		this.theUrl = thisUrl;
		this.theContext.putProgress("Loading ..." + thisUrl.toString(), false);
		final Model thisModel = parseModel(thisUrl);
		this.theContext.putProgress("Loaded " + thisUrl.toString(), false);
		return thisModel;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeTree(java.lang.String, boolean)
	 */
	@Override
	public Tree makeTree(final String thisSource, final boolean checkRecursion)
	{
		final URL thisUrl = makeUrl(thisSource);
		if (thisUrl == null)
			return null;

		// direct recursion prevention
		if (checkRecursion && thisUrl.equals(this.theUrl))
		{
			this.theContext.putMessage("Recursion: " + thisUrl.toString());
			return null;
		}

		this.theUrl = thisUrl;
		this.theContext.putProgress("Loading ..." + thisUrl.toString(), false);
		final Tree thisTree = parseTree(thisUrl);
		this.theContext.putProgress("Loaded " + thisUrl.toString(), false);
		return thisTree;
	}

	/**
	 * Make Url
	 * 
	 * @param thatSource
	 *        source
	 * @return URL
	 */
	private URL makeUrl(final String thatSource)
	{
		String thisSource = thatSource;
		while (true)
		{
			// process
			final URL thisUrl = this.theContext.makeURL(thisSource);
			if (thisUrl != null)
				return thisUrl;

			// ask
			thisSource = ask("Source: " + thisSource + "\nEnter data URL:\n");
			if (thisSource == null || thisSource.isEmpty())
			{
				// give up
				inform("Source is null.\nAborting.");
				return null;
			}
		}
	}

	// P A R S E

	/**
	 * Populate model with data from URL
	 * 
	 * @param thisURL
	 *        url
	 * @return model or null
	 */
	private Model parseModel(final URL thisURL)
	{
		// parse
		// assumes URL is valid
		final Parser thisParser = new Parser(thisURL, this);
		if (thisParser.isValid())
			return thisParser.parseModel();
		return null;
	}

	/**
	 * Populate tree with data from URL
	 * 
	 * @param thisURL
	 *        url
	 * @return tree or null
	 */
	private Tree parseTree(final URL thisURL)
	{
		// parse
		// assumes URL is valid
		final Parser thisParser = new Parser(thisURL, this);
		if (thisParser.isValid())
			return thisParser.parseTree();
		return null;
	}

	/**
	 * Ask dialog
	 * 
	 * @param thisMessage
	 *        message
	 * @return input
	 */
	private String ask(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n");
		return JOptionPane.showInputDialog(null, theseLines);
	}

	/**
	 * Inform dialog
	 * 
	 * @param thisMessage
	 *        message
	 */
	private void inform(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n");
		JOptionPane.showMessageDialog(null, theseLines, "Treebolic XML provider", JOptionPane.WARNING_MESSAGE);
	}
}
