/**
 * Title: Treebolic
 * Description: Treebolic
 * Version: 2.x
 * Copyright: (c) 2001-2008
 * Terms of use: see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.provider.xml.graph;

import java.net.URL;

import treebolic.model.IProvider;
import treebolic.model.IProviderContext;
import treebolic.model.Model;
import treebolic.model.Tree;

/**
 * Graph provider
 * 
 * @author Bernard Bou
 */
public abstract class AbstractProvider implements IProvider
{
	// D A T A

	/**
	 * Provider client
	 */
	protected IProviderContext theContext;

	// C O N S T R U C T O R

	/**
	 * Constructor
	 */
	public AbstractProvider()
	{
		// do nothing
	}

	// A B S T R A C T . M E T H O D S

	/**
	 * Fallback hook to determine source
	 * 
	 * @param thisSource
	 *        passed source
	 * @return source
	 */
	abstract public String getSource(String thisSource);

	/**
	 * Get parser
	 * 
	 * @return abstract parser
	 */
	abstract public AbstractParser getParser();

	// P R O V I D E R . I N T E R F A C E

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#set(treebolic.model.IProviderContext)
	 */
	@Override
	public void connect(final IProviderContext thisContext)
	{
		this.theContext = thisContext;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeModel(java.lang.String)
	 */
	@Override
	public Model makeModel(final String thatSource)
	{
		// get source
		final String thisSource = getSource(thatSource);

		// parse source
		if (thisSource != null)
		{
			// URL
			final URL thisUrl = this.theContext.makeURL(thisSource);
			if (thisUrl != null)
			{
				// parse
				this.theContext.putProgress("Loading ..." + thisSource, false);
				final Model thisModel = getParser().parseModel(thisUrl);
				if (thisModel != null)
				{
					this.theContext.putProgress("Loaded " + thisSource, false);
					return thisModel;
				}
				this.theContext.putMessage("Cannot load " + thisSource);
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisSource);
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.model.IProvider#makeTree(java.lang.String, boolean)
	 */
	@Override
	public Tree makeTree(final String thatSource, final boolean checkRecursion)
	{
		// get source
		final String thisSource = getSource(thatSource);

		// parse XML file
		if (thisSource != null)
		{
			// URL
			final URL thisUrl = this.theContext.makeURL(thisSource);
			if (thisUrl != null)
			{
				// parse
				this.theContext.putProgress("Loading ..." + thisSource, false);
				final Tree thisTree = getParser().parseTree(thisUrl);
				if (thisTree != null)
				{
					this.theContext.putProgress("Loaded " + thisSource, false);
					return thisTree;
				}
				this.theContext.putMessage("Cannot load" + thisSource);
			}
			else
			{
				this.theContext.putMessage("Bad URL " + thisSource);
			}
		}
		return null;
	}
}
