/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.site;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import org.w3c.dom.Document;

public class FileSiteMaker extends SiteMaker
{
	/**
	 * Path url
	 */
	private final String thePathUrl;

	/**
	 * Extra path
	 */
	private String theXtraPath = ""; //$NON-NLS-1$

	/**
	 * Constructor
	 * 
	 * @param thisDocument
	 *        document
	 * @param thisRepositoryFolder
	 *        repository folder (source)
	 * @param thisImageFolder
	 *        image folder (source)
	 * @param thisPath
	 *        path (source)
	 * @param thisHtmlFile
	 *        HTML file (target)
	 * @param thisXmlFile
	 *        XML file (target)
	 * @param thisTitle
	 *        HTML file title (target)
	 * @throws URISyntaxException
	 * @throws IOException
	 */
	public FileSiteMaker(final Document thisDocument, final String thisRepositoryFolder, final String thisImageFolder, final String thisPath, final String thisHtmlFile, final String thisXmlFile, final String thisTitle) throws URISyntaxException,
			IOException
	{
		super(thisDocument, thisRepositoryFolder, thisImageFolder, thisHtmlFile, thisXmlFile, thisTitle);

		final File thisFolder = new File(thisPath);
		if (!thisFolder.exists() || !thisFolder.isDirectory())
			throw new FileNotFoundException(thisPath + " not found"); //$NON-NLS-1$

		this.thePathUrl = thisFolder.toURI().toURL().toString();
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.site.SiteMaker#connect()
	 */
	@Override
	public boolean connect()
	{
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.site.SiteMaker#disconnect()
	 */
	@Override
	public boolean disconnect()
	{
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.site.SiteMaker#makeOutStream(java.lang.String)
	 */
	@Override
	public OutputStream makeOutStream(final String thisFilename) throws MalformedURLException, IOException, URISyntaxException
	{
		final URL thisPathUrl = new URL(this.thePathUrl);
		final URL thisUrl = new URL(thisPathUrl, this.theXtraPath + thisFilename);
		final File thisFile = new File(thisUrl.toURI());
		return new FileOutputStream(thisFile);
	}

	/*
	 * (non-Javadoc)
	 * @see treebolic.site.SiteMaker#changeFolder(java.lang.String)
	 */
	@Override
	public boolean changeFolder(final String thisRelativeFolder) throws MalformedURLException, URISyntaxException
	{
		final String thisXtraPath = thisRelativeFolder;
		if (thisXtraPath == null || thisXtraPath.isEmpty())
			return true;
		final URL thisPathUrl = new URL(this.thePathUrl);
		final URL thisUrl = new URL(thisPathUrl, thisXtraPath);
		final File thisFolder = new File(thisUrl.toURI());
		if (!thisFolder.exists())
		{
			if (!thisFolder.mkdirs())
				return false;
		}
		// update extra path
		this.theXtraPath = thisXtraPath;
		if (this.theXtraPath != "" && !this.theXtraPath.endsWith("/"))
		{
			this.theXtraPath += "/"; //$NON-NLS-1$
		}
		return true;
	}
}
