/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.site;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Observable;
import java.util.Observer;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.w3c.dom.Document;

import treebolic.Messages;
import treebolic.commons.FileDialogs;
import treebolic.commons.Persist;
import treebolic.commons.Utils;
import treebolic.dialogs.main.XSiteDialog;
import treebolic.generator.Pair;
import treebolic.provider.xml.dom.Parser;

/**
 * Make site dialog
 * 
 * @author Bernard Bou
 */
public class MakeSiteDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	/**
	 * DOM document (source)
	 */
	private Document theDocument;

	// C O M P O N E N T S

	/**
	 * Site information
	 */
	private JLabel theSiteLabel;

	/**
	 * Template repository information
	 */
	private JLabel theRepositoryLabel;

	/**
	 * Image repository information
	 */
	private JLabel theImageRepositoryLabel;

	/**
	 * Title input
	 */
	private JTextField theTitleTextField;

	/**
	 * XML file input
	 */
	private JTextField theXMLFileTextField;

	/**
	 * HTML file input
	 */
	private JTextField theHTMLFileTextField;

	/**
	 * Progress label
	 */
	private JLabel theProgressLabel;

	/**
	 * Progress bar
	 */
	private JProgressBar theProgressBar;

	/**
	 * Wheter to chain with run site
	 */
	private JCheckBox theRunCheckBox;

	/**
	 * Properties (input/output)
	 */
	private final Properties theProperties;

	// C O N S T R U C T OR

	/**
	 * Constructor
	 */
	public MakeSiteDialog(final Properties theseProperties)
	{
		super();
		this.theProperties = theseProperties;
		initialize();
	}

	/**
	 * Initialize
	 */
	private void initialize()
	{
		setTitle(Messages.getString("MakeSiteDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// checkbox
		this.theRunCheckBox = new JCheckBox(Messages.getString("MakeSiteDialog.8")); //$NON-NLS-1$

		// buttons
		final JButton thisTransferButton = new JButton();
		thisTransferButton.setText(Messages.getString("MakeSiteDialog.2")); //$NON-NLS-1$
		final JButton thisDestinationButton = new JButton();
		thisDestinationButton.setText(Messages.getString("MakeSiteDialog.3")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton();
		thisCancelButton.setText(Messages.getString("MakeSiteDialog.4")); //$NON-NLS-1$
		final JButton thisBrowseRepositoryButton = new JButton();
		thisBrowseRepositoryButton.setText(Messages.getString("MakeSiteDialog.5")); //$NON-NLS-1$
		final JButton thisBrowseImageRepositoryButton = new JButton();
		thisBrowseImageRepositoryButton.setText(Messages.getString("MakeSiteDialog.5")); //$NON-NLS-1$
		final JButton thisModeButton = new JButton();
		thisModeButton.setText(Messages.getString("MakeSiteDialog.5")); //$NON-NLS-1$

		// images
		final Icon thisIcon = new ImageIcon(MakeSiteDialog.class.getResource("images/sitemake.png")); //$NON-NLS-1$
		final JLabel thisImageLabel = new JLabel();
		thisImageLabel.setBackground(Color.RED);
		thisImageLabel.setIcon(thisIcon);
		thisImageLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisImageLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisImageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		thisImageLabel.setText(Messages.getString("MakeSiteDialog.1")); //$NON-NLS-1$

		// progress
		this.theProgressBar = new JProgressBar();
		this.theProgressLabel = new JLabel(Messages.getString("MakeSiteDialog.6")); //$NON-NLS-1$

		// labels
		this.theRepositoryLabel = new JLabel();
		this.theRepositoryLabel.setEnabled(false);
		this.theImageRepositoryLabel = new JLabel();
		this.theImageRepositoryLabel.setEnabled(false);
		this.theSiteLabel = new JLabel();
		this.theSiteLabel.setEnabled(false);

		// textfields
		this.theTitleTextField = new JTextField(16);
		this.theXMLFileTextField = new JTextField(16);
		this.theHTMLFileTextField = new JTextField(16);

		// assemble
		final JPanel thisDataPanel = new JPanel();
		thisDataPanel.setLayout(new GridBagLayout());
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.11")), new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.12")), new GridBagConstraints(0, 1, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.13")), new GridBagConstraints(0, 2, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.14")), new GridBagConstraints(0, 3, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.15")), new GridBagConstraints(0, 4, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$
		thisDataPanel.add(new JLabel(Messages.getString("MakeSiteDialog.16")), new GridBagConstraints(0, 5, 1, 1, 0., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 20, 0, 10), 0, 0)); //$NON-NLS-1$

		thisDataPanel.add(this.theTitleTextField, new GridBagConstraints(1, 0, 2, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		thisDataPanel.add(this.theXMLFileTextField, new GridBagConstraints(1, 1, 2, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		thisDataPanel.add(this.theHTMLFileTextField, new GridBagConstraints(1, 2, 2, 1, 1., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 20), 0, 0));
		thisDataPanel.add(this.theSiteLabel, new GridBagConstraints(1, 3, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		thisDataPanel.add(this.theRepositoryLabel, new GridBagConstraints(1, 4, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));
		thisDataPanel.add(this.theImageRepositoryLabel, new GridBagConstraints(1, 5, 1, 1, 0., 0., GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 10, 0, 10), 0, 0));

		thisDataPanel.add(thisModeButton, new GridBagConstraints(2, 3, 1, 1, 10., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		thisDataPanel.add(thisBrowseRepositoryButton, new GridBagConstraints(2, 4, 1, 1, 10., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));
		thisDataPanel.add(thisBrowseImageRepositoryButton, new GridBagConstraints(2, 5, 1, 1, 10., 0., GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 10, 0, 20), 0, 0));

		final JPanel thisProgressPanel = new JPanel();
		thisProgressPanel.setLayout(new BoxLayout(thisProgressPanel, BoxLayout.Y_AXIS));
		thisProgressPanel.setLayout(new GridBagLayout());
		thisProgressPanel.add(this.theProgressLabel, new GridBagConstraints(0, 0, 1, 1, 0., 0., GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(10, 20, 0, 20), 0, 0));
		thisProgressPanel.add(this.theProgressBar, new GridBagConstraints(0, 1, 1, 1, 1., 0., GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 20, 10, 20), 0, 0));

		final JPanel thisCommandPanel = new JPanel();
		thisCommandPanel.add(this.theRunCheckBox);
		thisCommandPanel.add(thisCancelButton);
		thisCommandPanel.add(thisTransferButton);

		// events
		thisBrowseRepositoryButton.addActionListener(new java.awt.event.ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final String thisPath = FileDialogs.getFolder(MakeSiteDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisPath != null && !thisPath.isEmpty())
				{
					MakeSiteDialog.this.theProperties.setProperty("repository", thisPath); //$NON-NLS-1$
					MakeSiteDialog.this.theRepositoryLabel.setText(thisPath);
				}
				else
				{
					MakeSiteDialog.this.theProperties.remove("repository"); //$NON-NLS-1$
					MakeSiteDialog.this.theRepositoryLabel.setText("<internal>"); //$NON-NLS-1$
				}
			}
		});

		thisBrowseImageRepositoryButton.addActionListener(new java.awt.event.ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final String thisPath = FileDialogs.getFolder(MakeSiteDialog.this.theProperties.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
				if (thisPath != null)
				{
					MakeSiteDialog.this.theProperties.setProperty("images", thisPath); //$NON-NLS-1$
					MakeSiteDialog.this.theImageRepositoryLabel.setText(thisPath);
				}
			}
		});

		thisModeButton.addActionListener(new java.awt.event.ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final XSiteDialog thisDialog = new XSiteDialog(MakeSiteDialog.this.theProperties);
				thisDialog.setModal(true);
				thisDialog.setVisible(true);
				MakeSiteDialog.this.theSiteLabel.setText(getSiteUrlString());
			}
		});

		thisTransferButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent e)
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					@SuppressWarnings("synthetic-access")
					@Override
					public void run()
					{
						transfer();
					}
				});
			}
		});

		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final java.awt.event.ActionEvent e)
			{
				setVisible(false);
			}
		});

		this.theTitleTextField.getDocument().addDocumentListener(new DocumentListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void removeUpdate(final DocumentEvent e)
			{
				final String thisTitle = MakeSiteDialog.this.theTitleTextField.getText();
				MakeSiteDialog.this.theHTMLFileTextField.setText(thisTitle + ".html"); //$NON-NLS-1$
				MakeSiteDialog.this.theXMLFileTextField.setText(thisTitle + ".xml"); //$NON-NLS-1$
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public void insertUpdate(final DocumentEvent e)
			{
				final String thisTitle = MakeSiteDialog.this.theTitleTextField.getText();
				MakeSiteDialog.this.theHTMLFileTextField.setText(thisTitle + ".html"); //$NON-NLS-1$
				MakeSiteDialog.this.theXMLFileTextField.setText(thisTitle + ".xml"); //$NON-NLS-1$
			}

			@Override
			public void changedUpdate(final DocumentEvent e)
			{
				// do nothing
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisImageLabel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisDataPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisProgressPanel);
		thisPanel.add(thisCommandPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	/**
	 * Set document
	 * 
	 * @param thisDocument
	 *        document
	 */
	public void setDocument(final Document thisDocument)
	{
		this.theDocument = thisDocument;
	}

	// H E L P E R S

	/**
	 * Get destination Url
	 * 
	 * @return site url string
	 */
	private String getSiteUrlString()
	{
		final String thisMode = this.theProperties.getProperty("mode"); //$NON-NLS-1$
		if ("NET".equals(thisMode)) //$NON-NLS-1$
		{
			// ftp-specific check
			final String thisServer = this.theProperties.getProperty("server", ""); //$NON-NLS-1$ //$NON-NLS-2$
			final String thisDirectory = this.theProperties.getProperty("directory", ""); //$NON-NLS-1$ //$NON-NLS-2$
			final String thisLogin = this.theProperties.getProperty("login", ""); //$NON-NLS-1$ //$NON-NLS-2$
			final String thisPassword = this.theProperties.getProperty("password", ""); //$NON-NLS-1$ //$NON-NLS-2$
			if (!thisServer.isEmpty())
			{
				final StringBuffer thisUrlBuffer = new StringBuffer();
				thisUrlBuffer.append("ftp://"); //$NON-NLS-1$
				thisUrlBuffer.append(thisLogin);
				thisUrlBuffer.append(":"); //$NON-NLS-1$
				thisUrlBuffer.append(thisPassword);
				thisUrlBuffer.append("@"); //$NON-NLS-1$
				thisUrlBuffer.append(thisServer);
				thisUrlBuffer.append("/"); //$NON-NLS-1$
				if (thisDirectory != null && !thisDirectory.isEmpty())
				{
					thisUrlBuffer.append(thisDirectory);
					thisUrlBuffer.append("/"); //$NON-NLS-1$
				}
				thisUrlBuffer.append(";type=i"); //$NON-NLS-1$
				return thisUrlBuffer.toString();
			}
		}
		else
		{
			// file system-specific check
			final String thisPath = this.theProperties.getProperty("path", ""); //$NON-NLS-1$ //$NON-NLS-2$
			if (thisPath != null)
			{
				final File thisDirectory = new File(thisPath);
				try
				{
					return thisDirectory.toURI().toURL().toString();
				}
				catch (final MalformedURLException thisException)
				{
					// do nothing
				}
			}
		}
		return ""; //$NON-NLS-1$
	}

	// T R A N S F E R

	/**
	 * Transfer
	 */
	private void transfer()
	{
		System.err.println("FTP transfer"); //$NON-NLS-1$
		boolean hasFailed = false;
		try
		{
			// repository check
			String thisRepository = this.theProperties.getProperty("repository"); //$NON-NLS-1$
			if (thisRepository != null && !thisRepository.isEmpty())
			{
				File thisRepositoryFolder = null;
				if (thisRepository.startsWith("file:"))
				{
					thisRepositoryFolder = new File(new URI(thisRepository));
				}
				else
				{
					thisRepositoryFolder = new File(thisRepository);
				}
				if (!thisRepositoryFolder.exists() || !thisRepositoryFolder.isDirectory())
				{
					inform(Messages.getString("MakeSiteDialog.30") + thisRepository + Messages.getString("MakeSiteDialog.31")); //$NON-NLS-1$ //$NON-NLS-2$
					thisRepository = null;
				}
				else
				{
					thisRepository = thisRepositoryFolder.toURI().toURL().toString();
				}
			}
			if (thisRepository == null || thisRepository.isEmpty())
			{
				thisRepository = SiteMaker.class.getResource("repository/").toURI().toURL().toString(); //$NON-NLS-1$
			}

			// image repository check
			String thisImageRepository = this.theProperties.getProperty("images"); //$NON-NLS-1$
			if (thisImageRepository != null && !thisImageRepository.isEmpty())
			{
				File thisImageRepositoryFolder = null;
				if (thisImageRepository.startsWith("file:"))
				{
					thisImageRepositoryFolder = new File(new URI(thisImageRepository));
				}
				else
				{
					thisImageRepositoryFolder = new File(thisImageRepository);
				}
				if (!thisImageRepositoryFolder.exists() || !thisImageRepositoryFolder.isDirectory())
				{
					inform(Messages.getString("MakeSiteDialog.30") + thisImageRepository + Messages.getString("MakeSiteDialog.31")); //$NON-NLS-1$ //$NON-NLS-2$
					thisImageRepository = null;
				}
				else
				{
					thisImageRepository = thisImageRepositoryFolder.toURI().toURL().toString();
				}
			}
			if (thisImageRepository == null || thisImageRepository.isEmpty())
			{
				thisImageRepository = new File(System.getProperty("user.home") + "/images").toURI().toURL().toString(); //$NON-NLS-1$ //$NON-NLS-2$
				inform(Messages.getString("MakeSiteDialog.32") + thisImageRepository + Messages.getString("MakeSiteDialog.33") + thisImageRepository); //$NON-NLS-1$ //$NON-NLS-2$
			}

			// data check
			final String thisHTMLFile = this.theHTMLFileTextField.getText();
			final String thisXMLFile = this.theXMLFileTextField.getText();
			final String thisTitle = this.theTitleTextField.getText();
			if (thisHTMLFile.isEmpty() || thisXMLFile.isEmpty())
				return;

			// create site maker
			SiteMaker thisSiteMaker = null;
			final String thisMode = this.theProperties.getProperty("mode"); //$NON-NLS-1$
			if ("NET".equals(thisMode)) //$NON-NLS-1$
			{
				// ftp-specific check
				final String thisServer = this.theProperties.getProperty("server"); //$NON-NLS-1$
				final String thisDirectory = this.theProperties.getProperty("directory"); //$NON-NLS-1$
				final String thisLogin = this.theProperties.getProperty("login"); //$NON-NLS-1$
				final String thisPassword = this.theProperties.getProperty("password"); //$NON-NLS-1$
				if (thisServer.isEmpty() || thisLogin.isEmpty())
					return;

				// create
				thisSiteMaker = new FtpSiteMaker(this.theDocument, thisRepository, thisImageRepository, thisHTMLFile, thisXMLFile, thisTitle, thisServer, thisDirectory, thisLogin, thisPassword);
			}
			else
			{
				// file system-specific check
				final String thisPath = this.theProperties.getProperty("path"); //$NON-NLS-1$
				if (thisPath != null)
				{
					final File thisFolder = new File(thisPath);
					if (!thisFolder.exists())
					{
						inform(Messages.getString("MakeSiteDialog.40") + thisPath + Messages.getString("MakeSiteDialog.41")); //$NON-NLS-1$ //$NON-NLS-2$
						if (!thisFolder.mkdirs())
						{
							inform(Messages.getString("MakeSiteDialog.40") + thisPath + Messages.getString("MakeSiteDialog.42")); //$NON-NLS-1$ //$NON-NLS-2$
							return;
						}
					}
				}

				// create
				thisSiteMaker = new FileSiteMaker(this.theDocument, thisRepository, thisImageRepository, thisPath, thisHTMLFile, thisXMLFile, this.theTitleTextField.getText());
			}

			if (thisSiteMaker != null)
			{
				// add observers
				thisSiteMaker.addObserver(new Observer()
				{
					@Override
					@SuppressWarnings({ "unchecked", "synthetic-access" })
					public void update(final Observable thisO, final Object thisArg)
					{
						final Pair<String, Integer> thisInfo = (Pair<String, Integer>) thisArg;
						MakeSiteDialog.this.theProgressLabel.setText(thisInfo.theFirst);
						MakeSiteDialog.this.theProgressBar.setValue(thisInfo.theSecond);
					}
				});

				// make site
				final boolean hasSucceeded = thisSiteMaker.make();

				// delete observers
				thisSiteMaker.deleteObservers();

				// result
				if (!hasSucceeded)
				{
					hasFailed = true;
					return;
				}
			}
		}
		catch (final MalformedURLException thisException)
		{
			System.err.println(Messages.getString("MakeSiteDialog.50") + thisException.toString()); //$NON-NLS-1$
			return;
		}
		catch (final URISyntaxException thisException)
		{
			System.err.println("Transfer: " + thisException.toString()); //$NON-NLS-1$
			return;
		}
		catch (final IOException thisException)
		{
			System.err.println("Transfer: " + thisException.toString()); //$NON-NLS-1$
			return;
		}

		// chain to run
		if (!hasFailed & this.theRunCheckBox.isSelected())
		{
			// update data from properties
			this.theProperties.setProperty("page", this.theHTMLFileTextField.getText()); //$NON-NLS-1$
			final RunSiteDialog thisDialog = new RunSiteDialog(this.theProperties);
			thisDialog.setModal(true);
			thisDialog.setVisible(true);
		}

		setVisible(false);
	}

	// I N F O R M

	/**
	 * Inform dialog
	 * 
	 * @param thisMessage
	 *        message
	 */
	protected void inform(final String thisMessage)
	{
		final String[] theseLines = thisMessage.split("\n"); //$NON-NLS-1$
		JOptionPane.showMessageDialog(null, theseLines, Messages.getString("MakeSiteDialog.60"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			// read properties into components
			this.theSiteLabel.setText(getSiteUrlString());
			this.theRepositoryLabel.setText(this.theProperties.getProperty("repository", "<internal>")); //$NON-NLS-1$ //$NON-NLS-2$
			this.theImageRepositoryLabel.setText(this.theProperties.getProperty("images")); //$NON-NLS-1$
			this.theProgressLabel.setText(Messages.getString("MakeSiteDialog.20")); //$NON-NLS-1$
			this.theProgressBar.setValue(0);

			pack();
			Utils.center(this);
		}
		else
		{
			// update properties from components
			this.theProperties.setProperty("page", this.theHTMLFileTextField.getText()); //$NON-NLS-1$
		}
		super.setVisible(thisFlag);
	}

	/**
	 * Main
	 * 
	 * @param theseArgs
	 *        arguments
	 * @throws MalformedURLException
	 */
	static public void main(final String[] theseArgs) throws MalformedURLException
	{
		UIManager.put("swing.boldMetal", false); //$NON-NLS-1$
		final Properties theseSettings = Persist.getSettings("treebolic-generator"); //$NON-NLS-1$
		String thisStringUrl = null;
		if (theseArgs.length >= 1)
		{
			thisStringUrl = theseArgs[0];
		}
		else
		{
			JOptionPane.showMessageDialog(null, new String[] { Messages.getString("MakeSiteDialog.70") }, Messages.getString("MakeSiteDialog.71"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$ //$NON-NLS-2$
			thisStringUrl = FileDialogs.getXmlUrl(theseSettings.getProperty("base", ".")); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (thisStringUrl == null)
		{
			System.exit(0);
		}
		final Document thisDocument = new Parser().makeDocument(new URL(thisStringUrl), null);
		if (thisDocument == null)
		{
			System.exit(0);
		}
		final MakeSiteDialog thisDialog = new MakeSiteDialog(theseSettings);
		thisDialog.setDocument(thisDocument);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		Persist.saveSettings("treebolic-generator", theseSettings); //$NON-NLS-1$
		System.exit(0);
	}
}
