/**
 * Title :	Treebolic generator
 * Description: Treebolic generator
 * Version:	provider
 * Copyright (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 */
package treebolic.site;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import treebolic.Messages;
import treebolic.commons.Persist;
import treebolic.commons.Utils;
import treebolic.dialogs.main.XSiteDialog;

/**
 * Run site dialog
 * 
 * @author Bernard Bou
 */
public class RunSiteDialog extends JDialog
{
	private static final long serialVersionUID = 1L;

	// C O M P O N E N T S

	/**
	 * Home input
	 */
	private JTextField theHomeTextField;

	/**
	 * Command information
	 */
	private JLabel theCommandLabel;

	/**
	 * Properties (input/output)
	 */
	private final Properties theProperties;

	// C O N S T R U C T OR

	/**
	 * Constructor
	 * 
	 * @param theseProperties
	 *        settings
	 */
	public RunSiteDialog(final Properties theseProperties)
	{
		super();
		this.theProperties = theseProperties;
		initialize();
	}

	/**
	 * Initialize
	 */
	private void initialize()
	{
		setTitle(Messages.getString("RunSiteDialog.0")); //$NON-NLS-1$
		setResizable(true);

		// text fields
		this.theHomeTextField = new JTextField(16);

		// images
		final Icon thisIcon = new ImageIcon(MakeSiteDialog.class.getResource("images/siterun.png")); //$NON-NLS-1$
		final JLabel thisImageLabel = new JLabel();
		thisImageLabel.setIcon(thisIcon);
		thisImageLabel.setVerticalTextPosition(SwingConstants.TOP);
		thisImageLabel.setHorizontalTextPosition(SwingConstants.CENTER);
		thisImageLabel.setText(Messages.getString("RunSiteDialog.1")); //$NON-NLS-1$
		thisImageLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

		// labels
		final JLabel thisHtmlFileLabel = new JLabel(Messages.getString("RunSiteDialog.2")); //$NON-NLS-1$

		this.theCommandLabel = new JLabel();
		this.theCommandLabel.setToolTipText(Messages.getString("RunSiteDialog.6")); //$NON-NLS-1$
		this.theCommandLabel.setEnabled(false);
		this.theCommandLabel.setFont(new Font("Dialog", Font.ITALIC, 10)); //$NON-NLS-1$

		// buttons
		final JButton thisRunButton = new JButton(Messages.getString("RunSiteDialog.10")); //$NON-NLS-1$
		final JButton thisCancelButton = new JButton(Messages.getString("RunSiteDialog.11")); //$NON-NLS-1$
		final JButton thisBrowserButton = new JButton(Messages.getString("RunSiteDialog.3")); //$NON-NLS-1$
		final JButton thisModeButton = new JButton(Messages.getString("RunSiteDialog.4")); //$NON-NLS-1$

		// panels
		final JPanel thisDataPanel = new JPanel();
		thisDataPanel.setLayout(new FlowLayout());
		thisDataPanel.add(thisHtmlFileLabel);
		thisDataPanel.add(this.theHomeTextField);

		final JPanel thisCommandPanel = new JPanel();
		thisCommandPanel.setLayout(new FlowLayout());
		thisCommandPanel.add(this.theCommandLabel);

		final JPanel thisButtonPanel = new JPanel();
		thisButtonPanel.setLayout(new FlowLayout());
		thisButtonPanel.add(thisBrowserButton);
		thisButtonPanel.add(thisModeButton);
		thisButtonPanel.add(thisCancelButton);
		thisButtonPanel.add(thisRunButton);

		// event
		thisRunButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				run();
				setVisible(false);
			}
		});

		thisCancelButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				setVisible(false);
			}
		});

		thisBrowserButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final String thisMessage = Messages.getString("RunSiteDialog.5"); //$NON-NLS-1$
				final String[] theseLines = thisMessage.split("\n"); //$NON-NLS-1$
				final String thisBrowser = JOptionPane.showInputDialog(null, theseLines);
				if (thisBrowser != null && !thisBrowser.isEmpty())
				{
					RunSiteDialog.this.theProperties.setProperty("browser", thisBrowser); //$NON-NLS-1$
					RunSiteDialog.this.theCommandLabel.setText(makeCommand());
				}
			}
		});

		thisModeButton.addActionListener(new ActionListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void actionPerformed(final ActionEvent e)
			{
				final XSiteDialog thisDialog = new XSiteDialog(RunSiteDialog.this.theProperties);
				thisDialog.setModal(true);
				thisDialog.setVisible(true);
				if (thisDialog.ok)
				{
					RunSiteDialog.this.theCommandLabel.setText(makeCommand());
				}
			}
		});

		// change handler
		this.theHomeTextField.getDocument().addDocumentListener(new DocumentListener()
		{
			@SuppressWarnings("synthetic-access")
			@Override
			public void removeUpdate(final DocumentEvent e)
			{
				RunSiteDialog.this.theCommandLabel.setText(makeCommand());
			}

			@SuppressWarnings("synthetic-access")
			@Override
			public void insertUpdate(final DocumentEvent e)
			{
				RunSiteDialog.this.theCommandLabel.setText(makeCommand());
			}

			@Override
			public void changedUpdate(final DocumentEvent e)
			{
				// do nothing
			}
		});

		// assemble
		final JPanel thisPanel = new JPanel();
		thisPanel.setLayout(new BoxLayout(thisPanel, BoxLayout.Y_AXIS));
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		thisPanel.add(thisImageLabel);
		thisPanel.add(thisDataPanel);
		thisPanel.add(thisCommandPanel);
		thisPanel.add(thisButtonPanel);
		thisPanel.add(Box.createRigidArea(new Dimension(0, 10)));

		setContentPane(thisPanel);
	}

	// C O M M A N D . H A N D L I N G

	/**
	 * Make command
	 * 
	 * @return command
	 */
	private String makeCommand()
	{
		String thisCommand;
		thisCommand = this.theProperties.getProperty("browser", ""); //$NON-NLS-1$ //$NON-NLS-2$
		if (thisCommand.isEmpty())
			return null;
		thisCommand += " "; //$NON-NLS-1$

		final String thisMode = this.theProperties.getProperty("mode"); //$NON-NLS-1$
		if ("NET".equals(thisMode)) //$NON-NLS-1$
		{
			final String thisServer = this.theProperties.getProperty("server", ""); //$NON-NLS-1$ //$NON-NLS-2$
			final String thisDirectory = this.theProperties.getProperty("directory", ""); //$NON-NLS-1$ //$NON-NLS-2$
			thisCommand += "http://" + thisServer + "/"; //$NON-NLS-1$ //$NON-NLS-2$
			if (!thisDirectory.isEmpty())
			{
				thisCommand += thisDirectory;
				thisCommand += "/"; //$NON-NLS-1$
			}
			thisCommand += this.theHomeTextField.getText();
		}
		else
		{
			String thisPath = this.theProperties.getProperty("path", ""); //$NON-NLS-1$ //$NON-NLS-2$
			if (!thisPath.endsWith(File.separator))
			{
				thisPath += File.separator;
			}
			thisPath += this.theHomeTextField.getText();
			final File thisFolder = new File(thisPath);
			try
			{
				thisPath = thisFolder.toURI().toURL().toString();
			}
			catch (final MalformedURLException thisException)
			{
				// do nothing
			}
			thisCommand += thisPath;
		}
		return thisCommand;
	}

	/**
	 * Run command
	 */
	private void run()
	{
		final String thisCommand = makeCommand();
		if (thisCommand != null && !thisCommand.isEmpty())
		{
			try
			{
				Runtime.getRuntime().exec(thisCommand);
			}
			catch (final Exception e)
			{
				System.err.println("Cannot run " + thisCommand); //$NON-NLS-1$
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	@Override
	public void setVisible(final boolean thisFlag)
	{
		if (thisFlag)
		{
			// read properties into components
			this.theHomeTextField.setText(this.theProperties.getProperty("page")); //$NON-NLS-1$
			this.theCommandLabel.setText(makeCommand());

			pack();
			Utils.center(this);
		}
		else
		{
			// update properties from components
			this.theProperties.setProperty("page", this.theHomeTextField.getText()); //$NON-NLS-1$
		}

		super.setVisible(thisFlag);
	}

	/**
	 * Main
	 * 
	 * @param theseArgs
	 *        arguments
	 */
	static public void main(final String[] theseArgs)
	{
		UIManager.put("swing.boldMetal", false); //$NON-NLS-1$
		final Properties theseSettings = Persist.getSettings("treebolic-generator"); //$NON-NLS-1$
		final RunSiteDialog thisDialog = new RunSiteDialog(theseSettings);
		thisDialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		thisDialog.setModal(true);
		thisDialog.setVisible(true);
		Persist.saveSettings("treebolic-generator", theseSettings); //$NON-NLS-1$
		System.exit(0);
	}
}
