/**
 * Title: Hyperbolic engine
 * Description: Treebolic Engine
 * Version: provider
 * Copyright: (c) 2001-2008
 * Terms of use:see license agreement at http://treebolic.sourceforge.net/en/license.htm
 * Author: Bernard Bou
 * Company: bsys
 * Update: Mon Mar 10 00:00:00 CEST 2008
 */
package treebolic.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;

import javax.swing.Timer;

import treebolic.core.HyperTransform;

/**
 * Animator implements animation of tree
 * 
 * @author Bernard Bou
 */
public class Animator extends Timer
{
	private static final long serialVersionUID = 1L;

	class Job implements ActionListener
	{
		/**
		 * Animation iterator
		 */
		private Iterator<HyperTransform> theAnimationIterator;

		public Job(final Animation thisAnimation)
		{
			this.theAnimationIterator = thisAnimation.theTransforms.iterator();
		}

		/*
		 * (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@SuppressWarnings("synthetic-access")
		@Override
		public void actionPerformed(final ActionEvent thisE)
		{
			if (this.theAnimationIterator == null || !this.theAnimationIterator.hasNext())
			{
				this.theAnimationIterator = null;
				stop();
				removeActionListener(this);
			}
			else
			{
				final HyperTransform thisTransform = this.theAnimationIterator.next();
				Animator.this.theView.applyTransform(thisTransform);
				Animator.this.theView.repaint();
			}
		}
	}

	/**
	 * View
	 */
	private final View theView;

	/**
	 * Start delay
	 */
	private final int theStartDelay;

	/**
	 * Constructor
	 * 
	 * @param thisView
	 *        view
	 * @param thisTimerLapse
	 *        timer lapse
	 * @param thisStartDelay
	 *        start delay
	 */
	public Animator(final View thisView, final int thisTimerLapse, final int thisStartDelay)
	{
		super(thisTimerLapse, null);
		this.theView = thisView;
		this.theStartDelay = thisStartDelay;
		setRepeats(true);
		setCoalesce(true);
	}

	/**
	 * Run animation
	 * 
	 * @param thisAnimation
	 *        animation
	 * @param now
	 *        whether to start now
	 * @return true if successful
	 */
	public boolean run(final Animation thisAnimation, final boolean now)
	{
		if (thisAnimation.theTransforms == null)
			return false;

		addActionListener(new Job(thisAnimation));

		setInitialDelay(now ? 0 : this.theStartDelay);
		start();
		return true;
	}
}
